/*
 * SISR.cpp
 *
 *  Created on: 25 juil. 2013
 *      Author: ppudlo
 */

#include "genetIS.h"
#include <fstream>
#include <iostream>
#include <algorithm> //CM 21.02.14
#include <string>

using namespace std;

double ESS(const vector<double> & w){
  double sumsquares = 0.;
  double sum = 0.;
  for(unsigned k = 0; k < w.size(); ++k) {
    sumsquares += w[k]*w[k];
    sum += w[k];
  }
  return sum * sum / sumsquares;
}

double max(const vector<double> & w) {
	double a = w[0];
	for(vector<double>::const_iterator it = w.begin(); it != w.end(); ++it) {
		if (*it > a) a = *it;
	}
	return a;
}

double ESS(const vector<SIS> & tab){
  double sumsquares = 0.;
  double sum = 0.;
  for(unsigned k = 0; k < tab.size(); ++k) {
    const double x = tab[k].weight();
    sumsquares += x*x;
    sum += x;
  }
  return sum * sum / sumsquares;
}

double mean(const vector<SIS> & tab){
	double sum = 0.;
	for(unsigned k = 0; k < tab.size(); ++k) {
		sum += tab[k].weight();
	}
	return sum / (double)tab.size();
}

bool plus_grand(SIS const & a, SIS const & b){
  return a.anything() > b.anything();
}

void resample(vector<SIS> & mesarbres, vector<double> & other,
		const double alpha=.5, const double beta =0.) {
    size_t nb_arbres = mesarbres.size();

	// calcul des proba de retirage
    vector<double> a(mesarbres.size(), 0.);
    for(unsigned k = 0; k < nb_arbres; ++k) {
      a[k] = pow(mesarbres[k].weight(), alpha); // a[k] indispensable ou direct dans anything ok ? CM 07.01.15
      a[k] *= exp(beta * other[k]);
      //a[k] *= exp( other[k]* beta * 2.0/mesarbres[k].getsum_n()*(mesarbres[k].getsum_n()-1)); //CM 11.02.15
      /*if (exp(beta * other[k])==0) {
        cout << "exp(beta * other["<<k<<"]) = " << exp(beta * other[k]) << " other["<<k<<"] = "<< other[k] << " beta = " <<beta << endl;
      }*/
      mesarbres[k].anything() = a[k];
    }
  std::sort(mesarbres.begin(), mesarbres.end(), plus_grand);
  for (unsigned k =0; k < nb_arbres; ++k) {
    a[k] = mesarbres[k].anything();
  }
    double sum_a = accumulate(a.begin(), a.end(), 0.0); // pas petit a petit car besoin normalisation CM 07.01.15
    vector<double> proba_a(a);
    for(unsigned k = 0; k < nb_arbres; ++k) {
      a[k] *= (double)nb_arbres / sum_a;
      proba_a[k] /= sum_a;
    }
    
    // retirage
  //ofstream TreeId("TreeId.txt", ios::app); // CM 13.03.14
    vector<SIS> mesarbres3 = mesarbres; //mesarbres3 necessaire ? couteux ?// CM 07.01.15
  
    for(unsigned k = 0; k < nb_arbres; ++k){
      int alea = discreterand_proba(proba_a);
      //TreeId << "old treeID[" << k << "] = " << mesarbres3[k].gettreeId();// CM 12.03.14
      mesarbres3[k] = mesarbres[alea];
      //TreeId << " new treeID[" << k << "] = " << mesarbres3[k].gettreeId() << endl;// CM 12.03.14
      mesarbres3[k].weight() *= 1.0 / a[alea];
    }
    //TreeId.close(); // CM 13.03.14
    swap(mesarbres, mesarbres3);
}


void resamplepartial(vector<SIS> & mesarbres, vector<double> other,
              const double alpha=.5, const double beta =0.) {
    size_t nb_arbres = mesarbres.size();
    double u=randunif();
	// calcul des proba de retirage petit à petit
    vector<double> a(mesarbres.size(), 0.);
    double sum_a =0.0;
    int alea=0;
    //retirage
    vector<SIS> mesarbres3 = mesarbres;
    for(unsigned k = 0; k < nb_arbres; ++k){
    while (sum_a<u) {
        a[alea] = pow(mesarbres[k].weight(), alpha);
        a[alea] *= exp(beta * other[k]);
        
        sum_a +=a[alea];
        alea++;
    }
    alea--;
    a[alea] *= (double)nb_arbres / sum_a;// Renormalisation nécessite la somme des nouveau poids
    mesarbres3[k] = mesarbres[alea];
    mesarbres3[k].weight() *= 1.0 / a[alea];
    mesarbres3[k].anything() *= 1.0 / a[alea];
    }

    /*double sum_a = accumulate(a.begin(), a.end(), 0.0);
    for(unsigned k = 0; k < nb_arbres; ++k)
        a[k] *= (double)nb_arbres / sum_a;
     */
    
    // retirage
    /*vector<SIS> mesarbres3 = mesarbres;
    for(unsigned k = 0; k < nb_arbres; ++k){
        int alea = discreterand(a);
        mesarbres3[k] = mesarbres[alea];
        mesarbres3[k].weight() *= 1.0 / a[alea];
        mesarbres3[k].anything() *= 1.0 / a[alea];
    }*/
    
    swap(mesarbres, mesarbres3);
}


void partial_deterministic_reallocation(vector<SIS> & mesarbres,
		vector<double> other, const double alpha=.5, const double beta = 0.) {
	size_t nb_arbres = mesarbres.size();

	// calcul des proba de retirage
	vector<double> a(mesarbres.size(), 0.);
	for(unsigned k = 0; k < nb_arbres; ++k) {
		a[k] = pow(mesarbres[k].weight(), alpha);
		a[k] *= exp(beta * other[k]);
	}
	double sum_a = accumulate(a.begin(), a.end(), 0.0);
	for(unsigned k = 0; k < nb_arbres; ++k)
		a[k] *= (double)nb_arbres / sum_a;

	vector<SIS> mesarbres2;
	mesarbres2.reserve(nb_arbres);
	for(unsigned j = 0; j < nb_arbres; ++j){
		if (a[j] >= 1.0) {
			int k = (int)floor(a[j]);
			mesarbres[j].weight() *= 1.0 / (double)k;
			mesarbres[j].anything() *= 1.0 / (double)k;
			for(int i = 0; i < k; ++i)
				mesarbres2.push_back(mesarbres[j]);
		} else { // here a[j] <1
			double u = randunif();
			mesarbres[j].weight() *= 1.0 / a[j];
			mesarbres[j].anything() *= 1.0 / a[j];
			if (u <= a[j])
				mesarbres2.push_back(mesarbres[j]);
		}
	}
	swap(mesarbres, mesarbres2);
}

void residual_resampling(vector<SIS> & mesarbres, vector<double> other, const double alpha = .5,
		const double beta = 0.) {
	size_t nb_arbres = mesarbres.size();

	// calcul des proba de retirage
	vector<double> a(mesarbres.size(), 0.);
	for(unsigned k = 0; k < nb_arbres; ++k) {
		a[k] = pow(mesarbres[k].weight(), alpha);
		a[k] *= exp(beta * other[k]);
	}
	double sum_a = accumulate(a.begin(), a.end(), 0.0);
	for(unsigned k = 0; k < nb_arbres; ++k)
		a[k] *= (double)nb_arbres / sum_a;

	vector<SIS> mesarbres2;
	mesarbres2.reserve(nb_arbres);
	// mise à jour des poids
	for(unsigned j = 0; j < nb_arbres; ++j) {
		mesarbres[j].weight() *= 1.0 / a[j];
		mesarbres[j].anything() *= 1.0 / a[j];
	}
	// les copies déterministes
	for(unsigned j = 0; j < nb_arbres; ++j){
		if (a[j] >= 1.0) {
			int k = (int)floor(a[j]);
			for(int i = 0; i < k; ++i)
				mesarbres2.push_back(mesarbres[j]);
		}
	}
	// les copies aléatoires pour combler
	size_t manque = nb_arbres - mesarbres2.size();
	cout << "#### manque = " << manque << endl;
	for(unsigned j = 0; j < manque; ++j) {
		int k = discreterand(a);
		mesarbres2.push_back(mesarbres[k]);
	}
	swap(mesarbres, mesarbres2);
}


typedef enum {random_rr, realloc_rr, residual_rr} resampling_t;
resampling_t strategie;

string path;

long double Calclogl_PCL_SISR(vector < vector< int > > SampleEffectif,int Sam_AlleleNbr,float thetaact, float thetaanc, float T,
                              size_t nb_arbres, bool UseIndivNbResampling,int IndivNbResampling, int MaxEventNbInPopAnc, int ESSMinFall, int stop, float alpha_resampling, float beta_resampling, int NumJobMin) {//CM 13.02.14  //CM 25.03.14
    long double lik,logMeanLik;
  bool TrajectoryPrint = false;
  bool TreeIdsPrint = false;
  bool ESSPrint = false;
  bool BoolTechniquePaquets=false;
  int RetardDebutResampling=0;
    
    strategie = random_rr;

    vector<int> n=SampleEffectif[0];
  
    string fileTreeIdsname;//CM 13.03.14
    {std::stringstream stst(std::stringstream::in | std::stringstream::out);//CM 13.03.14
      stst<<"TreeIds_PointNumber" << "_LocusNumber"<<"_JobNumber"<< NumJobMin<< ".txt";//CM 13.03.14
      stst>>fileTreeIdsname;//CM 13.03.14
    } //CM 13.03.14
  ofstream TreeId(fileTreeIdsname.c_str(), ios::app); //CM 13.03.14
  

  string fileWeightTrajectoryname;//CM 26.03.14
  {std::stringstream stst(std::stringstream::in | std::stringstream::out);//CM 26.03.14
    stst<<"WeightTrajectoryPointNumber_LocusNumber"<<"_JobNumber"<< NumJobMin<< ".txt";//CM 04.04.14 Un seul fichier (si on ne veut pas utiliser ce fichier mais on doit quand meme créer le fichier pour que ça compile donc on en crée un seul)
    stst>>fileWeightTrajectoryname;//CM 26.03.14
  } //CM 26.03.14
  ofstream WeightTrajectory(fileWeightTrajectoryname.c_str(), ios::app); //CM 26.03.14
  
  /*string fileWeightTrajectoryname;//CM 09.02.15
  {std::stringstream stst(std::stringstream::in | std::stringstream::out);//CM 09.02.15
    stst<<"WeightTrajectoryPointNumber"<< args.point<< "_LocusNumber"<< args.locus<< "_JobNumber"<< NumJobMin<< ".txt";//CM 04.04.14 Plusieurs fichiers
    stst>>fileWeightTrajectoryname;//CM 09.02.15
  } //CM 09.02.15
  ofstream WeightTrajectory(fileWeightTrajectoryname.c_str(), ios::app); //CM 09.02.15*/


  
  string fileESS;//CM 23.04.14
  {std::stringstream stst(std::stringstream::in | std::stringstream::out);//CM 23.04.14
    stst<<"ESS_PointNumber" << "_LocusNumber"<<"_JobNumber"<< NumJobMin<< ".txt";//CM 23.04.14
    stst>>fileESS;//CM 23.04.14
  } //CM 23.04.14
  ofstream ESSfile(fileESS.c_str(), ios::app); //CM 23.04.14

  if (ESSPrint) {//CM 23.04.14
    ESSfile << args.point << " " << args.locus<< " " << thetaact << " " << T << " " << thetaanc << " " ;//CM 23.04.14
  }//CM 23.04.14
  

//    cout << " ########### Hello ! ###########" << endl;
//    cout << "SIS with resampling when ESS is falling." << endl;
//    cout << "Computing ESS each " << stop << " iterations." << endl;
//    cout << " ###############################" << endl << endl;


	  vector<vector<double> > w_t; // A quoi ça sert ? CM 18.03.14
	  w_t.reserve(2000); // A quoi ça sert ? CM 18.03.14

  /***** basic SIS ************/


	  SIS monobjet (n, thetaact, thetaanc, T);
    CalculPCL monPCL(thetaanc, 500);

	  vector<SIS> mesarbres(nb_arbres, monobjet);

	  // début SIS basique
	  if (TreeIdsPrint){
    TreeId <<args.point << " " << args.locus<< " " ;
    }
	  for(size_t k = 0; k < nb_arbres; ++k) {
      if (TrajectoryPrint){
      WeightTrajectory << k << " "; //CM 27.03.14
      }
      int IndivNbResamplingInk= IndivNbResampling;
      if (!UseIndivNbResampling) {
        // va jusqu'à l'itération stop
        mesarbres[k].algo(stop+RetardDebutResampling);
      }else{
        // va jusqu'à avoir IndivNumber individu dans l'échantillon
        if (TrajectoryPrint) {//Seulement pour pouvoir tracer les trajectoires en stockant les poids à chaque fois que l'on enleve un indiv mais sans ré-échantillonner //CM 27.03.14
          while (mesarbres[k].getsum_n()>=2) {//Seulement pour pouvoir tracer les trajectoires en stockant les poids à chaque fois que l'on enleve un indiv mais sans ré-échantillonner, mettre IndivNbresampling=1 //CM 27.03.14
            mesarbres[k].algo_IndivNb(IndivNbResamplingInk);//CM 27.03.14
            if (TreeIdsPrint){
            TreeId  <<mesarbres[k].gettreeId()<< " ";// CM 12.03.14
            }
            
            if (TrajectoryPrint){
            WeightTrajectory << mesarbres[k].weight()<< " ";//CM 26.03.14
            }
          }// endwhile sum_n >= 2  //CM 27.03.14
        }else{
          mesarbres[k].algo_IndivNb(IndivNbResampling+RetardDebutResampling);

        }
        
      }
	  if (TreeIdsPrint){  
      mesarbres[k].settreeId((int)k); // CM 12.03.14
      TreeId  <<mesarbres[k].gettreeId()<< " ";// CM 12.03.14
      }
      
      if (TrajectoryPrint){
      WeightTrajectory << endl; //CM 27.03.14
      }
	  } //end boucle for k
  
  if (TrajectoryPrint){
  }
  if (TreeIdsPrint){
  TreeId << endl;// CM 12.03.14
  }
  
  if (stop>=0) {
    //cout << " ########### Hello ! ###########" << endl;//CM 23.04.14
    //cout << "SIS with resampling when ESS is falling."<< " stop = " << stop << endl;//CM 23.04.14
  
	  /********** SIS with resampling **************/
    
      //cout << "SIS with resampling each " << stop << " iterations." << endl;
	  bool fini = false;
	  double old_ESS = nb_arbres;
	  double current_ESS;
    bool TwoIndiveachTree= true;
	  do{
	    current_ESS = ESS(mesarbres);
      for(size_t k = 0; k < nb_arbres; ++k) {
        TwoIndiveachTree=(mesarbres[k].getsum_n()>=2)&&(TwoIndiveachTree);
      }
      
	    if((current_ESS <= old_ESS/(double)ESSMinFall) && (TwoIndiveachTree)){//CM 13.02.14 CM 18.02.14 conversion double
	      // On calcule les PCL et on retranche le maximum
        // à ne calculer que si on re-échantillonne // CM .01.15
	      vector<double> lPCL (mesarbres.size(), 1.);
	      for(unsigned k = 0; k < mesarbres.size(); ++k) {
	    	  lPCL[k] = monPCL.logPCL(mesarbres[k].current_sample()); // Pour theta_anc et pas theta_act ? CM 07.10.14
	      }
        double maxlogPCL=max(lPCL);
        for(unsigned k = 0; k < mesarbres.size(); ++k) {
          lPCL[k] = lPCL[k]-maxlogPCL; // CM 11.02.15
        }
  
        int ResamplingGroupNumber = 1 ;// un diviseur de NRunsPerPoint pour commencer
        int RunPerResamplingGroupNumber = 100 /ResamplingGroupNumber;
        
        SIS monobjetparpaquet (n, thetaact, thetaanc, T); // Un objet SIS : un arbre avec tous ses attribus
        vector<SIS> mesarbresunpaquet(RunPerResamplingGroupNumber, monobjetparpaquet);
        // un vecteur d'abres(avec leurs attribus), autant d'abres que le nombre d'abres par paquets
        vector<vector<SIS> > mesarbrespaquets(ResamplingGroupNumber, mesarbresunpaquet);
        // une matrice dont chaque ligne représente un paquet de ré-échantillonnage
        vector<double> lPCLunpaquet(RunPerResamplingGroupNumber, 1.);
        vector<vector<double> > lPCLparpaquets(ResamplingGroupNumber,lPCLunpaquet);
        
        
        
        if(BoolTechniquePaquets){
        /*SIS monobjetparpaquet (n, thetaact, thetaanc, T); // Un objet SIS : un arbre avec tous ses attribus
        vector<SIS> mesarbresunpaquet(RunPerResamplingGroupNumber, monobjetparpaquet);// un vecteur d'abres(avec leurs attribus), autant d'arbres que le nombre d'abres par paquets
        vector<vector<SIS> > mesarbrespaquets(ResamplingGroupNumber, mesarbresunpaquet);// une matrice dont chaque ligne représente un paquet de ré-échantillonnage
        vector<double> lPCLunpaquet(RunPerResamplingGroupNumber, 1.);
        vector<vector<double> > lPCLparpaquets(ResamplingGroupNumber,lPCLunpaquet);*/
        int compteurmesarbrespaquets=0;
        for (unsigned i=0; i<ResamplingGroupNumber; ++i) {
          for (unsigned j = 0; j<RunPerResamplingGroupNumber; ++j) {
            mesarbrespaquets[i][j] = mesarbres[compteurmesarbrespaquets];
            lPCLparpaquets[i][j] = lPCL[compteurmesarbrespaquets];
            compteurmesarbrespaquets++;
          }
        }
        //BoolTechniquePaquets=!BoolTechniquePaquets //pour changer de technique à chaque fois
        // Fin de la fabrication des paquets de ré-échantillonnage CM 18.03.14
        
        // Deuxième façon de faire les paquets (pas contigu)
        
        /*if(!BoolTechniquePaquets)
         
        BoolTechniquePaquets=!BoolTechniquePaquets //pour changer de technique à chaque fois */
        
        // Fin de la deuxième façon de faire les paquets (pas contigu)
        
        // Code pour ré-échantillonner par paquets CM 18.03.14
        // D'abord seulement pour la procédure de ré-échantillonnage resample
        //if (strategie == random_rr){
        //  for (unsigned i=0; i<ResamplingGroupNumber; ++i) {
        //    resample(mesarbrespaquets[i], lPCLparpaquets[i], alpha_resampling, beta_resampling);
        //  }
        //}
      
        // Fin du code pour ré-échantillonner par paquets CM 18.03.14
        
        // Examen de la composition des différents paquets en termes de diversité des TreeId
        //TreeId <<args.point << " " << args.locus << " ";  // CM 17.03.14
        //for(unsigned k = 0; k < mesarbres.size(); ++k) {
          //// cout << " encore plus loin   treeID[" << k << "] = " << mesarbres[k].gettreeId() << endl;// CM 12.03.14
          //TreeId << mesarbres[k].gettreeId()<< " ";// CM 12.03.14
        //}
        
        // Fin d'examen de la composition des différents paquets en termes de diversité des TreeId
        } // Fin if BoolTechniquePaquets
        

	      if (strategie == residual_rr)
	    	  residual_resampling(mesarbres, lPCL, alpha_resampling, beta_resampling);
	      else if (strategie == random_rr){
          if (BoolTechniquePaquets) {
            for (unsigned i=0; i<ResamplingGroupNumber; ++i) { // CM 18.03.14
              resample(mesarbrespaquets[i], lPCLparpaquets[i], alpha_resampling, beta_resampling); // CM 18.03.14
            } // CM 18.03.14
            int compteurmesarbrespaquets=0;// CM 18.03.14
            for (unsigned i=0; i<ResamplingGroupNumber; ++i) {// CM 18.03.14
              for (unsigned j = 0; j<RunPerResamplingGroupNumber; ++j) {// CM 18.03.14
                mesarbres[compteurmesarbrespaquets] =mesarbrespaquets[i][j];// CM 18.03.14
                // Est-ce suffisant pour que chaque arbre emporte son poids, Id ... ?
                compteurmesarbrespaquets++;// CM 18.03.14
              }// CM 18.03.14
            }// CM 18.03.14
          }else{
            resample(mesarbres, lPCL, alpha_resampling, beta_resampling);  // CM 18.03.14
          }
          

          
          if (TreeIdsPrint){
          TreeId <<args.point << " " << args.locus << " ";  // CM 17.03.14
          }
          for(unsigned k = 0; k < mesarbres.size(); ++k) {
           if (TreeIdsPrint){
            TreeId << mesarbres[k].gettreeId()<< " ";// CM 12.03.14
            }
            
            if (TrajectoryPrint){
            ofstream WeightTrajectory(fileWeightTrajectoryname.c_str(), ios::app); //CM 26.03.14
            WeightTrajectory << mesarbres[k].weight() << " ";//CM 26.03.14
            WeightTrajectory.close();// CM 26.03.14
            }
            
          }
          if (TreeIdsPrint){
          TreeId << endl;
          }
          if (TrajectoryPrint){
          WeightTrajectory << endl; //CM 26.03.14
          }
        }
          else if (strategie == realloc_rr) // Instruction entre crochet ? CM 01.10.15
	    	  partial_deterministic_reallocation(mesarbres, lPCL, alpha_resampling, beta_resampling);
	      old_ESS = ESS(mesarbres);
	      nb_arbres = mesarbres.size();
	    } // end if current_ESS <= nb_arbres/10

	    nb_arbres = mesarbres.size();
      
	    // Quelques pas de plus
	    if (TreeIdsPrint){
      TreeId <<args.point << " " << args.locus << " ";  // CM 26.03.14
      }
	    for(unsigned k = 0; k < mesarbres.size(); ++k){
        if (!UseIndivNbResampling) {
          // va jusqu'à l'itération stop
          mesarbres[k].algo(stop);
        }else{
          mesarbres[k].algo_IndivNb(IndivNbResampling);
        }
        if (TreeIdsPrint){
        TreeId << mesarbres[k].gettreeId()<< " ";// CM 26.03.14
        }
        if (TrajectoryPrint){
        ofstream WeightTrajectory(fileWeightTrajectoryname.c_str(), ios::app); //CM 26.03.14
        WeightTrajectory << mesarbres[k].weight() << " ";//CM 26.03.14
        WeightTrajectory.close();// CM 26.03.14
        }
      }
      if (TreeIdsPrint){
      TreeId << endl; //CM 26.03.14
      }
      if (TrajectoryPrint){
      WeightTrajectory << endl; //CM 26.03.14
      }
      
	      //mesarbres[k].algo(stop);
      
	    // Les arbres sont-ils tous finis ?
	    fini = true;
	    unsigned nb = 0;
      int minNbEventInPopAnc=mesarbres[0].NbEventInPopAnc();// CM 21.02.14
      int maxNbEventInPopAnc=0;// CM 21.02.14
	    for(unsigned k = 0; k < mesarbres.size(); ++k){
	      fini = fini and mesarbres[k].finished();
	      if (mesarbres[k].finished()) ++nb;
          minNbEventInPopAnc=min(minNbEventInPopAnc,mesarbres[k].NbEventInPopAnc());// CM 21.02.14
          maxNbEventInPopAnc=max(maxNbEventInPopAnc, mesarbres[k].NbEventInPopAnc());// CM 21.02.14
	    }
       if ((nb >= (100./100.0)*nb_arbres ) || ((minNbEventInPopAnc > MaxEventNbInPopAnc) || (maxNbEventInPopAnc > MaxEventNbInPopAnc + 5 ) ) ) { //peu d'arbres ou bien on est en pop ancestrale depuis quelque temps // CM 21.02.14 CM 07.03.14
         for (unsigned k = 0; k < nb_arbres; ++k){
           if (!UseIndivNbResampling) {
             // va jusqu'à l'itération stop
             mesarbres[k].algo(-1);
           }else{
             // va jusqu'à avoir IndivNumber individu dans l'échantillon
             mesarbres[k].algo_IndivNb(-1);
           }
         }
	        //mesarbres[k].algo(-1);
	      fini = true;
	    }  //CM 13.02.14
	  } while (not fini);
    
  } // end if (stop>=0)
 
  if (ESSPrint) {//CM 23.04.14
    ESSfile << ESS(mesarbres) << " " << nb_arbres << " "<< (double)ESS(mesarbres)/(double)nb_arbres<< " "<< UseIndivNbResampling<< " "<< ESSMinFall << " "<< stop << " "<< alpha_resampling << " "<< beta_resampling << " " << IndivNbResampling<< " " <<  RetardDebutResampling<< endl ;//CM 23.04.14
  }//CM 23.04.14
  

    //repasser de LogL en L
    
    //moyenne L sur tous les arbres
    lik=0;
    for(size_t k = 0; k < nb_arbres; ++k) {
        lik+= mesarbres[k].weight();
    }
    lik/=nb_arbres;
  
  logMeanLik=log(lik);
  TreeId.close();// CM 13.03.14
  ESSfile.close(); //CM 23.04.14
    return(logMeanLik);
}
