/***************************************************************************
© F. Rousset, R. Leblois 2007

francois.rousset@univ-montp2.fr
raphael.leblois@mnhn.fr

This file is part of Migraine. This software is a computer program
whose purpose is to perform statistical analyses.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 ***************************************************************************/
#ifndef H_DGCSAMPLE
#define H_DGCSAMPLE
#include <iomanip>
#include <string>
#include <cstdlib>
#include <fstream>
#include <cstdio>
#include <iostream>
#include <fstream>

#include "locus.h" // CProcessedLocus
#include "dg.h"
#include "dgCMigCMut.h"
#include "computeId.h"


//class echantillon
class CSample{
	//	CBR (10.05.2013) started a private section (just in case) when circumventing CSample::Change
private:
	enum DIGEventType {noDIGEvent, migPlusISMtype1, migPlusISMtype2, migOnly};	// enumeration of ISM events for a chosen haplotype (De Iorio & Griffiths 2004b)
	enum HOBEventType {noHOBEvent, mutISMtype1, mutISMtype2, coalescence};	// enumeration of ISM events for a chosen haplotype (Hobolth et al 2008)

public:
	int PopSampledOrNot;
	int Sam_AlleleNbr;
	int Size;//taille de l'echantillon de depart en nbre de genes
	int SampleStateRange;//ra,nge de taille es alleles de l'echant. = taille max de l'echant.
	int GeoSpan; //max geo dist between subsamples
	picalctype proposalCorrectionFactor; //used to correct the proposal probabilties in case of FRmixed...AlternativeIS, 19012011
	bool PIjabool;//for alternative FR mixed PI's, indicate if the current PI computation coresponds to the PIja computation or not, 25012011
	//(uniquement pour GetSample en SMM...)
	std::vector <int> AllelicRange;//range des alleles dans sample pour SMM, permet de gagner beaucoup de temps.
	long generation; // the time in the past; set to 0 by GetSample();
	scaledTimeType scaled_generation; // the scaled (2N_T) time in the past; set to 0 by GetSample();
	scaledTimeType scaled_generationCoa;
	scaledTimeType scaled_generationOthers;
	int events; //counts number of dispersal or coalescence events
	int locusNum;	//	keeps track of the current locus being analysed
	long int runNum;	//	keeps track of the current run being simulated
#ifdef ASYMPTO_PAR_DISP_EVENTS
	int disp_events; //counts number of dispersal events
#endif
	std::vector<int> alleleIndices; //FR added 10/01/08
	std::vector< std::vector<int> > Effectif;
	std::vector<int> PopEffectif;
	std::vector<int> StateEffectif;	// CBR (25.04.2012) mainly used to check for migration events among singleton genes in CSample::Change
	std::vector<int> siteMutCountVec;	// CBR (05.03.2012) the number of HAPLOTYPES (unique sequences) containing a given mutation at a nucleotide position
	std::vector<int> DmVec;	// CBR (03.06.2013) the number of SEQUENCES containing a given mutation at a nucleotide position
	std::vector<std::vector<int> > mutPathMatrix;	// CBR (05.03.2012) contains the tree version (mutation paths to the MRCA) of the haplotype matrix
	std::vector<int> nZero;	// CBR (26.04.2012) equal to the PopEffectif of the chosen pop but depends on the Mab matrix of that pop and select singleton genes
//	valarray<valarray<picalctype> >L_;
//==== variable for Pi calculation
	int PiCalcPimDG2002(const CMigMod *Mig,const CMutMod *Mut, const int& Type);
	int PiCalcTwoPopDG2002(const CMigMod *Mig,const CMutMod *Mut,const int& Type);
	int PiCalcTwoPopSmmDG2002(const CMigMod *Mig,const CMutMod *Mut,const int &Type);
	int PiCalcOnePop(const CMigMod *Mig,const CMutMod *Mut,const int& Type);
	int PiCalcISM(const CMigMod *Mig,const CMutMod *Mut,const int &Type);
	void setNZeroForPiCalcISM(const CMigMod *Mig, const int &Pop, const int &Type);
//#ifdef LDLsymTRUE
	int PicalcPIMLDLsym(const CMigMod *Mig,const CMutMod *Mut, const int& Type);
//#endif
//this needed to be declared in the abstract class. I need to learn more...
	picalctype *D_A; //Diagonal de la matrice A
	picalctype *D_M; //Diagonale dans M=LDL'
	picalctype *ligne;
	picalctype *gamma;
	picalctype *delta;
	picalctype *Golubx;
	picalctype *Golubz;
	picalctype *Golubr;
	picalctype *Golubp;
	picalctype *GolubAp;
	int *haut;
	int *bas;
	int PrintPi(const CMigMod *Mig,const CMutMod *Mut);
#ifdef largeSPD_TRUE
	int large_SPD(const CMigMod *Mig,const CMutMod *Mut, const int& Type);
#endif
	//	CSample(string FileName,int* j,int* k); //FER: nv constructeur remplace GetSample
//	CSample(int* StateNum,int* PopSorN);
//	CSample(CMutMod* Mut);//constructeur
	CSample(CSample* Sample);//constructeur
	CSample();//constructeur
	virtual ~CSample() {};//destructeur should be virtual (Primer+ p. 665)
    changetype computeIIS(CcomputeId* loccalculQ,CMutMod* MutMod);
	changetype Change(CMigMod* Mig,CMutMod* Mut);
	changetype calcHobolthChange(CMigMod *Mig,CMutMod *Mut); // CBR (10.05.2012) alternate procedure for calculating the proposal Q(k|S,n) under ISM (Hobolth et al., (2008), Eq. 10)
	changetype calcHobolth_pThetaOfDm(const parstype &theta, const int &d); // CBR (10.05.2013) calculates p_theta(d) for each segregating site given in Hobolth et al., (2008) (Eq. 11)
	changetype PACpi(CMigMod* Mig,CMutMod* Mut);
	double LnPr_sample_from_coal_KAM(CMigMod *Mig,CMutMod *Mut); //proba sample sous n-coalescent
	double LnPr_sample_from_island_KAM(CMigMod *Mig,CMutMod *Mut);
	double LnMultinomialCoeff();	//coeff multinomial pour l'['e]chnntillon
	double LogHypergeom(); //log hypergeometric issu de cornuet 042007
    double LnHypergeom2tables(CProcessedLocus *PLptr);
//	double MultinomialCoeff();	//coeff multinomial should become obsolete
	int GetSample(std::string FileName,const int locAlleleNbr);
	int GetSample(CProcessedLocus* it,int locAlleleNbr,bool& nomotifChecked);
    //AddSample could also use pointer rather than vector iterator...
    int AddSample(std::vector<CProcessedLocus>::iterator PLptr); // add counts to an existing CSample
	virtual int new_L_etc()=0; 	//pure virtual, see Primer+ p. 673 et http://gcc.gnu.org/faq.html#vtables
	virtual int delete_L_etc()=0;
// virtual needed as there is a pointer to this member fn of CSample
	virtual int lin_abs_sparsified_1D_penta(const CMigMod *Mig,const CMutMod *Mut, const int& Type)=0;
	virtual int sparsified(const CMigMod *Mig,const CMutMod *Mut, const int& Type)=0;
//	int old_lin_abs_sparsified(const CMigMod *Mig,const CMutMod *Mut, const int& Type);
//	int cierges(const CMigMod *Mig,const CMutMod *Mut, const int& Type);
	int GetAllelicRange(int flag);
	int Print();
	void PrintSeq();
	changetype ChoosePopType(int* p_a, int* p_i,CMigMod* Mig,CMutMod* Mut,bool firstPassbool);//choisi une pop, un type proport. a leurs occurences dans l'echantillon
	scaledTimeType TimeCompute(CMigMod* Mig,CMutMod* Mut,bool firstPassbool); //	CBR (13.05.2013) keeping ONLY the time computation part of CSample::ChoosePopType for IM and varsize models
	int ChoosePopType(int* p_a, int* p_i,
			std::vector<std::vector<int> >::iterator &ait,
			std::vector<int>::iterator &PopEffectif_it,
			std::vector<int>::iterator &iit,
			std::vector<pascumultype>::iterator &Mig_Popsize_it,
			std::vector<pascumultype>::iterator &Mig_Ma_it,
			CMigMod* Mig,CMutMod* Mut);
    int setMutMod(CMutMod* MutMod1,CMigMod* MigMod1);
	int MergePop(int Sink, int Source);//ajuste l'['e]chantillon apres un fusion de pops
	//friend scaledTimeType ExpIntensityCDF(cumultype random,const int SampleSize,scaledTimeType currentTime,scaledTimeType previousTime,const CMigMod* Mig,const CMutMod* Mut);
	//friend scaledTimeType ExpInverseIntensityCDF(cumultype random,const int SampleSize,scaledTimeType currentTime,scaledTimeType previousTime,const CMigMod* Mig,const CMutMod* Mut);
    int CalcMatSD(const CMutMod *Mut,const CMigMod* Mig,const parstype _ThetaDIG,bool AllocateMemory);
    int setMigMod(CMigMod* MigMod1);
    int DemographicChange(CMigMod* MigMod1,CMutMod* MutMod1);
    int EssaiSequentialSample();
    int checkOptions();
};


class CSampleWout : public CSample {
public :
	CSampleWout() : CSample() {};
	CSampleWout(CSample* Sample) : CSample(Sample) {};
    ~CSampleWout() {};
	int new_L_etc();
	int delete_L_etc();
	// il faut une declaration+def de toutes les fonctions virtual dnas la classe parente...
	int lin_abs_sparsified_1D_penta(const CMigMod *Mig,const CMutMod *Mut, const int& Type);
	int sparsified(const CMigMod *Mig,const CMutMod *Mut, const int& Type);
};


class CSampleWithGolub : public CSample {
public :
	CSampleWithGolub() : CSample() {};
	CSampleWithGolub(CSample* Sample) : CSample(Sample) {};
    ~CSampleWithGolub() {};
	int new_L_etc();
	int delete_L_etc();
	int lin_abs_sparsified_1D_penta(const CMigMod *Mig,const CMutMod *Mut, const int& Type);
	int sparsified(const CMigMod *Mig,const CMutMod *Mut, const int& Type);
};

//pointer to member function PAC/IS: cf Primer+ p. 1071
//extern changetype (CSample::*ChangefnPtr)(CMigMod*,CMutMod*);
//extern int (CSample::*PicalcPIMfnPtr)(const CMigMod*,const CMutMod*, const int&);
extern changetype (CSample::*ChangefnPtr)(CMigMod*,CMutMod*);
//extern changetype (CSample::*InitialChangefnPtr)(CMigMod*,CMutMod*);
// idem for direct solver/PCG
extern int (CSample::*PicalcfnPtr)(const CMigMod*,const CMutMod*, const int&);
//extern int (CSample::*InitialPicalcfnPtr)(const CMigMod*,const CMutMod*, const int&);

// define the type ptrToMemberFn, pointer to member functions of class CSample with given arguments and given 'int' return values
 typedef int (CSample::*ptrToMemberFn) (const CMigMod*,const CMutMod*, const int&);
 ptrToMemberFn setPiCalcFnPtr(std::string locPiSolver); //setPiCalcFnPtr return value is of the above defined type

double LogComb(int n, int m); //log combinatoire issu de cornuet 042007
scaledTimeType bisection_search(scaledTimeType (*func)(cumultype random, const int SampleSize,scaledTimeType currentTime,scaledTimeType previousTime,const CMigMod* Mig,const CMutMod* Mut),cumultype random,const int SampleSize,scaledTimeType previousTime,const CMigMod* Mig,const CMutMod* Mut,scaledTimeType x1,scaledTimeType x2); //pour calcul du temps avec changement continu quand pas de solution analytique

#endif
