/*
 * genetIS.h
 *
 *  Created on: 25 juil. 2013
 *      Author: cmerle
 */

#ifndef GENETIS_H_
#define GENETIS_H_

#include <vector>
#include <cstdlib>
#include <numeric>
#include "MersenneTwister.h"
#include "latin.h"
//#include <boost/random/mersenne_twister.hpp>
//#include <boost/random/uniform_01.hpp>

/**************************************************************************
 *  Useful functions                                                      *
 **************************************************************************/

inline int sum(const std::vector<int> & n){
	return std::accumulate(n.begin(), n.end(), 0);
}


/**************************************************************************
 *  Pairwise composite likelihood of a given sample                       *
 **************************************************************************/


class CalculPCL {
private:
  double theta;
  double rho;
  std::vector<double> data;
  std::vector<bool> is_computed;
public:
  CalculPCL(double th, std::size_t Nmax): theta(th), rho(th /(1.0 + th + sqrt(1.0 + 2.0 * th))),
  data(Nmax, 0.), is_computed(Nmax, false) {} ;
  double logPCL2(int i1, int i2) {
    int stepnumber = abs(i1-i2);
    if (is_computed[stepnumber] == false) {
      data[stepnumber] = stepnumber * log(rho) - .5 * log( 1.0 + 2.0 * theta );
      is_computed[stepnumber] = true;
    }
    return data[stepnumber];
  }
  
  double logPCL(const std::vector<int> & n);
};





/**************************************************************************
 *  Random stuff                                                          *
 **************************************************************************/
//extern boost::random::mt19937 gen;

inline long double randunif(){
	long double random;
    //boost::random::uniform_01<> unif;
	//return unif(gen);
    random=alea();
    return (long double) random;
}

template<class T>
int discreterand(const std::vector<T> & p) {
	int tmp = 0;


	double sum_proba = std::accumulate(p.begin(), p.end(), .0);
	double u = randunif() * sum_proba;
	double partial_sum = p[0];

	while ((u - partial_sum)>numeric_limits<double>::epsilon()) {
		++tmp;
		partial_sum += p[tmp];
	}

	return tmp;
}


template<class T>
int discreterand_sum(const std::vector<T> & p, double sum_p, int minall) {
	int tmp = minall;
  double u = randunif() * sum_p;
	double partial_sum = p[minall];
  
	while ((u - partial_sum)>numeric_limits<double>::epsilon()) {
		++tmp;
		partial_sum += p[tmp];
	}
	return tmp;
}

template<class T>
int discreterand_proba(const std::vector<T> & proba) {
	int tmp = 0;
    
	double u = randunif();
	double partial_sum = proba[0];
    
	while ((u - partial_sum)>numeric_limits<double>::epsilon()) {
		++tmp;
		partial_sum += proba[tmp];
  }
	return tmp;
}

class SIS {
public:
	SIS (const std::vector<int> nn, double actual_theta, double ancestral_theta,
			double TT): n(nn),t(.0), w(1.), thetaact(actual_theta),
			thetaanc(ancestral_theta), T(TT), sum_n(sum(n)), nballele((int)n.size()),minall((int)0),maxall((int)n.size()-1),
			fini(false),EventNbInPopAnc((int)0), treeId((int)0), anyt(0.){}//CM 13.02.14

	std::vector<int> & current_sample(){ return n; }

	double & weight() {return w;}
	const double weight() const {return w;}
	double & anything() {return anyt;}
	const double anything() const {return anyt;}
	double & time() {return t;}
  bool &  finished() { return fini;}
  double & getsum_n(){return sum_n;}
  int & NbEventInPopAnc() {return EventNbInPopAnc; } //CM 20.02.14
  void settreeId(int k){treeId=k;} // CM 12.03.14
  int & gettreeId(){return treeId;} // CM 12.03.14

	// lambda as a function of time
	double fct_lambda ();
	double theta();
	// jump rate
	double tauxsaut();

	// draw the next date of jump
	void tiragetpsacceptrejet();

	bool update_current_timenew(); // CM 24.03.14
  void update_current_timeold(); // CM 31.03.14
    
    int minallele();
    int maxallele();

	double pihat(const int k);


	// si return = 0 on a une mutation vers le type alpha - 1
	// si return = 1 on a une mutation vers le type alpha + 1
	// si return = 2 on a une coalescence
	int randbeta(int alpha);
    int randbetapartiel(int alpha);

	void algo_one_stepnew();
  void algo_one_stepold();

	// ajoute au maximum nb_iter itérations de SIS
	// si nb_iter < 0, va jusqu'au MRCA
	void algo(int nb_iter);
  void algo_IndivNb(int IndivNb);

private:
	std::vector<int> n; // current sample
	double t;      // current date
	double w;      // current weight
	double thetaact, thetaanc, T;  // value of the parameters
	double sum_n;
	int nballele;
    int minall;
    int maxall;
	bool fini;
  int EventNbInPopAnc; // CM 13.02.14
  int treeId; // CM 12.03.14
	double anyt;

};



#endif /* GENETIS_H_ */
