/***************************************************************************
© F. Rousset,  R. Leblois 2007

francois.rousset@univ-montp2.fr
raphael.leblois@mnhn.fr

This file is part of Migraine. This software is a computer program
whose purpose is to perform statistical analyses.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA,  CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license,  users are provided only
with a limited warranty  and the software's author,   the holder of the
economic rights,   and the successive licensors  have only  limited
liability.

In this respect,  the user's attention is drawn to the risks associated
with loading,   using,   modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,   and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,   more generally,  to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 ***************************************************************************/
#ifndef H_HYPERCUBE
#define H_HYPERCUBE
#include <limits> //should have std::isinf
#include <cmath>
#include <ctime>
#include <vector>
#include <sstream>
#include <cstdlib>
#include <unistd.h> //getcwd
#include "MersenneTwister.h"
#include "latin.h"
#include "migraine.h"


const bool __same_points_all_mls=true;


extern unsigned int ParameterNumber;
//extern int krigIterations;
extern int PointNumber;
extern int parDigits;
extern std::string krigingCov;
extern bool noRCallBool, noRfileBool,
			singlePointBool,
      		calculQused,
      		userPointMinOrMax,
            allowNewReplicates;
extern bool use_Rpackage;

extern std::ofstream ptls;
extern std::vector<bool> fittedindic;
extern std::vector<int> fittedpars;
extern std::vector<double> testvalue;
extern int fittedparamnbr;
extern int _mls;
extern double maxSmoothness, minSmoothness;

std::string krig();
void findandreplace( std::string & source,  const std::string & find,  const std::string & replace );
double formatAsInFile(double d, int digits);
float groot(float fS2);

class Ctestpoint {
public:
    std::vector<std::string> names;
    std::vector<float> values;
    int write(std::ostream& stream, std::string initstring);
    float scanForVar(std::string varname);
};

extern std::vector<Ctestpoint> testPointList;
extern Ctestpoint ParMaxima, ParMinima;


template <typename argtype>
int set_fittedpars(argtype lbloc[], argtype ubloc[]) {
int jj=0;
    fittedindic.resize(ParameterNumber, false);
    fittedpars.resize(0);
    for(unsigned int ii=0;ii<ParameterNumber;ii++) {
    	if(ubloc[ii]-lbloc[ii]>0.0000000001) {
        fittedindic[ii]=true;
        jj++;
  		fittedpars.push_back(ii+1); // no longer forwarded to R code
        }
    }
fittedparamnbr=jj;
return 0;
}

class CRParameters {
      std::string nextBounds;
      int nextPointNumber, iter;
//      std::vector<float> _ub, _lb;
      std::string graphicsFormat;
      double hullExpandFactor;
      std::string profileType;
      std::string DemographicModel;
public:
      CRParameters() {nextPointNumber=-1;};
      virtual ~CRParameters() {};
      int fill(int it);
      int writeR(std::ostream& stream);
      int write_R_GP_code(std::ostream& stream);
      friend int apply_final_settings();
      unsigned int getNextPtNbr() {
          if (nextPointNumber<0) {
            std::cerr<<"(!) From CRParameters::getNextPtNbr(): negative 'nextPointNumber' value. I exit."<<std::endl;
            if (cinGetOnError) std::cin.get();
            exit(-1);
          }
          return(nextPointNumber);
      };
};

extern CRParameters Rparams;


class CSampling {
public:
int boundsVariableParNbr, pointsVariableParNbr;
std::vector<std::vector<double> >DesignPoint;  //double  FR 27/12/2010 important for FormatAsInFile and then the use of set in sampling loop to remove too close values
CSampling() {
    // goshhhh numeric_limits<int>::quiet_NaN() returns 0    !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
    //boundsVariableParNbr=pointsVariableParNbr=numeric_limits<int>::quiet_NaN();
    boundsVariableParNbr=pointsVariableParNbr=-1;
}; //seems required if object is declared before main()... !
int redim(const int parnbr, const int ptnbr);
int firstfill(int mls, std::string write, const float* lb, const float* ub);
int samplingloop(const float* lb, const float* ub);
int hypercube_sampling(const float* lb, const float* ub, int ptnbr);
int read_points_from_R(std::string filename);

int analyse(const int& mls);
virtual ~CSampling() {};
};

extern CSampling DesignPoints;


class CAnalysePARAL: public CSampling {
int MaxSimultJobs;
public:
CAnalysePARAL(CSampling& DesignPoints, const int& MaxSimultJobs);
};



////////////////////////////////



#endif
