/***************************************************************************
© F. Rousset, R. Leblois 2007

francois.rousset@univ-montp2.fr
raphael.leblois@mnhn.fr

This file is part of Migraine. This software is a computer program
whose purpose is to perform statistical analyses.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and,  more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 ***************************************************************************/
#ifndef MYUTILS_H
#define MYUTILS_H

// proc['e]dures d'affichages de vecteurs et de temps ['e]coul['e]

#include <vector>
#include <iterator>
#include <fstream>



template <typename Type>
int ostream_vector(const std::vector<Type> &v, std::ostream& stream,const char*  sep=" "){
if (v.size()==0) {stream<<"\"attempted to write vector of length zero\"";}
std::ostream_iterator<Type> locout(stream,sep);
copy(v.begin(),v.end()-1,locout); //-1 otherwise the separator is added affect the last value
stream<<v.back();
//stream << endl;
return 0;
}

template <typename Type>
int ofstream_vector(std::vector<Type> &v, std::ofstream& stream,const char*  sep=" "){
if (v.size()==0) {stream<<"\"attempted to write vector of length zero\"";}
std::ostream_iterator<Type> locout(stream,sep);
copy(v.begin(),v.end()-1,locout); //-1 otherwise the separator is added affect the last value
stream<<v.back();
//stream << endl;
return 0;
}

template <typename Type1,typename Type2>
int ofstream_vector_mll(std::vector<Type1> &v,Type2 mll, std::ofstream& stream,int firstprec=6){
std::ostream_iterator<Type1> locout1(stream," ");
int prevprec=(int)stream.precision(); //CM->RL 23.01.14
stream.precision(firstprec);
copy(v.begin(),v.end(),locout1);
stream.precision(8);  // for mll, but previous precision for  v
stream<< mll << std::endl;
stream.precision(prevprec);
return 0;
}

template <typename Type1,typename Type2>
int ofstream_vector_mll(std::vector<Type1> &v,std::vector<Type2> &mll, std::ofstream& stream,int firstprec=6){
std::ostream_iterator<Type1> locout1(stream," ");
std::ostream_iterator<Type2> locout2(stream," ");
int prevprec=(int)stream.precision(); //CM->RL 23.01.14
stream.precision(firstprec);
copy(v.begin(),v.end(),locout1);
/*On the default floating-point notation, the precision field specifies the *****maximum***** number of meaningful digits to display in total counting
    ****both**** those ****before**** and those after the decimal point. Notice that it is not a minimum and therefore it does not pad the displayed number
    with trailing zeros if the number can be displayed with less digits than the precision.*/
stream.precision(8); // for mll, but previous precision for  v
copy(mll.begin(),mll.end(),locout2);
stream.precision(prevprec);
stream<<std::endl;
return 0;
}


template <typename Type>
int ostream_vec_vector(std::vector<std::vector<Type> > &v, std::ostream& stream){
std::ostream_iterator<Type> locout(stream," ");
    // Without the keyword typename, the ANSI rules require that
    // the qualified identifer T::A is not the name of a type,
    // because it is dependent upon a template paramter.
    //
    // The comment shows where the ANSI rules require the keyword typename.
///////    typedef /*typename*/ T::A type;
for(typename std::vector<std::vector<Type> >::iterator it=v.begin();it<v.end();it++) {
	copy((*it).begin(),(*it).end(),locout);
	stream << std::endl;
}
return 0;
}

/*
Pas simplement la class declaration mais aussi definition de functions
=> pas sa place dans header file
#include <ctime>


class ShowTime{
protected:
string description;
clock_t avant,mtnt;

public:
ShowTime(string description) {
	this->description=description;
	avant=clock();
}
~ShowTime() {
	mtnt=clock();
	cout<<(float) (mtnt- avant)/CLOCKS_PER_SEC<<"s "<<description;cout.flush();
}
};
*/


#endif //MYUTILS_H
