/***************************************************************************
© F. Rousset, R. Leblois 2007

francois.rousset@univ-montp2.fr
raphael.leblois@mnhn.fr

This file is part of Migraine. This software is a computer program
whose purpose is to perform statistical analyses.

This software is governed by the CeCILL license under French law and
abiding by the rules of distribution of free software.  You can  use,
modify and/ or redistribute the software under the terms of the CeCILL
license as circulated by CEA, CNRS and INRIA at the following URL
"http://www.cecill.info/licences/Licence_CeCILL_V2-en.txt".

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author, the holder of the
economic rights, and the successive licensors  have only  limited
liability.

In this respect, the user's attention is drawn to the risks associated
with loading, using, modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate, and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or
data to be ensured and, more generally, to use and operate it in the
same conditions as regards security.

The fact that you are presently reading this means that you have had
knowledge of the CeCILL license and that you accept its terms.

 ***************************************************************************/
#include <vector>
#include <string>
#include <fstream> // ofstream..
#include "latin.h" //numero version...
#include "genepop.h" //slmt pour inputCheckBool et fichier_genepop;
#include "hypercube.h" //krigIterations,testPointList...
#include "mainDG.h" //golubbool;
#include "migraine.h"
#include "locus.h"
#include "myutils.h"



#ifdef WIN32
#include <windows.h>
#endif


using namespace std;

//////////////////////////////
int nloci=__nloci;
int coreNbr=__coreNbr;
// for R file
int krigmax=-1; // if no positive value is given by user, then apply system deficit defaults defined in R code
int kriglength=2000;
int krigoverlap=500;
int _replicatedPoints=-1;//maxKrigedReplicatedPoints=30; //RL->FR 092016 : a enlever définitivement
int meanNeighbor=-1; //poor stuff for Taper...
double designRetain=1; //FR->RL 10/2015 de facto 1 et non 0.9 depuis un certain temps;
double GCVdesignRetain=1;
double scalefactor=1.;
double CILevel=0.05;
double NextBoundsLevel=0.001;
double maxSmoothness=4.;
double minSmoothness=2.;
double hullExpandFactor=2;
int GCVptnbr=-1,minKrigPtNbr=-1,maxKrigPtNbr=-1,GCVoptimSteps=100,trendOrder=0;

//PCL_SISR
bool PCL_SISR_UseIndivNbResamplingbool= false;  // CM 25.03.14
int PCL_SISR_IndivNbResampling = 10; // CM 25.03.14
int PCL_SISR_MaxEventNbInPopAnc = 100000;//CM 13.02.14
int PCL_SISR_ESSMinFall = 10;//CM 13.02.14
int PCL_SISR_StartESSAtEvent=10;
float PCL_SISR_Alpha= (float) 0.5;
float PCL_SISR_Beta= (float) 0.1;
//
//const int localndemes=10; //<<sert uniquement [`a] calibrer les bornes ci dessous <<<<<<<<<<<<<<<<
// waste a huge amount of time on large values of third param
/*float lb[ParameterNumber]={8./9*localndemes,4*localndemes,0.001};
float ub[ParameterNumber]={8./9*localndemes,15*localndemes,0.8}; */
//float lb[ParameterNumber]={0.25*localndemes,4*localndemes,0.001};
//float ub[ParameterNumber]={5*localndemes,15*localndemes,0.8};
CgeoInfo geoInfo;

vector<CProcessedLocus>ProcessedLoci;
vector<string>testParName;
vector<string>ancestralSuffixes(0);
vector<scaledTimeType>ancestralScaledTimes(0);
vector<float>testParValue;
vector<float>CovFnParam;
vector<float>habitatPars;
vector<float>LRTmin(0);
vector<float>LRTmax(0);

string mutationModel="PIM";//default
vector<string> mutationModelVector(0);
map<int, CStoreNexus> storeSeqCSamples;
vector<string> piOnePopSolverVector(0);
vector<int> lociPerMarker(0);
vector<bool> mutModelBool(5,false);
vector<bool> piOnePopSolverBool(3,false);
int markerCount = 0;
int totalLociCount = 0;
bool multipleMarkersBool = false; // default
string DemographicModel;
string subsetPointls="";
string glob_gp_filename; // make it accessible to R code writing
string globalNexusFilename; // CBR: imitating Genepop filename setup
vector<string>standardParamNames;
vector<string>standardDModelNames;
vector<string>oneDimCIvars;
vector<string>spec1DProfiles;
vector<string>spec2DProfiles;
vector<string>sequenceStat;
vector<string>sequenceWrite;
vector<string>CanonicalNames(0);
vector<int>sequenceNrunsPerPoint;
vector<int>sequenceGridSteps(0);
vector<int>sequencePointNumber;
vector<int>sequenceGivenK(0);
vector<int>sequenceAddRepl(0);
vector<int>SMMstepSizes(0);
vector<int>alleleNbrVec(0);
vector<unsigned int>KrigLoci(0);
vector<bool>SkipLoci(0);
vector<int> TempSkipLoci(0);
vector<string>BoundsDefs(0);
vector<string>KrigSpace(0);
vector<string>KrigScale(0);
vector<string>extraNames(0);
vector<string>extraScale(0);
vector<string>boundsSource(0);
vector<string>samplingScale(0);
vector<string>Rarguments(0);
vector<string>PlotOptions(0);
map<string,vector<float> > plotRange;
map<string,vector<float> > graphicPars;
stringstream RcodeStString;
string problem;
string profileType;
#ifdef __APPLE__ // && __MACH__
string graphicsFormat="eps"; //RL: was pdf, but pdf files can not be opened unless the analysis is finished and pdf file correctly close.
#else
string graphicsFormat="eps";
#endif

int geoBinNbr=0; // creating distance classes, but can take negative values

bool C_c_bool=false; // handling Control-C
void C_c_handler(int sig) {
    C_c_bool=true;
    signal(sig,C_c_handler); // this says that signal 'sig' must be handled by C_c_handler next time rather than by defautl system behavior
    // and thus the effect of signal 'sig' is to set C_c_bool to true
}
//Aim of Seg fault handling def: to be able to place a breakpoint on the fn by gdb
void SegFault_handler(int sig) {exit(-1);}
vector<double>geoAreaMin(2),geoAreaMax(2);
bool useCbool=true; // use C library, through Rmigraine or directly
bool geoAreaMinBool=false,geoAreaMaxBool=false; // indic que trouv['e] dans fichier settings
bool optimizeKrigingBool=true; //still useful within migraine.cpp
bool extrapTrueBool=false;
bool lociPolymorphicBool=false; // if true, considers only polymorphic loci
bool IBDbool=false,D1IBDbool=false,D2IBDbool=false;
bool perLocusBool=true; //whether per locus likelihoods will be in the pointls file
bool readAdHocInSettingsBool=false;
bool writeAdHocInSettingsBool=false;
bool readAdHocFileBool=false;
bool writeAdHocFileBool=false;
bool dataFileNeededBool=true;
int  thisMessage1=0;
bool useKnots=false;
bool timeNeededBool=false; /*indicator whether scaled_generation will be computed*/
bool alleleNbrChecked=false;
vector<unsigned int> samplesPos;
string gp_file,gp_fileRoot,geoUnit=""; //nom
string nexusFilename, nexusRootFilename; // CBR: imitating Genepop filename setup
double geoBinWidth=0;
double anyTwoNmuMin=0.0,anyTwoNmuMax=INFINITY,anyTwoNmMin=0.0,anyTwoNmMax=INFINITY,anyTimeMin=0.0,anyTimeMax=INFINITY; // FR->RL 01/2015 eliminer tout ca ???
int mloglikdim; // size of vector of -ln(lik) like statistics (including per locus values)
bool PACinSeqBool=false,PACancinSeqBool=false,linabsbool=true;
string samplerootname="re_dg";
float __lb[]={ (float) 0.0125, (float) 2.5, (float) 0.05}; //0.2
float __ub[]={ (float) 0.5, (float) 40.0 , (float) 0.8}; //0.999
// quick interactivity patch
vector<float>_lb(__lb,__lb+__ParameterNumber); //vector initialization from array
vector<float>_ub(__ub,__ub+__ParameterNumber);
Arguments args;



string Rtermfullpath,illConditioningMethod="ignoreSmallEigen",PiSolverFromSettingsInfo="";



int evaluateBool(bool &boolean, string buf) { // safe assignment of value `buf' to 'boolean'
	stringstream strstr(buf);
    string locstring;
    strstr>>locstring;
    if(cmp_nocase(locstring,"")==0 || cmp_nocase(locstring,"T")==0 ||
          cmp_nocase(locstring,"True")==0 || cmp_nocase(locstring,"Yes")==0 ||
          cmp_nocase(locstring,"Y")==0)
       boolean=true;
    else if(cmp_nocase(locstring,"F")==0 || cmp_nocase(locstring,"False")==0 ||
          cmp_nocase(locstring,"No")==0 || cmp_nocase(locstring,"N")==0)
       boolean=false;
    else {
        cout<<"(!) Suspicious specification for a boolean: "<< buf <<endl;
		cout<<"(!) Only \"\", \"T\", \"True\", \"Yes\", \"Y\", \"F\", \"False\", \"No\", and \"N\" are allowed"<<endl;
		cout<<"I exit."<< endl;
		if (cinGetOnError) cin.get();
		exit(-1);
    }
return(0);
}

//  transforms unspecified/partly specified/marker-wise specified parameter values into a locus-wise specification
template <typename T>
void expandParamVector(vector<T> &paramVector, string dispVar) {

	size_t vectorSize = paramVector.size();

	//	This means this parameter hasn't been declared in the settings file, use the default value found at the zeroth index.
	if (vectorSize == 1)
		paramVector.assign(totalLociCount, paramVector[0]);
	//	IF the parameter values have been in the marker-wise format, use them for all loci of the respective marker.
	else if (vectorSize == ( (size_t) markerCount ))
		for (size_t i = vectorSize; i >= 1; i--)
			paramVector.insert((paramVector.begin() + i), lociPerMarker[i - 1] - 1, paramVector[i - 1]);
	//	ELSE unspecified format, report it and quit Migraine.
	else {
		if (inputCheckBool) {
			//	FOR DEBUGGING PURPOSES ONLY: code for outputting parameter contents locus-wise
			cout << "\n" << dispVar << " contains:" << endl;
			for (typename vector<T>::iterator it = paramVector.begin(); it < paramVector.end(); it++)
				cout << *it << " ";
			cout << "\n";
		}

		cerr << "\nWrong number of arguments for \"" << dispVar << "\". \nPlease check " << settingsfilename
			 << "\nAborting Migraine...Press any key to exit.";
		if (cinGetOnError)
			cin.get();
		exit(-1);
	}
}


int StandardizeName(vector<string> &Names,vector<string> StandardNames) {
	bool found;
	//ostream_vector(Names,cout);
	//ostream_vector(StandardNames,cout);
	for (vector<string>::iterator it=Names.begin();it!=Names.end();it++) {
		found=false;
		for (vector<string>::iterator it2=StandardNames.begin();it2!=StandardNames.end();it2++) {
			if (cmp_nocase((*it),(*it2))==0) {(*it)=(*it2); found=true; break;}
		}
		if( ! found) {
			//cout << "Old name (not found) : " << (*it) << endl;
             if(cmp_nocase((*it),"2Nm")==0) (*it)="twoNm";
			else if(cmp_nocase((*it),"2Nmu")==0) (*it)="twoNmu";
			else if(cmp_nocase((*it),"2Nfoundermu")==0) (*it)="twoNfoundermu";
			else if(cmp_nocase((*it),"2Nancmu")==0) (*it)="twoNancmu";
			else if(cmp_nocase((*it),"geoNb")==0) (*it)="Nb";
			else if(cmp_nocase((*it),"2Nm1")==0 || cmp_nocase((*it),"TwoNm1")==0 ||
                    cmp_nocase((*it),"2N1m1")==0 || cmp_nocase((*it),"TwoN1m1")==0) (*it)="M1";
			else if(cmp_nocase((*it),"2Nm2")==0 || cmp_nocase((*it),"TwoNm2")==0 ||
                    cmp_nocase((*it),"2N2m2")==0 || cmp_nocase((*it),"TwoN2m2")==0) (*it)="M2";
            else if(cmp_nocase((*it),"2Nmratio")==0 || cmp_nocase((*it),"twoNmratio")==0) (*it)="NMratio";
            else if(cmp_nocase((*it),"m1m2ratio")==0) (*it)="mratio";
            else if(cmp_nocase((*it),"Q")==0) (*it)="Q1";
			else if(cmp_nocase((*it),"TSplit")==0 || cmp_nocase((*it),"TChange")==0  || cmp_nocase((*it),"Tstart")==0) (*it)="T";
            else if(cmp_nocase((*it),"Tgmu")==0) (*it)="Tgmu";
			else if(cmp_nocase((*it),"Duration")==0 || cmp_nocase((*it),"DChange")==0) (*it)="D";
            else if(cmp_nocase((*it),"Dgmu")==0) (*it)="Dgmu";
			else if(cmp_nocase((*it),"Growth")==0 || cmp_nocase((*it),"rate")==0) (*it)="growthRate";
			else if(cmp_nocase((*it),"PopSizeRatio")==0 || cmp_nocase((*it),"Ratio")==0) (*it)="Nratio";
			else if(cmp_nocase((*it),"AncPopSizeRatio")==0 || cmp_nocase((*it),"AncRatio")==0 || cmp_nocase((*it),"Nancratio")==0) (*it)="Nratio";
			else if(cmp_nocase((*it),"ActFounderPopSizeRatio")==0 || cmp_nocase((*it),"ActFounderRatio")==0 || cmp_nocase((*it),"Nfounderratio")==0) (*it)="NactNfounderratio";
            else if(cmp_nocase((*it),"FounderAncPopSizeRatio")==0 || cmp_nocase((*it),"FounderAncRatio")==0 || cmp_nocase((*it),"NfounderAncratio")==0) (*it)="NfounderNancratio";
			else if(cmp_nocase((*it),"2NmuSEQ")==0) (*it)="twoNmuSEQ";
			else if(cmp_nocase((*it),"2NancmuSEQ")==0) (*it)="twoNancmuSEQ";
			else {cout << "Found a non-standard parameter name : |" << (*it) <<"|"<< endl;return(-1);}
		}
	}
	return(0);
}


int read_settings_file(const string filename) {
string buf="",var="";
//stringstring good mostly for string/numbers conversions
string::size_type pos=0;
//int tempo;
char bidon;
//double val;
RcodeStString.clear();
int lineCounter=1;// to check whether cmdline is the first argument

standardParamNames.resize(0);//used by standardizeNames
standardParamNames.push_back("lattNb"); //etc toutes les variables en capitalization lisible et standardis['e]e
standardParamNames.push_back("Nb");
standardParamNames.push_back("condS2");
standardParamNames.push_back("twoNmu");
standardParamNames.push_back("pGSM");
standardParamNames.push_back("twoNancmu");
standardParamNames.push_back("twoNfoundermu");
standardParamNames.push_back("g");
standardParamNames.push_back("twoNm");
standardParamNames.push_back("Q1");
standardParamNames.push_back("M1");
standardParamNames.push_back("M2");
standardParamNames.push_back("m1overmu");
standardParamNames.push_back("m2overmu");
standardParamNames.push_back("NMratio");
standardParamNames.push_back("mratio");
standardParamNames.push_back("T");
standardParamNames.push_back("D");
standardParamNames.push_back("Tgmu");
standardParamNames.push_back("Dgmu");
standardParamNames.push_back("Nratio");
//standardParamNames.push_back("Nancratio");
standardParamNames.push_back("NactNfounderratio");
standardParamNames.push_back("NfounderNancratio");
standardParamNames.push_back("growthRate");
standardParamNames.push_back("twoNmuSEQ");	//	SEQ = sequence marker
standardParamNames.push_back("twoNancmuSEQ");	//	SEQ = sequence marker
standardParamNames.push_back("twoNfoundermuSEQ");	//	SEQ = sequence marker

standardDModelNames.resize(0);//used by read settings file
standardDModelNames.push_back("LinearIBD"); //etc toutes les variables en capitalization lisible et standardis['e]e
standardDModelNames.push_back("PlanarIBD"); //etc toutes les variables en capitalization lisible et standardis['e]e
standardDModelNames.push_back("1DIBD"); //etc toutes les variables en capitalization lisible et standardis['e]e
standardDModelNames.push_back("2DIBD"); //etc toutes les variables en capitalization lisible et standardis['e]e
standardDModelNames.push_back("Npop");
//standardDModelNames.push_back("2pop"); //RL 27112017 : to be removed
standardDModelNames.push_back("OnePopVarSize");
standardDModelNames.push_back("OnePopFounderFlush");
standardDModelNames.push_back("OnePop");
standardDModelNames.push_back("IM");
standardDModelNames.push_back("Generic");


ifstream settings(filename.c_str(),ios::in);
if(!settings.is_open()) {
	cout << "Unable to open file "<<filename<< endl;
	cerr << "Unable to open file "<<filename<< endl;
	if (!noRCallBool) cout<<"(!) Hence uses default path for Rterm.exe"<<endl;
//	getchar();
//	exit(-1);
	}
else {
     cout<<"Reading settings file "<<filename<<endl;
     do {
		getline(settings,buf);
		if (inputCheckBool) {cout<<"Read line:\n"<<buf<<endl;}
		if(buf.length()==0) goto nextline;
		while((buf[0]==' ')||(buf[0]=='\t')) {buf.erase(0,1);}//vire les blancs initiaux
		pos=std::min(buf.find('='),std::min(buf.find('\t'),buf.length()));
//		pos=std::min(buf.find('='),buf.length());
        if ((buf[pos])=='=') while (buf[pos-1]==' ') {buf.erase(pos-1,1); pos--;}// vire les blancs avant le =
		var=buf.substr(0,pos).c_str();
		if (inputCheckBool) {cout<<"Parsed: |"<<var<< "|" << endl;}
		if(var.length()==0) goto nextline;
		if(var[0]=='/'||var[0]=='%'||var[0]=='#') goto nextline;
		if(cmp_nocase(var,"Rcode")==0) {
			stringstream strstr(buf.substr(pos+1));
			RcodeStString<<strstr.str(); //appends for any Rcode line.
			goto nextline;
		}
   		if(cmp_nocase(var,"InputCheck")==0) {
            evaluateBool(inputCheckBool,buf.substr(pos+1));
   			goto nextline;
   		}
		/*if(cmp_nocase(var,"NumPrecision")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>numPrecision;
			if(cmp_nocase(numPrecision,"float")==0 || cmp_nocase(numPrecision,"double")==0) goto nextline;
			else {
                cerr<<"NumPrecision must be float or double'. Check settings. I exit";
                if (cinGetOnError) cin.get();
                exit(-1);
            }
		}*/
		if(cmp_nocase(var,"AncestralSwitch")==0) {
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>locstring;
			if(cmp_nocase(locstring,"DispersalRate")==0) asymptoBool=AsymptoPerDispRateBool=true;
			if(cmp_nocase(locstring,"MutationRate")==0) asymptoBool=AsymptoPerMutRateBool=true;
			goto nextline;
		}
		if(cmp_nocase(var,"DemographicModel")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>DemographicModel;
			//if(cmp_nocase(locstring,"IM")==0) {cout<<"DemographicModel=IM est casse cou...";getchar();}
			if(cmp_nocase(DemographicModel,"Linear_IBD")==0 || cmp_nocase(DemographicModel,"LinearIBD")==0) {
			    DemographicModel="LinearIBD";
				IBDbool=true,D2IBDbool=false;D1IBDbool=true;
			}
			if(cmp_nocase(DemographicModel,"Planar_IBD")==0 || cmp_nocase(DemographicModel,"PlanarIBD")==0) {
			    DemographicModel="PlanarIBD";
				IBDbool=true,D2IBDbool=true;D1IBDbool=false;
			}
			else if(cmp_nocase(DemographicModel,"Npop")==0 or cmp_nocase(DemographicModel,"2pop")==0) {
			    DemographicModel="Npop";
				IBDbool=false;
            } else if(cmp_nocase(DemographicModel,"NpopSplit")==0 || cmp_nocase(DemographicModel,"IM")==0) {
			    DemographicModel="IM";
                VarSizeFunction="Discrete";
               IBDbool=false;
            } else if(cmp_nocase(DemographicModel,"OnePop")==0){
			    DemographicModel="OnePop";
               IBDbool=false;
            } else if(cmp_nocase(DemographicModel,"OnePopVarSize")==0){
			    DemographicModel="OnePopVarSize";
                if(VarSizeFunction=="") VarSizeFunction="Exponential";
               IBDbool=false;
            } else if(cmp_nocase(DemographicModel,"OnePopFounderFlush")==0 || cmp_nocase(DemographicModel,"FounderFlush")==0){
            	cerr << "The OnePopFounderFlush model is currently under construction..." << endl;
			    DemographicModel="OnePopFounderFlush";
                if(VarSizeFunction=="") VarSizeFunction="Exponential";
               IBDbool=false;
            }
			goto nextline;
		}
		if(cmp_nocase(var,"VarSizeFunction")==0) {
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>locstring;
			if(cmp_nocase(locstring,"Discret")==0 or cmp_nocase(locstring,"Discrete")==0) VarSizeFunction="Discrete";
			else if(cmp_nocase(locstring,"Exponential")==0 or cmp_nocase(locstring,"Exp")==0) VarSizeFunction="Exponential";
			 else if(cmp_nocase(locstring,"Logistic")==0 or cmp_nocase(locstring,"Log")==0) VarSizeFunction="Logistic";
			  else {
                cout<<"(!) Misspecification : '" << locstring << "' of \"VarSizeFunction\"  in " << filename << ";"<<endl;
                cout<<"(!) Only logistic, exponential or discrete change are implemented;"<<endl;
                cout<<"(!) a exponential function will be considered instead;"<<endl;
                VarSizeFunction="exponential";
            }
			goto nextline;
		}
		if(cmp_nocase(var,"Geometry")==0) {
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>locstring;
			if(cmp_nocase(locstring,"Circular")==0) linabsbool=false;
			goto nextline;
		}
		if(cmp_nocase(var,"OutputType")==0) {
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>locstring;
			if(cmp_nocase(locstring,"perLocus")==0) perLocusBool=true;
			if(cmp_nocase(locstring,"notperLocus")==0) perLocusBool=false; //private
			goto nextline;
		}

		if(cmp_nocase(var,"ReadAdHocFiles")==0 || cmp_nocase(var,"ReadAdHocFile")==0) {
  //controls uses of processedLoci vs ad hoc files through two differetn versions of GetSample()
            readAdHocInSettingsBool=true;
            evaluateBool(readAdHocFileBool,buf.substr(pos+1));
			goto nextline;
		}
		if(cmp_nocase(var,"writeAdHocFiles")==0 || cmp_nocase(var,"writeAdHocFile")==0) {
  //controls output to ad hoc files in crunchLocTable()
            writeAdHocInSettingsBool=true;
            evaluateBool(writeAdHocFileBool,buf.substr(pos+1));
			goto nextline;
		}
		if(cmp_nocase(var,"WriteSequence")==0 ||
           cmp_nocase(var,"SequenceWrite")==0 ||
           cmp_nocase(var,"OutputWrite")==0 ||
           cmp_nocase(var,"WritePoints")==0 ) {
            string tmpbuf,prevbuf;
            int addReplicatedPoints,prevAddReplicatedPoints;
			sequenceWrite.resize(0);
			sequenceAddRepl.resize(0);
			buf=buf.substr(pos+1);
			pos=0;
            while (buf.length()>0) {
                while (buf.length()>pos && !isalnum(buf[pos]) ) pos++; //skips space and comma bfore first keyword
                if (buf.length()==pos) goto nextline;
                buf=buf.substr(pos);
                pos=0;
                int lengthToEqual=0;
                while (buf.length()>pos && (isalnum(buf[pos]) || buf[pos]=='=') ) { // seeks next space or comma, catching any '=' on the way
                    if  (buf[pos]=='=') lengthToEqual= (int) (pos + 1);//CM->RL 23.01.14
                    pos++;
                }
                tmpbuf=buf.substr(0,pos);
                if (inputCheckBool) {cout<<"Parsed: |" <<tmpbuf<< "|" << endl;}
                //standardization for later tests on these variables
                addReplicatedPoints=0; // default value, may be updated in the next block:;
                if(lengthToEqual>0) {
                    if (cmp_nocase(tmpbuf.substr(0,lengthToEqual),"AddReplicates=")==0) {
                        stringstream stst;
                        stst<<tmpbuf.substr(lengthToEqual);
                        stst>>addReplicatedPoints;
                        tmpbuf="Append";
                        allowNewReplicates=true; /// to override a check that is otherwise made in ::analyse
                    } else {
                        cerr<<"Unknown writeSequence keyword '"<<tmpbuf<<"'. Check settings. I exit.";
                        if (cinGetOnError) cin.get();
                        exit(-1);
                    }
                }
		       if (cmp_nocase(tmpbuf,"writeRpackage")==0) {
                   cerr<<"Option writeRpackage is obsolete. I exit"<<endl;
		           //cout<<"Base files for R package written. See Rmigraine.R for building/installing the package."<<endl;
                   if (pauseFinal) cin.get();
                   exit(0); // normal exit
		       }
		       if (cmp_nocase(tmpbuf,"writeRonly")==0) tmpbuf="WriteROnly";
		       if (cmp_nocase(tmpbuf,"readPoints")==0) tmpbuf="ReadPoints";
		       if (cmp_nocase(tmpbuf,"readPointsFromR")==0) tmpbuf="ReadPoints";
		       if (cmp_nocase(tmpbuf,"readBoundsOnly")==0) tmpbuf="ReadBounds";
		       if (cmp_nocase(tmpbuf,"noRCall")==0) tmpbuf="NoRCall";
		       //checks *after* standardization
		       if ((tmpbuf=="ReadPoints" || tmpbuf=="ReadBounds") && sequenceWrite.size()>1) {
		           cerr<<"(!) ReadPoints and ReadBounds forbidden after first iteration. I exit.";
		           if (cinGetOnError) cin.get();
		           exit(-1);
               }
               //appends (or not) *after* checking
               if (isdigit(tmpbuf[0])) { // e.g. 3 in "Append,3,..."
                   int locit;
                   stringstream stst;
                   stst<<tmpbuf;
                   stst>>locit;
                   for (int ii=0;ii<locit-1;ii++) {
                       sequenceWrite.push_back(prevbuf);
                       sequenceAddRepl.push_back(prevAddReplicatedPoints);
                   }
               } else {
		           if (cmp_nocase(tmpbuf,"NoRCall")==0) {
		               noRCallBool=true;  //nothing appended to sequenceWrite !
		           } else {
		               sequenceWrite.push_back(tmpbuf);
                       sequenceAddRepl.push_back(addReplicatedPoints);
		           }
               }
		       // looking for next statement
			   if (buf.length()>pos) {
			    prevbuf=tmpbuf;
			    prevAddReplicatedPoints=addReplicatedPoints;
				buf=buf.substr(pos+1);
			   	pos=0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase(var,"SamplingSpace")==0 || cmp_nocase(var,"BoundsSpace")==0 || cmp_nocase(var,"BoundsDefs")==0 || cmp_nocase(var,"BoundsDef")==0) {
		    if(cmp_nocase(var,"BoundsSpace")==0 || cmp_nocase(var,"BoundsDefs")==0 || cmp_nocase(var,"BoundsDef")==0) {
                cout<<"(!) Obsolete '"<<var<<"' setting interpreted as SamplingSpace'"<<endl;
            }
			BoundsDefs.resize(0);
			buf=buf.substr(pos+1);
			pos=0;
            while (buf.length()>0) {
			   while (buf.length()>pos && !isalnum(buf[pos]) && !(buf[pos]==',')) pos++;
			   if (buf[pos]==',') { // rencontre une , avant de l'alphanum
                  BoundsDefs.push_back("");
		          if (inputCheckBool) {cout<<"Parsed: |"<<buf.substr(0,pos)<< "|" << endl;}
                  pos++;
               }
			   if (buf.length()==pos) goto nextline;
			   // s'il arrive ici c'est qu'il a trouve de l'alphanum
			   buf=buf.substr(pos);
			   pos=0;
			   // apr[`e]s le test est invers['e], il avance tant qu'il a de l'alphanum
			   while (buf.length()>pos && isalnum(buf[pos])) pos++;
			   BoundsDefs.push_back(buf.substr(0,pos));
		       if (inputCheckBool) {cout<<"Parsed: |"<<buf.substr(0,pos)<< "|" << endl;}
			   if (buf.length()>pos) {
				buf=buf.substr(pos+1);
			   	pos=0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase(var,"KrigSpace")==0) { // variables composite vs canonical
			KrigSpace.resize(0);
			buf=buf.substr(pos+1);
			pos=0;
            while (buf.length()>0) {
			   while (buf.length()>pos && !isalnum(buf[pos]) && !(buf[pos]==',')) pos++;
			   if (buf[pos]==',') { // rencontre une , avant de l'alphanum
                  KrigSpace.push_back("");
		          if (inputCheckBool) {cout<<"Parsed: |"<<buf.substr(0,pos)<< "|" << endl;}
                  pos++;
               }
			   if (buf.length()==pos) goto nextline;
			   // s'il arrive ici c'est qu'il a trouve de l'alphanum
			   buf=buf.substr(pos);
			   pos=0;
			   // apr[`e]s le test est invers['e], il avance tant qu'il a de l'alphanum
			   while (buf.length()>pos && isalnum(buf[pos])) pos++;
			   KrigSpace.push_back(buf.substr(0,pos));
		       if (inputCheckBool) {cout<<"Parsed: |"<<buf.substr(0,pos)<< "|" << endl;}
			   if (buf.length()>pos) {
				buf=buf.substr(pos+1);
			   	pos=0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase(var,"graphicsFormat")==0 || cmp_nocase(var,"graphicFormat")==0) { // simple string
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>graphicsFormat;
			goto nextline;
		}
		if(cmp_nocase(var,"ProfileType")==0) { // simple string
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>profileType;
			goto nextline;
		}
		if(cmp_nocase(var,"Bounds")==0 || cmp_nocase(var,"NextBounds")==0) {
            if (cmp_nocase(var,"Bounds")==0) {
              cout<<"(!) Obsolete 'Bounds' setting interpreted as 'NextBounds'"<<endl;
            }
			boundsSource.resize(0);
			buf=buf.substr(pos+1);
			pos=0;
            while (buf.length()>0) {
			   while (buf.length()>pos && !isalnum(buf[pos]) && !(buf[pos]==',')) pos++;
			   if (buf[pos]==',') { // rencontre une , avant de l'alphanum
                  boundsSource.push_back("Default");
		          if (inputCheckBool) {cout<<"Parsed: |"<<buf.substr(0,pos)<< "|" << endl;}
                  pos++;
               }
			   if (buf.length()==pos) goto nextline;
			   // ELSE: s'il arrive ici c'est qu'il a trouve de l'alphanum
			   buf=buf.substr(pos);
			   pos=0;
			   // apr[`e]s le test est invers['e], il avance tant qu'il a de l'alphanum
			   while (buf.length()>pos && isalnum(buf[pos])) pos++;
		       if (inputCheckBool) {cout<<"Parsed: |"<<buf.substr(0,pos)<< "|" << endl;}
               if (cmp_nocase(buf,"FromR")==0|| cmp_nocase(buf,"From2DCI")==0) {
                  cout<<"(!) Obsolete NextBounds specification '"<<buf<<"' converted to 'pointsfromR'"<<endl;
			      boundsSource.push_back("pointsfromR");
               } else boundsSource.push_back(buf.substr(0,pos));
			   if (buf.length()>pos) {
				buf=buf.substr(pos+1);
			   	pos=0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase(var,"ExtrapolateSurfaces")==0 || cmp_nocase(var,"ExtrapolateSurface")==0) { // control plots in R
            cerr << "(!) 'ExtrapolateSurfaces' keyword is obsolete. Use Rarguments' instead." << endl;
            cerr<<"I exit."<< endl;
            if (cinGetOnError) cin.get();
            exit(-1);
		}
		if(cmp_nocase(var,"Roptions")==0) {
            cerr << "(!) 'Roptions' keyword is obsolete. Use Rarguments' instead." << endl;
            cerr<<"I exit."<< endl;
            if (cinGetOnError) cin.get();
            exit(-1);
		}
		if(cmp_nocase(var,"KrigingCov")==0) { // controls kriging covariance family
            string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>locstring;
            if (cmp_nocase(locstring,"Matern")==0) krigingCov="Matern";
            if (cmp_nocase(locstring,"GneitingMatern")==0) krigingCov="GneitingMatern";

            if (cmp_nocase(locstring,"Exp")==0 || cmp_nocase(locstring,"Exponential")==0) {
                //krigingCov="Exponential";
                cerr << "(!) Option 'KrigingCov=Exp' is now obsolete. Use 'KrigingCov=Matern' or 'KrigingCov=GneitingMatern' instead." << endl;
                cerr<<"I exit."<< endl;
                if (cinGetOnError) cin.get();
                exit(-1);

            }
            if (cmp_nocase(locstring,"Taper")==0){
                    //krigingCov="Taper";
                    cerr << "(!) Option 'KrigingCov=Taper' is now obsolete. Use 'KrigingCov=Matern' or 'KrigingCov=GneitingMatern' instead." << endl;
                    cerr<<"I exit."<< endl;
                    if (cinGetOnError) cin.get();
                    exit(-1);
                    
                }
			goto nextline;
		}
/*		if(cmp_nocase(var,"meanNeighbor")==0) { this was a poor hack for [Taper which does not work]
			stringstream strstr(buf.substr(pos+1));
			strstr>>meanNeighbor;
			goto nextline;
		} */
		if(cmp_nocase(var,"trendOrder")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>trendOrder;
			goto nextline;
		}
		if(cmp_nocase(var,"KrigScale")==0 || cmp_nocase(var,"ParameterScale")==0) { //logscale etc
			if(cmp_nocase(var,"ParameterScale")==0) cout << "(!) Old Krig scale specification : 'ParameterScale' converted to 'KrigScale'" << endl;
			KrigScale.resize(0);
			buf=buf.substr(pos+1);
            while((buf[0]==' ')||(buf[0]=='\t')) {buf.erase(0,1);}//vire les blancs initiaux
            pos=buf.length();
            while (buf[pos-1]==' ') {buf.erase(pos-1,1); pos--;}// RL : vire les blancs finaux
            
            pos=0;
            while (buf.length()>0) {
                while((buf[0]==' ')||(buf[0]=='\t')) {buf.erase(0,1);}//vire les blancs initiaux
                while (buf.length()>pos && !isalnum(buf[pos]) && !(buf[pos]==',')) pos++;
                if (buf[pos]==',') { // rencontre une , avant de l'alphanum
                    KrigScale.push_back("");
                    if (inputCheckBool) {cout<<"(+1) Parsed comma following: |"<<buf.substr(0,pos)<< "|" << endl;}
                    pos++;
                    // FR ici on pourrait eventuellement reconnaitre une virgule terminale ? :RL : je dirais non... a vérifier
                    if (buf.length()==pos) {
                        KrigScale.push_back("");
                        if (inputCheckBool) {cout<<"(+1) Parsed final comma."<< endl;}
                    }
                }
                if (buf.length()==pos) goto nextline;
                buf=buf.substr(pos);
                while((buf[0]==' ')||(buf[0]=='\t')) {buf.erase(0,1);}//vire les blancs initiaux
                if (inputCheckBool) {cout<<"Remainder: |"<<buf<< "|" << endl;}
                pos=0;
                // apr[`e]s le test est invers['e], il avance tant qu'il a de l'alphanum
                while (buf.length()>pos && isalnum(buf[pos])) pos++;
                if (inputCheckBool) {cout<<"(+1) Parsed : |"<<buf.substr(0,pos)<< "|" << endl;}
                KrigScale.push_back(buf.substr(0,pos));
                if (buf.length()>pos) {
                    if(buf[pos]==',' && (buf.substr(pos+1)).length()==0) {//RL : on a une virgule a la fin de la ligne...
                        KrigScale.push_back("");
                        if (inputCheckBool) {cout<<"Parsed final comma: |"<<buf.substr(pos+1)<< "|" << endl;}
                        goto nextline;
                    }
                    buf=buf.substr(pos+1);
                    pos=0;
                } else {
                    goto nextline;
                }
            }
            goto nextline;
		}
		if(cmp_nocase(var,"samplingScale")==0) { //logscale etc
			samplingScale.resize(0);
			buf=buf.substr(pos+1);
            while((buf[0]==' ')||(buf[0]=='\t')) {buf.erase(0,1);}//vire les blancs initiaux
            pos=buf.length();
            while (buf[pos-1]=='\r') {buf.erase(pos-1,1); pos--;}// RL : vire le saut de ligne (\r) final (et empechait de trouver la virgule en fin de ligne)
            pos=buf.length();
            while ((buf[pos-1]==' ')||(buf[0]=='\t')) {buf.erase(pos-1,1); pos--;}// RL : vire les blancs finaux

			pos=0;
            while (buf.length()>0) {
                while((buf[0]==' ')||(buf[0]=='\t')) {buf.erase(0,1);}//vire les blancs initiaux
			   while (buf.length()>pos && !isalnum(buf[pos]) && !(buf[pos]==',')) pos++;
			   if (buf[pos]==',') { // rencontre une , avant de l'alphanum
                  samplingScale.push_back("");
		          if (inputCheckBool) {cout<<"(+1) Parsed comma following: |"<<buf.substr(0,pos)<< "|" << endl;}
                   pos++;
                  // FR ici on pourrait eventuellement reconnaitre une virgule terminale ? :RL : je dirais non... a vérifier
                   if (buf.length()==pos) {
                      samplingScale.push_back("");
   		              if (inputCheckBool) {cout<<"(+1) Parsed final comma."<< endl;}
                  }
               }
			   if (buf.length()==pos) goto nextline;
			   buf=buf.substr(pos);
               while((buf[0]==' ')||(buf[0]=='\t')) {buf.erase(0,1);}//vire les blancs initiaux
               if (inputCheckBool) {cout<<"Remainder: |"<<buf<< "|" << endl;}
			   pos=0;
			   // apr[`e]s le test est invers['e], il avance tant qu'il a de l'alphanum
			   while (buf.length()>pos && isalnum(buf[pos])) pos++;
		       if (inputCheckBool) {cout<<"(+1) Parsed : |"<<buf.substr(0,pos)<< "|" << endl;}
			   samplingScale.push_back(buf.substr(0,pos));
			   if (buf.length()>pos) {
                   if(buf[pos]==',' && (buf.substr(pos+1)).length()==0) {//RL : on a une virgule a la fin de la ligne...
                       samplingScale.push_back("");
                       if (inputCheckBool) {cout<<"Parsed final comma: |"<<buf.substr(pos+1)<< "|" << endl;}
                       goto nextline;
                   }
				buf=buf.substr(pos+1);
			   	pos=0;
               } else {
                   goto nextline;
               }
			}
            goto nextline;
		}
		if(cmp_nocase(var,"plotRange")==0) {
            string subbuf,szut;
            stringstream stst;
            float zut;
            char bidon;
            vector<float> locvec;
            vector<string> checkname;
			unsigned int eqpos;
			buf=buf.substr(pos+1); // =]...[end
            while (buf.length()>0) { // examen de RHS complet de plotRange
               locvec.resize(0);checkname.resize(0);
               pos=0;
			   while (buf.length()>pos && !(buf[pos]==';')) pos++; //cherche ; ou eol
		       if (inputCheckBool) {cout<<"Parsed: |"<<buf.substr(0,pos)<< "|" << endl;}
			   subbuf=buf.substr(0,pos); //]varname= pair of values[,
			   eqpos=0;
			   // il analyse la substring de forme par=pair of values
			   while (subbuf.length()>eqpos && !(subbuf[eqpos]=='=')) eqpos++; //cherche =
			   stst<<subbuf.substr(0,eqpos);
			   stst>>szut;  //gets rid of the space chars
			   checkname.push_back(szut); //]varname[ =
                if(StandardizeName(checkname,standardParamNames)!=0) {
                    cerr<<"(!) Suspicious parameter name specification in plotRange option: "<< szut <<endl;
                    cerr<<"    If in doubt, use one of the standard names: "<<endl;
                    ostream_vector(standardParamNames,cerr);
                    cerr<<"I exit."<< endl;
                    if (cinGetOnError) cin.get();
                    exit(-1);
                }
			   stst.clear();
			   stst<<subbuf.substr(eqpos+1);
			   stst>>zut;
			   locvec.push_back(zut); //= ]value1[,
               while (!stst.eof() && !(isdigit(bidon=stst.peek())) && bidon!='.' && bidon!='-') stst.get();
			   stst>>zut;
			   locvec.push_back(zut); //= ]value2[,
			   // checkname[0] is a standard name, while szut may not be.
			   plotRange[checkname[0]]=locvec;  // insertion in map
			   stst.clear();
			   if (buf.length()>pos) {
				buf=buf.substr(pos+1); //tout le reste apr[`e]s ',' eventuellement
			   	pos=0;
			   } else buf="";
			} // now we're at the end of the plotRange line
			goto nextline;
		}
		if(cmp_nocase(var,"GraphicPars")==0 || cmp_nocase(var,"GraphicsPars")==0) {
            string subbuf,szut;
            stringstream stst;
            float zut;
            char bidon;
            string zutst;
            vector<float> locvec;
            vector<string> checkname;
			unsigned int eqpos;
			buf=buf.substr(pos+1); // =]...[end
            while (buf.length()>0) { // examen de RHS complet de plotRange
               locvec.resize(0);checkname.resize(0);
               pos=0;
			   while (buf.length()>pos && !(buf[pos]==';')) pos++; //cherche ; ou eol
		       if (inputCheckBool) {cout<<"Parsed (should be single R 'par' argument): |"<<buf.substr(0,pos)<< "|" << endl;}
			   subbuf=buf.substr(0,pos); //]varname= pair of values[,
			   eqpos=0;
			   // il analyse la substring de forme par=pair of values
			   while (subbuf.length()>eqpos && !(subbuf[eqpos]=='=')) eqpos++; //cherche =
			   stst<<subbuf.substr(0,eqpos);
			   stst>>szut;  //gets rid of the space chars
	           if (inputCheckBool) {cout<<"Parsed (should be single R 'par' parameter): |"<<szut<< "|" << endl;}
			   checkname.push_back(szut); //]varname[ =
			   stst.clear();
			   stst<<subbuf.substr(eqpos+1);
			   if (cmp_nocase(szut,"pty")==0) {
			       stst>>zutst;
			       cerr<<"Need to finish coding of string 'par' options such as pty"<<endl;
			       exit(-1);
               }
			   while (!stst.eof()) {//FR->FR faire un fonction de ce genre de boucles de lecture
				  stst>>zut;
		          if (inputCheckBool) {cout<<"Parsed (should be (an element of) the value of a single R 'par' parameter): |"<<zut<< "|" << endl;}
                  while (!stst.eof() && !(isdigit(bidon=stst.peek())) && bidon!='.' && bidon!='-') {
                      stst.get();
//cout<<stst.str();getchar();
/**
FR-F>R if we used , rather than ; as a separator, there's an infinite loop here
the cout shows a problem with only an '=' that has disappeared
The conception of this of the whole stringstream reading should be reconsidered: first locate all ';' then check there is only one '='
between each ';'...
**/
                  }
				  locvec.push_back(zut);
			   }
			   // checkname[0] is a standard name, while szut may not be.
			   graphicPars[checkname[0]]=locvec;  // insertion in map
			   stst.clear();
			   if (buf.length()>pos) {
				buf=buf.substr(pos+1); //tout le reste apr[`e]s ',' eventuellement
			   	pos=0;
			   } else buf="";
			} // now we're at the end of the plotRange line
			goto nextline;
		}
		if(cmp_nocase(var,"extraScale")==0) {
            string subbuf,zut;
            stringstream stst;
            extraNames.resize(0);
            extraScale.resize(0);
			unsigned int eqpos;
			buf=buf.substr(pos+1); // =]...[end
            while (buf.length()>0) { // examen de RHS complet de TestPoint
               pos=0;
			   while (buf.length()>pos && !(buf[pos]==',')) pos++; //cherche , ou eol
		       if (inputCheckBool) {cout<<"Parsed: |"<<buf.substr(0,pos)<< "|" << endl;}
			   subbuf=buf.substr(0,pos); //]varname=value[,
			   eqpos=0;
			   // il analyse la substring de forme par=value
			   while (subbuf.length()>eqpos && !(subbuf[eqpos]=='=')) eqpos++; //cherche =
			   stst<<subbuf.substr(0,eqpos);
			   stst>>zut;  //gets rid of the space chars
			   extraNames.push_back(zut); //]varname[ =
			   stst.clear();
			   stst<<subbuf.substr(eqpos+1);
			   stst>>zut;  //gets rid of the space chars
			   extraScale.push_back(zut); //= ]value[,
			   stst.clear();
			   if (buf.length()>pos) {
				buf=buf.substr(pos+1); //tout le reste apr[`e]s ',' eventuellement
			   	pos=0;
			   } else buf="";
			} // now we're at the end of the TestPoint line
			if(StandardizeName(extraNames,standardParamNames)!=0) {
				cerr<<"(!) Suspicious parameter name specification in extraScale option: "<< buf <<endl;
                cerr<<"    If in doubt, use one of the standard names: "<<endl;
                ostream_vector(standardParamNames,cerr);
				cerr<<"I exit."<< endl;
				if (cinGetOnError) cin.get();
				exit(-1);
			}
			goto nextline;
		}
		if(cmp_nocase(var,"TimeScale")==0 || cmp_nocase(var,"TimeScaled")==0) {// To choose between pop size or mutatio  rate scaling for time
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>locstring;
			if(cmp_nocase(locstring,"PopSize")==0) {
				PopSizeScaleForTimebool=true;
			} else if(cmp_nocase(locstring,"MutationRate")==0) {
					PopSizeScaleForTimebool=false;
				} else {
                    cout<<"(!) Option for TimeScale in " << filename << " is misspecified." << endl;
						cout << "\n (!)PopSize or MutationRate are the only implemented options, I exit."<<endl;
						if (cinGetOnError) cin.get();
						exit(-1);
                }
			goto nextline;
		}
		if(cmp_nocase(var,"Loci")==0) { // simple string
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>locstring;
			if(cmp_nocase(locstring,"Polymorphic")==0 || cmp_nocase(locstring,"PolymorphicOnly")==0) lociPolymorphicBool=true;
            if(cmp_nocase(locstring,"All")==0) lociPolymorphicBool=false;
			goto nextline;
		}
          //selection of loci to skip for the beginning of then run
         // used to check consistency of data under GSM/SMM
         if(cmp_nocase(var,"SkipLoci")==0 || cmp_nocase(var,"SkipLocus")==0 || cmp_nocase(var,"SkippedLoci")==0) {
             TempSkipLoci.resize(0); // prophylactic
             unsigned int value;
             string reste=buf.substr(pos+1);
             stringstream strstr(reste);
             while (!strstr.eof()) {
                 //strstr.get*(); to skip any one-character separator which is here
                 while (!(isdigit(bidon=strstr.peek())) && bidon!='.') strstr.get();
                 strstr>>value;
                 TempSkipLoci.push_back(value);
             }
             strstr.clear(); // c'est le truc essentiel pour le r['e]util... snif
             goto nextline;
         }
		if(cmp_nocase(var,"krigIterations")==0) {
           cout << "(!) Obsolete keyword 'krigIterations' is ignored. Use 'WriteSequence' instead." << endl;
/*			stringstream strstr(buf.substr(pos+1));
			strstr>>krigIterations;
			if(krigIterations==-3) {
              cout << "(!) Obsolete specification krigIterations==-3 converted to 'WriteRKrig' in WriteSequence" << endl;
              if(cinGetOnError) cin.get();
              exit(-1);
            }
*/
			goto nextline;
		}
		if(cmp_nocase(var,"OptimizeKriging")==0) {
           cerr << "(!) 'OptimizeKriging' keyword is obsolete. Use 'Rarguments' instead." << endl;
            if(cinGetOnError) cin.get();
            exit(-1);
		}
		if(cmp_nocase(var,"Control-cPeek")==0) {
			stringstream strstr(buf.substr(pos+1));
            evaluateBool(C_c_bool,buf.substr(pos+1));
			goto nextline;
		}
		if(cmp_nocase(var,"SegFault")==0) {
            signal(SIGSEGV,SegFault_handler);
			goto nextline;
		}
		if(cmp_nocase(var,"RunsTempFile")==0 || cmp_nocase(var,"RunTempFiles")==0) {
            evaluateBool(runsTempFileBool,buf.substr(pos+1));
			goto nextline;
		}
		if(cmp_nocase(var,"subsetPointls")==0) {
			subsetPointls=buf.substr(pos+1); // verbatim copy, spaces included
			goto nextline;
		}
		if(cmp_nocase(var,"krigmax")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>krigmax;
			goto nextline;
		}
		if(cmp_nocase(var,"kriglength")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>kriglength;
			goto nextline;
		}
		if(cmp_nocase(var,"krigoverlap")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>krigoverlap;
			goto nextline;
		}
		if(cmp_nocase(var,"replicatedPoints")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>_replicatedPoints;
			goto nextline;
		}
        if(cmp_nocase(var,"maxKrigedReplicatedPoints")==0) {
            cerr << "(!) 'maxKrigedReplicatedPoints' keyword has no current use." << endl;
            cerr<<" Execution will nevertheless proceed as intended." << endl;
 		}
		if(cmp_nocase(var,"gridSteps")==0 || cmp_nocase(var,"gridStepsNbr")==0 || cmp_nocase(var,"gridStep")==0) {
			sequenceGridSteps.resize(0);
			int value;
			string reste=buf.substr(pos+1);
			stringstream strstr(reste);
			while (!strstr.eof()) {
				strstr>>value;
/*strstr.get(); to skip any one-character separator which is necessarily here
is not sufficient if there are whitespaces after the last value. If so, eof is not reached
but no new value is read -> the last value is duplicated */
//(((bidon=strstr.peek())<'0') || (bidon>'9'))
                 while (!strstr.eof() && !(isdigit(bidon=strstr.peek()))) strstr.get();
				sequenceGridSteps.push_back(value);
			}
			strstr.clear(); // c'est le truc essentiel pour le r['e]util... snif
			goto nextline;
		}
		if(cmp_nocase(var,"CrossValidationPointsNumber")==0 || cmp_nocase(var,"GCVptnbr")==0 || cmp_nocase(var,"GCVptnb")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>GCVptnbr;
			goto nextline;
		}
		if(cmp_nocase(var,"minKrigPtNbr")==0 || cmp_nocase(var,"minKrigPtNb")==0 || cmp_nocase(var,"minKrigPointNumber")==0 || cmp_nocase(var,"minKrigPointsNumber")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>minKrigPtNbr;
			goto nextline;
		}
		if(cmp_nocase(var,"maxKrigPtNbr")==0 || cmp_nocase(var,"maxKrigPointNumber")==0 || cmp_nocase(var,"maxKrigPointsNumber")==0 || cmp_nocase(var,"maxKrigPtNb")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>maxKrigPtNbr;
			goto nextline;
		}
		if(cmp_nocase(var,"CrossValidationOptimSteps")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>GCVoptimSteps;
			goto nextline;
		}
		if(cmp_nocase(var,"CovFnParam")==0 || cmp_nocase(var,"CovFnParams")==0 || cmp_nocase(var,"CovFnParameters")==0) {// doit contenir smoothness ?!
			CovFnParam.resize(0); // discards default values
			float value;
			string reste=buf.substr(pos+1);
			stringstream strstr(reste);
			while (!strstr.eof()) {
				strstr>>value;
/*strstr.get(); to skip any one-character separator which is necessarily here
is not sufficient is there are whitespaces after the last value. If so, eof is not reached
but no new value is read -> the last value is duplicated */
//(((bidon=strstr.peek())<'0') || (bidon>'9'))
                 while (!strstr.eof() && !(isdigit(bidon=strstr.peek())) && bidon!='.' && bidon!='-') strstr.get();
				CovFnParam.push_back(value);
			}
			strstr.clear(); // c'est le truc essentiel pour le r['e]util... snif
			optimizeKrigingBool=false;
			goto nextline;
		}
		if(cmp_nocase(var,"FixedSmoothness")==0) {
			string locstring;
			stringstream strstr(buf.substr(pos+1));
            strstr>>minSmoothness;
            maxSmoothness=minSmoothness; // sufficient for CKrigCoefs; no fixedSmoohtnessBool required
            goto nextline;
		}
		if(cmp_nocase(var,"maxSmoothness")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>maxSmoothness;
			goto nextline;
		}
		if(cmp_nocase(var,"minSmoothness")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>minSmoothness;
			goto nextline;
		}
		if(cmp_nocase(var,"SelectTop")==0) {
				cerr<<"(!) SelectTop is obsolete and ignored. "<<endl;
			goto nextline;
		}
#ifdef UNIX_PARAL
		if(cmp_nocase(var,"ProcessorNumber")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>AvailableProcessors;
			goto nextline;
		}
#endif
         //RL 072017 : nbr of cores for paralelle computations of profiles in blackbox()
         if(cmp_nocase(var,"CoreNumberForR")==0 || cmp_nocase(var,"CoreNbrForR")==0 || cmp_nocase(var,"CoreNbForR")==0 || cmp_nocase(var,"CoreNumber")==0  || cmp_nocase(var,"CoreNbr")==0  || cmp_nocase(var,"CoreNb")==0) {
             stringstream strstr(buf.substr(pos+1));
             strstr>>coreNbr;
             goto nextline;
         }
		if(cmp_nocase(var,"Job")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>JobMin;
			JobMax=JobMin;
			goto nextline;
		}
		if(cmp_nocase(var,"JobMin")==0 || cmp_nocase(var,"MinJob")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>JobMin;
			goto nextline;
		}
		if(cmp_nocase(var,"JobMax")==0 || cmp_nocase(var,"MaxJob")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>JobMax;
			goto nextline;
		}
		if(cmp_nocase(var,"LowerBound")==0 || cmp_nocase(var,"LowerBounds")==0) {
			_lb.resize(0); // discards default values
			float value;
			string reste=buf.substr(pos+1);
			stringstream strstr(reste);
			while (!strstr.eof()) {
				strstr>>value;
/*strstr.get(); to skip any one-character separator which is necessarily here
is not sufficient is there are whitespaces after the last value. If so, eof is not reached
but no new value is read -> the last value is duplicated */
//(((bidon=strstr.peek())<'0') || (bidon>'9'))
                 while (!strstr.eof() && !(isdigit(bidon=strstr.peek())) && bidon!='.' && bidon!='-') strstr.get();
				_lb.push_back(value);
			}
			strstr.clear(); // c'est le truc essentiel pour le r['e]util... snif
			singlePointBool=false; // FR rajout 12/2009

			goto nextline;
		}
		if(cmp_nocase(var,"UpperBound")==0 || cmp_nocase(var,"UpperBounds")==0) {
			_ub.resize(0); // discards default values
			float value;
			string reste=buf.substr(pos+1);
			stringstream strstr(reste);
			while (!strstr.eof()) {
				strstr>>value;
                 while (!strstr.eof() && !(isdigit(bidon=strstr.peek())) && bidon!='.' && bidon!='-') strstr.get();
				_ub.push_back(value);
			}
			strstr.clear(); // c'est le truc essentiel pour le r['e]util... snif
			singlePointBool=false; // FR rajout 12/2009

			goto nextline;
		}
		if(cmp_nocase(var,"ParameterValue")==0 || cmp_nocase(var,"ParameterPoint")==0) {
			_ub.resize(0); // discards default values
			float value;
			string reste=buf.substr(pos+1);
			stringstream strstr(reste);
			while (!strstr.eof()) {
				strstr>>value;
                 while (!strstr.eof() && !(isdigit(bidon=strstr.peek())) && bidon!='.' && bidon!='-') strstr.get();
				_ub.push_back(value);
			}
		    _lb=_ub;	//copy(_ub.begin(),_ub.end(),_lb.begin());
            // will set lowerbound=upperbound and NoRCallBool in apply_...
			singlePointBool=true;
			goto nextline;
		}
		if(cmp_nocase(var,"ParMaxima")==0) {
            string subbuf;
            testParName.resize(0); // using the same structure as for test points
            testParValue.resize(0);
			float value;
			unsigned int eqpos;
			buf=buf.substr(pos+1); // =]...[end
            while (buf.length()>0) { // examen de RHS complet de TestPoint
               pos=0;
			   while (buf.length()>pos && !(buf[pos]==',')) pos++; //cherche , ou eol
			   subbuf=buf.substr(0,pos); //]varname=value[,
               // il analyse la substring de forme par=value
                string::size_type pos2=0;
                while (subbuf.length()>pos2 && !isalnum(subbuf[pos2]) ) pos2++; //skips all non alpha numeric char before first keyword
                subbuf=subbuf.substr(pos2);
               if (inputCheckBool) {cout<<"Parsed: |"<<subbuf<< "|" << endl;}
			   eqpos=0;
			   while (subbuf.length()>eqpos && !(subbuf[eqpos]=='=')) eqpos++; //cherche =
			   testParName.push_back(subbuf.substr(0,eqpos)); //]varname[=
			   string remainder=subbuf.substr(eqpos+1); //=]value[,
			   { stringstream strstr(remainder);
			     strstr>>value; //conversion en nombre
               } //clears strstr
			   testParValue.push_back(value);
			   ParMaxima.names=testParName;  // there may be several names per point...
			   ParMaxima.values=testParValue; // and several values
//ostream_vector(testParName,cout);
//ostream_vector(testParValue,cout);getchar();
			   if (buf.length()>pos) {
				buf=buf.substr(pos+1); //tout le reste apr[`e]s ',' eventuellement
			   	pos=0;
			   } else buf="";
			} // now we're at the end of the TestPoint line
			if(StandardizeName(ParMaxima.names,standardParamNames)!=0) {
				cerr<<"(!) Suspicious parameter name specification in Parmaxima option: "<< buf <<endl;
                cerr<<"    If in doubt, use one of the standard names: "<<endl;
                ostream_vector(standardParamNames,cerr);
				cerr<<"I exit."<< endl;
				if (cinGetOnError) cin.get();
				exit(-1);
			}
			goto nextline;
		}
		if(cmp_nocase(var,"ParMinima")==0) {
            string subbuf;
            testParName.resize(0); // using the same structure as for test points
            testParValue.resize(0);
			float value;
			unsigned int eqpos;
			buf=buf.substr(pos+1); // =]...[end
            while (buf.length()>0) { // examen de RHS complet de TestPoint
               pos=0;
			   while (buf.length()>pos && !(buf[pos]==',')) pos++; //cherche , ou eol
			   subbuf=buf.substr(0,pos); //]varname=value[,
               // il analyse la substring de forme par=value
                string::size_type pos2=0;
                while (subbuf.length()>pos2 && !isalnum(subbuf[pos2]) ) pos2++; //skips all non alpha numeric char before first keyword
                subbuf=subbuf.substr(pos2);
               if (inputCheckBool) {cout<<"Parsed: |"<<subbuf<< "|" << endl;}
			   eqpos=0;
			   while (subbuf.length()>eqpos && !(subbuf[eqpos]=='=')) eqpos++; //cherche =
			   testParName.push_back(subbuf.substr(0,eqpos)); //]varname[=
			   string remainder=subbuf.substr(eqpos+1); //=]value[,
			   { stringstream strstr(remainder);
			     strstr>>value; //conversion en nombre
               } //clears strstr
			   testParValue.push_back(value);
			   ParMinima.names=testParName;  // there may be several names per point...
			   ParMinima.values=testParValue; // and several values
//ostream_vector(testParName,cout);
//ostream_vector(testParValue,cout);getchar();
			   if (buf.length()>pos) {
				buf=buf.substr(pos+1); //tout le reste apr[`e]s ',' eventuellement
			   	pos=0;
			   } else buf="";
			} // now we're at the end of the TestPoint line
			if(StandardizeName(ParMinima.names,standardParamNames)!=0) {
				cerr<<"(!) Suspicious parameter name specification in Parmaxima option: "<< buf <<endl;
                cerr<<"    If in doubt, use one of the standard names: "<<endl;
                ostream_vector(standardParamNames,cerr);
				cerr<<"I exit."<< endl;
				if (cinGetOnError) cin.get();
				exit(-1);
			}
			goto nextline;
		}
         if(cmp_nocase(var,"anyTwoNmuMin")==0) {
             cerr << "\n (!) anyTwoNmuMin syntaxe trop moche, utiliser ParMinima"<<endl;
             cerr<<"I exit."<< endl;
             if (cinGetOnError) cin.get();
             exit(-1);
             string locstring;
             stringstream strstr(buf.substr(pos+1));
             strstr>>anyTwoNmuMin;
             goto nextline;
         }
         if(cmp_nocase(var,"anyTwoNmuMax")==0) {
             cerr << "\n (!) anyTwoNmuMax syntaxe trop moche, utiliser ParMaxima"<<endl;
             cerr<<"I exit."<< endl;
             if (cinGetOnError) cin.get();
             exit(-1);
             string locstring;
             stringstream strstr(buf.substr(pos+1));
             strstr>>anyTwoNmuMax;
             goto nextline;
         }
         if(cmp_nocase(var,"anyTwoNmMin")==0) {
             cerr << "\n (!) anyTwoNmMin syntaxe trop moche, utiliser ParMinima"<<endl;
             cerr<<"I exit."<< endl;
             if (cinGetOnError) cin.get();
             exit(-1);
             string locstring;
             stringstream strstr(buf.substr(pos+1));
             strstr>>anyTwoNmMin;
             goto nextline;
         }
         if(cmp_nocase(var,"anyTwoNmMax")==0) {
             cerr << "\n (!) anyTwoNmMax syntaxe trop moche, utiliser ParMaxima"<<endl;
             cerr<<"I exit."<< endl;
             if (cinGetOnError) cin.get();
             exit(-1);
             string locstring;
             stringstream strstr(buf.substr(pos+1));
             strstr>>anyTwoNmMax;
             goto nextline;
         }
         if(cmp_nocase(var,"anyTimeMin")==0) {
             cerr << "\n (!) anyTimeMin syntaxe trop moche, utiliser ParMinima"<<endl;
             cerr<<"I exit."<< endl;
             if (cinGetOnError) cin.get();
             exit(-1);
             string locstring;
             stringstream strstr(buf.substr(pos+1));
             strstr>>anyTimeMin;
             goto nextline;
         }
         if(cmp_nocase(var,"anyTimeMax")==0) {
             cerr << "\n (!) anyTimeMax syntaxe trop moche, utiliser ParMaxima"<<endl;
             if (cinGetOnError) cin.get();
             cerr<<"I exit."<< endl;
             exit(-1);
             string locstring;
             stringstream strstr(buf.substr(pos+1));
             strstr>>anyTimeMax;
             goto nextline;
         }
		if(cmp_nocase(var,"TestPoint")==0) {
            string subbuf;
            testParName.resize(0);
            testParValue.resize(0);
            Ctestpoint loctp;
			float value;
			unsigned int eqpos;
			buf=buf.substr(pos+1); // =]...[end
            while (buf.length()>0) { // examen de RHS complet de TestPoint
               pos=0;
			   while (buf.length()>pos && !(buf[pos]==',')) pos++; //cherche , ou eol
			   subbuf=buf.substr(0,pos); //]varname=value[,
               // il analyse la substring de forme par=value
                string::size_type pos2=0;
                while (subbuf.length()>pos2 && !isalnum(subbuf[pos2]) ) pos2++; //skips all non alpha numeric char before first keyword
                subbuf=subbuf.substr(pos2);
               if (inputCheckBool) {cout<<"Parsed: |"<<subbuf<< "|" << endl;}
			   eqpos=0;
			   while (subbuf.length()>eqpos && !(subbuf[eqpos]=='=')) eqpos++; //cherche =
			   testParName.push_back(subbuf.substr(0,eqpos)); //]varname[=
			   string remainder=subbuf.substr(eqpos+1); //=]value[,
			   { stringstream strstr(remainder);
			     strstr>>value; //conversion en nombre
               } //clears strstr
			   testParValue.push_back(value);
			   loctp.names=testParName;  // there may be several names per point...
			   loctp.values=testParValue; // and several values
//ostream_vector(testParName,cout);
//ostream_vector(testParValue,cout);getchar();
			   if (buf.length()>pos) {
				buf=buf.substr(pos+1); //tout le reste apr[`e]s ',' eventuellement
			   	pos=0;
			   } else buf="";
			} // now we're at the end of the TestPoint line
			if(StandardizeName(loctp.names,standardParamNames)!=0) {
				cerr<<"(!) Suspicious parameter name specification in TestPoint option: "<< buf <<endl;
                cerr<<"    If in doubt, use one of the standard names: "<<endl;
                ostream_vector(standardParamNames,cerr);
				cerr<<"I exit."<< endl;
				if (cinGetOnError) cin.get();
				exit(-1);
			}
		   testPointList.push_back(loctp); // pushes back one point = one or more names and values
			goto nextline;
		}
		if(cmp_nocase(var,"1DCI")==0 || cmp_nocase(var,"oneDimCI")==0) {
			string bufSaved="";
            oneDimCIvars.resize(0);
			buf=buf.substr(pos+1);
            bufSaved=buf.c_str();
			pos=0;
            while (buf.length()>0) {
			   while (buf.length()>pos && !isalnum(buf[pos])) pos++;
			   if (buf.length()==pos) goto nextline;
			   buf=buf.substr(pos);
			   pos=0;
			   while (buf.length()>pos && isalnum(buf[pos])) pos++;
			   if(cmp_nocase(buf.substr(0,pos),"Nmu")==0) {
                    cout << "(!) Old 1DCI parameters specification : 'Nmu' converted to 'twoNmu' in 1DCI" << endl;
                    oneDimCIvars.push_back("twoNmu");
                } else if(cmp_nocase(buf.substr(0,pos),"Nanc")==0) {
                    cout << "(!) Old 1DCI parameters specification : 'Nanc' converted to 'twoNancmu' in 1DCI" << endl;
                    oneDimCIvars.push_back("twoNancmu");
                } else oneDimCIvars.push_back(buf.substr(0,pos));
		       if (inputCheckBool) {cout<<"Parsed: |"<<buf.substr(0,pos)<< "|" << endl;}
			   if (buf.length()>pos) {
				buf=buf.substr(pos+1);
			   	pos=0;
			   } else {
                   if( !(oneDimCIvars.size()==1 && cmp_nocase(oneDimCIvars[0],"all")==0) && StandardizeName(oneDimCIvars,standardParamNames)!=0) {
					   cerr<<"(!) Suspicious parameter name specification in 1DCI/oneDimCI option:\n "<< bufSaved <<endl;
                       cerr<<"    If in doubt, use one of the standard names: "<<endl;
                       ostream_vector(standardParamNames,cerr);
					   cerr<<"I exit."<< endl;
					   if (cinGetOnError) cin.get();
					   exit(-1);
				   }
				   goto nextline;
			   }
			}
            if( !(oneDimCIvars.size()==1 && cmp_nocase(oneDimCIvars[0],"all")==0) && StandardizeName(oneDimCIvars,standardParamNames)!=0) {
				cerr<<"(!) Suspicious parameter name specification in 1DCI/oneDimCI option: "<< buf <<endl;
                cerr<<"    If in doubt, use one of the standard names: "<<endl;
                ostream_vector(standardParamNames,cerr);
				cerr<<"I exit."<< endl;
				if (cinGetOnError) cin.get();
				exit(-1);
			}
			goto nextline;
		}
		if(cmp_nocase(var,"CIBoundsLevel")==0
		   || cmp_nocase(var,"CILevel")==0
		   || cmp_nocase(var,"CIErrorLevel")==0) {// value = a low probability !
			if(cmp_nocase(var,"CIBoundsLevel")==0)
                cout << "(!) CIBoundsLevel is obsolete, use CIErrorLevel" << endl;
			if(cmp_nocase(var,"CILevel")==0)
                cout << "(!) CILevel is obsolete, use CIErrorLevel" << endl;
			stringstream strstr(buf.substr(pos+1));
			strstr>>CILevel;
			goto nextline;
		}
		if(cmp_nocase(var,"CICoverageLevel")==0 ) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>CILevel;
            CILevel=1.-CILevel;
			goto nextline;
		}
		if(cmp_nocase(var,"NextBoundsLevel")==0) {// value = a low probability ! // from1DCI, nextpoints...
			stringstream strstr(buf.substr(pos+1));
			strstr>>NextBoundsLevel;
			goto nextline;
		}
		if(cmp_nocase(var,"hullExpandFactor")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>hullExpandFactor;
			goto nextline;
		}
		if(cmp_nocase(var,"designRetain")==0) { // affects Kriging but NOT GCV
			stringstream strstr(buf.substr(pos+1));
			strstr>>designRetain;
			goto nextline;
		}
		if(cmp_nocase(var,"GCVdesignRetain")==0) { // affects GCV but NOT Kriging
			stringstream strstr(buf.substr(pos+1));
			strstr>>GCVdesignRetain;
			goto nextline;
		}
		if(cmp_nocase(var,"scaleFactor")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>scalefactor;
			goto nextline;
		}
		if(cmp_nocase(var,"nloci")==0 || cmp_nocase(var,"nlocus")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>nloci;
			goto nextline;
		}
		if(cmp_nocase(var,"NrunsPerPoint")==0) {
//			stringstream strstr(buf.substr(pos+1));
//			strstr>>NrunsPerPoint;
			sequenceNrunsPerPoint.resize(0);
			int value;
			string reste=buf.substr(pos+1);
			stringstream strstr(reste);
			while (!strstr.eof()) {
				strstr>>value;
/*strstr.get(); to skip any one-character separator which is necessarily here
is not sufficient if there are whitespaces after the last value. If so, eof is not reached
but no new value is read -> the last value is duplicated */
//(((bidon=strstr.peek())<'0') || (bidon>'9'))
                 while (!strstr.eof() && !(isdigit(bidon=strstr.peek()))) strstr.get();
				sequenceNrunsPerPoint.push_back(value);
			}
			strstr.clear(); // c'est le truc essentiel pour le r['e]util... snif
			goto nextline;
		}
		if(cmp_nocase(var,"PointNumber")==0) {
			sequencePointNumber.resize(0);
			int value;
			string reste=buf.substr(pos+1);
			stringstream strstr(reste);
			while (!strstr.eof()) {
				strstr>>value;
/*strstr.get(); to skip any one-character separator which is necessarily here
is not sufficient if there are whitespaces after the last value. If so, eof is not reached
but no new value is read -> the last value is duplicated */
//(((bidon=strstr.peek())<'0') || (bidon>'9'))
                 while (!strstr.eof() && !(isdigit(bidon=strstr.peek()))) strstr.get();
				sequencePointNumber.push_back(value);
			}
			strstr.clear(); // c'est le truc essentiel pour le r['e]util... snif
			goto nextline;
		}
		if(cmp_nocase(var,"point")==0 || cmp_nocase(var,"pointIndex")==0) {
            if(cmp_nocase(var,"point")==0) {
                cout<<"(!) Warning: Point is an obsolete keyword; use pointIndex instead.\n";
                cout<<"    Further execution interprets this as PointIndex.\n";
            }
			stringstream strstr(buf.substr(pos+1));
			strstr>>_pointmin;
			_pointmax=_pointmin;
			singlePointBool=true;  // not this is not the same as ub=lb; with pointIndex a non trivial set of points is sampled but only on is computed
			goto nextline;
		}
		if(cmp_nocase(var,"pointmin")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>_pointmin;
			userPointMinOrMax = true;
			goto nextline;
		}
		if(cmp_nocase(var,"pointmax")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>_pointmax;
			userPointMinOrMax = true;
			goto nextline;
		}
		if(cmp_nocase(var,"ptSamplingSeeds")==0 ||cmp_nocase(var,"ptSamplingSeed")==0 || cmp_nocase(var,"pointSamplingSeed")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>_ptSamplingSeed;
			goto nextline;
		}
		if(cmp_nocase(var,"EstimationSeed")==0 || cmp_nocase(var,"EstimationSeeds")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>_EstimationSeed;
			goto nextline;
		}
		if(cmp_nocase(var,"GeoBinNbr")==0 // the two standard versions are GeoBinNbr and AxialBinNbr
		   || cmp_nocase(var,"AxialBinNbr")==0
		   || cmp_nocase(var,"GeoDistanceBins")==0
		   || cmp_nocase(var,"AxialDistanceBins")==0) {
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>geoBinNbr; //note trick (1D): if negative, only generates gp_dg files with abs(geoBinNbr) bins
 			goto nextline;
		}
		if(cmp_nocase(var,"GeoBinWidth")==0) {
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>geoBinWidth; //note trick (1D): if negative, only generates gp_dg files with abs(geoBinNbr) bins
 			goto nextline;
		}
		if(cmp_nocase(var,"GeoUnit")==0) { // to give the unit of Nb in plots.
            string reste=buf.substr(pos+1);
            if(reste[0]=='(') {reste.erase(0,1);}//vire la parenthèse du début si elle existe
            pos=reste.length();
            if(reste[pos-1]==')') reste.erase(pos-1,1);// vire la parenthèse de la fin si elle existe

            stringstream strstr(reste);
			strstr>>geoUnit;
 			goto nextline;
		}
		if(cmp_nocase(var,"habitatPars")==0) {
			habitatPars.resize(0); // discards default values
			float value;
			string reste=buf.substr(pos+1);
			stringstream strstr(reste);
			while (!strstr.eof()) {
				strstr>>value;
/*strstr.get(); skipping any one-character separator
is not sufficient if there are whitespaces after the last value. If so, eof is not reached
but no new value is read -> the last value is duplicated */
//(((bidon=strstr.peek())<'0') || (bidon>'9'))
                 while (!strstr.eof() && !(isdigit(bidon=strstr.peek())) && bidon!='.' && bidon!='-') strstr.get();
				habitatPars.push_back(value);
			}
            if (habitatPars.size()!=5) {
               cerr<<"habitat setting does not give 5 values.\n I exit.";
               if (cinGetOnError) cin.get();
               exit(-1);
            }
			habitatPars[4]*= (float) ( 3.1415926535/180. ); // conversion from degrees to radians for all further uses.
			goto nextline;
		}
		if(cmp_nocase(var,"GeoAreaMin")==0 || cmp_nocase(var,"PSONmin")==0) {
            if(cmp_nocase(var,"GeoAreaMin")==0) {
                cout<<"(!) Warning: GeoAreaMin is an obsolete keyword; use PSONmin instead.\n";
            }
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>geoAreaMin[0]>>geoAreaMin[1];  // needs space, not simply comma !!! to be improved !!!
            geoAreaMinBool=true;
			goto nextline;
		}
		if(cmp_nocase(var,"GeoAreaMax")==0 || cmp_nocase(var,"PSONmax")==0) {
            if(cmp_nocase(var,"GeoAreaMax")==0) {
                cout<<"(!) Warning: GeoAreaMax is an obsolete keyword; use PSONmax instead.\n";
            }
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>geoAreaMax[0]>>geoAreaMax[1];
            geoAreaMaxBool=true;
			goto nextline;
		}
		if(cmp_nocase(var,"RootGenepopFileName")==0 || cmp_nocase(var,"GenepopRootFileName")==0 || cmp_nocase(var,"GenepopRootName")==0 || cmp_nocase(var,"GenepopRootFile")==0) {
            genepopBool=true;
            genepopRootBool=true;
			stringstream strstr(buf.substr(pos+1));
			strstr>>gp_fileRoot;
			samplerootname="re_dg"; //output to ad hoc format
// attention nloci va etre recalcul['e] d'apr[`e]s les loci informatifs dans crunchGenepopfile
            goto nextline;
		}
		if(cmp_nocase(var,"GenepopFileName")==0 || cmp_nocase(var,"GenepopInputFile")==0 ||cmp_nocase(var,"GenepopFile")==0) {
            genepopBool=true;
            genepopRootBool=false; //FR 02/04/2010
			stringstream strstr(buf.substr(pos+1));
			strstr>>gp_file;
			samplerootname="re_dg"; //output to ad hoc format
// attention nloci va etre recalcul['e] d'apr[`e]s les loci informatifs dans crunchGenepopfile
// beware job numbers are reset:
   			JobMax=JobMin=1; // equivalent to job=1
            goto nextline;
		}
		// CBR: imitating Genepop filename setup
		if (cmp_nocase(var, "RootNexusFileName") == 0 || cmp_nocase(var, "NexusRootFileName") == 0 || cmp_nocase(var, "NexusRootName") == 0 || cmp_nocase(var, "NexusRootFile") == 0) {
			nexusBool = true;
			nexusRootBool = true;
			stringstream strstr(buf.substr(pos + 1));
			strstr >> nexusRootFilename;
			goto nextline;
		}
		if (cmp_nocase(var, "NexusFileName") == 0 || cmp_nocase(var, "NexusInputFile") == 0 || cmp_nocase(var, "NexusFile") == 0) {
			nexusBool = true;
			nexusRootBool = false;
			stringstream strstr(buf.substr(pos + 1));
			strstr >> nexusFilename;
			goto nextline;
		}
         if (cmp_nocase(var, "OutlierGeneFile") == 0) {
             outlierGeneFileBool = true;
             goto nextline;
         }
		if (cmp_nocase(var, "DebugNexusSample") == 0) {
			DebugNexusSample = true;
			goto nextline;
		}
		if (cmp_nocase(var, "PrintSuiviLikelihood") == 0) {
			PrintSuiviLikelihood = true;
			goto nextline;
		}
		if (cmp_nocase(var, "useConstantRcode") == 0) {
            cerr<<"Option useConstantRCode is obsolete. the 'blackbox' R package will always be used" << endl;
            cerr<<"I exit.";
            if (cinGetOnError) cin.get();
            exit(-1);
		}
		if(cmp_nocase(var,"SamplesPosOnArray")==0 || cmp_nocase(var,"SamplesPosOnLattice")==0) {
			samplesPos.resize(0); // prophylactic
			int value;
//			float atuer;
			string reste=buf.substr(pos+1);
			stringstream strstr(reste);
			strstr>>geoBinNbr;
            //to skip any one-character separator which is here
            while (!strstr.eof() && !(isdigit(bidon=strstr.peek())) && bidon!='.') strstr.get();
			while (!strstr.eof()) {
                strstr>>value;
                while (!strstr.eof() && !(isdigit(bidon=strstr.peek())) && bidon!='.') strstr.get();
                if (value>geoBinNbr) {
                   cerr<<"Sample position ("<<value<<") out of lattice range ("<<geoBinNbr;
                   cerr<<").\n Check SamplesPosOnLattice option.\n I exit.";
                   if (cinGetOnError) cin.get();
                   exit(-1);
                } else samplesPos.push_back(value);

            }
//ostream_vector(samplesPos,cout);
			strstr.clear(); // c'est le truc essentiel pour le r['e]util... snif
			goto nextline;
		}
		if(cmp_nocase(var,"SampleRootName")==0 || cmp_nocase(var,"SampleRootFileName")==0 || cmp_nocase(var,"RootFileName")==0) {
			stringstream strstr(buf.substr(pos+1));
			strstr>>samplerootname;
			goto nextline;
		}
		if(cmp_nocase(var,"GivenK")==0 || cmp_nocase(var,"AllelicBounds")==0
           || cmp_nocase(var,"AllelicBound")==0
           || cmp_nocase(var,"GSMBounds")==0 || cmp_nocase(var,"GSMBound")==0
           || cmp_nocase(var,"SMMBounds")==0 || cmp_nocase(var,"SMMBound")==0) {
			sequenceGivenK.resize(0);
			buf = buf.substr(pos + 1);
			string arg;
			pos = 0;
            while (buf.length() > 0) {
			   while (buf.length() > pos && !isalnum(buf[pos])) {
				   if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t'))
					   cerr << "(!) Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
					   	    << filename << "! Ignored." << endl;
				   pos++;
			   }
			   if (buf.length() == pos)
				   goto nextline;
			   buf = buf.substr(pos);
			   pos = 0;
			   while (buf.length() > pos && isalnum(buf[pos]))
				   pos++;
			   arg = buf.substr(0,pos);
			   if (cmp_nocase(arg,"Auto") == 0 || cmp_nocase(arg,"Automatic") == 0 || cmp_nocase(arg,"") == 0)
				   arg = "-1";
			   sequenceGivenK.push_back(atoi(arg.c_str()));
               if (buf.length() > pos) {
    				buf = buf.substr(pos+1);
    			   	pos = 0;
			   } else goto nextline;
			}
			goto nextline;
		}
         if(cmp_nocase(var,"ForceGivenK")==0) {
             //controls uses of processedLoci vs ad hoc files through two differetn versions of GetSample()
             ForceGivenKbool=false;
             evaluateBool(ForceGivenKbool,buf.substr(pos+1));
             goto nextline;
         }

		if(cmp_nocase(var,"AllelicStates")==0) {
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>locstring;
			if(cmp_nocase(locstring,"AllStates")==0) allowsEmptyColumns=true; // allows alleles with null counts
			else if(cmp_nocase(locstring,"NonEmptyStates")==0) allowsEmptyColumns=false; //default
					else { //non d['e]plac['e] donc pas de v['e]rification de la syntaxe, par default allowsEmptyColumns=false
						cout << "(!) The AllelicStates option specification in" << filename << " is incorrect;" << endl;
						cout << "(!) Only AllStates and NonEmptyStates are implemented in the current version"<<endl;
                        cout<<"I exit."<< endl;
					    if (cinGetOnError) cin.get();
					    exit(-1);
				}
			goto nextline;
		}
		if(cmp_nocase(var,"MutationalModel")==0 || cmp_nocase(var,"MutationModel")==0) {
			mutationModelVector.resize(0);
			markerCount = 0;
			buf = buf.substr(pos + 1);
			string arg;
			pos = 0;
            while (buf.length() > 0) {
			   while (buf.length() > pos && !isalnum(buf[pos])) {
				   if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t'))
					   cerr << "(!) Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
					   	    << filename << "! Ignored." << endl;
				   pos++;
			   }
			   if (buf.length() == pos)
				   goto nextline;
			   buf = buf.substr(pos);
			   pos = 0;
			   while (buf.length() > pos && isalnum(buf[pos]))
				   pos++;
			   arg = buf.substr(0,pos);
			   if ((cmp_nocase(arg,"PIM") == 0) || (cmp_nocase(arg,"KAM") == 0)) {
				   mutationModelVector.push_back("PIM");
				   mutModelBool[PIM] = true;
				   ++markerCount;
			   }
			   else if (cmp_nocase(arg,"SMM") == 0) {
				   mutationModelVector.push_back("SMM");
				   mutModelBool[SMM] = true;
				   ++markerCount;
			   }
			   else if (cmp_nocase(arg,"GSM") == 0) {
				   mutationModelVector.push_back("GSM");
				   mutModelBool[GSM] = true;
				   ++markerCount;
			   }
			   else if (cmp_nocase(arg,"ISM") == 0) {
				   mutationModelVector.push_back("ISM");
				   mutModelBool[ISM] = true;
				   ++markerCount;
			   }
			   else if (cmp_nocase(arg,"SNP") == 0) {
				   mutationModelVector.push_back("SNP");
				   mutModelBool[SNP] = true;
				   ++markerCount;
                   nexusNumberLabels = true;
			   }
			   else {
				   cout << "(!) Mutational Model specification in " << filename << " is incorrect;" << endl;
				   cout << "(!) Only PIM (KAM), SMM, GSM, ISM and SNP are implemented in the current version" << endl;
				   cerr << "\nAborting Migraine...Press any key to exit.\n";
				   if (cinGetOnError)
					   cin.get();
				   exit(-1);
			   }
               if (buf.length() > pos) {
    				buf = buf.substr(pos+1);
    			   	pos = 0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase(var,"LociPerModel")==0 || cmp_nocase(var,"LociPerMarker")==0
			|| cmp_nocase(var,"LocusPerModel")==0 || cmp_nocase(var,"LocusPerMarker")==0) {
			lociPerMarker.resize(0);
			totalLociCount = 0;
			buf = buf.substr(pos + 1);
			string arg;
			pos = 0;
            while (buf.length() > 0) {
			   while (buf.length() > pos && !isdigit(buf[pos])) {
				   if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t'))
					   cerr << "(!) Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
					   	    << filename << "! Ignored." << endl;
				   pos++;
			   }
			   if (buf.length() == pos)
				   goto nextline;
			   buf = buf.substr(pos);
			   pos = 0;
			   while (buf.length() > pos && isdigit(buf[pos]))
				   pos++;
			   arg = buf.substr(0,pos);
			   lociPerMarker.push_back(atoi(arg.c_str()));
			   totalLociCount += lociPerMarker.back();
               if (buf.length() > pos) {
    				buf = buf.substr(pos+1);
    			   	pos = 0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase(var,"SMMstepSizes")==0 || cmp_nocase(var,"SMMstepSize")==0
		   || cmp_nocase(var,"SMMstep")==0 || cmp_nocase(var,"SMMsteps")==0
		   || cmp_nocase(var,"StepSize")==0 || cmp_nocase(var,"StepSizes")==0) {
			SMMstepSizes.resize(0);
			buf = buf.substr(pos + 1);
			string arg;
			pos = 0;
            while (buf.length() > 0) {
			   while (buf.length() > pos && !isdigit(buf[pos])) {
				   if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t'))
					   cerr << "(!) Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
					   	    << filename << "! Ignored." << endl;
				   pos++;
			   }
			   if (buf.length() == pos)
				   goto nextline;
			   buf = buf.substr(pos);
			   pos = 0;
			   while (buf.length() > pos && isdigit(buf[pos]))
				   pos++;
			   arg = buf.substr(0,pos);
			   SMMstepSizes.push_back(atoi(arg.c_str()));
               if (buf.length() > pos) {
    				buf = buf.substr(pos+1);
    			   	pos = 0;
			   } else goto nextline;
			}
			goto nextline;
		}
/* OBSOLETE			if(cmp_nocase(var,"Statistic")==0) {
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>locstring;
			if(cmp_nocase(locstring,"PAC")==0) {
				if (asymptoBool) {
					cout<<"(!) Option PAC-likelihood incompatible with ancestral approximation.\nPAC will be ignored"<<endl;
					goto nextline;
				} //else
				pacbool=true;
			}
			if(cmp_nocase(locstring,"Lik_IS")==0 || cmp_nocase(locstring,"IS")==0) pacbool=false;
			goto nextline;
		} OBSOLETE */
		if(cmp_nocase(var,"StatisticSequence")==0 || cmp_nocase(var,"Statistic")==0) {
			string subbuf,prevbuf;
			sequenceStat.resize(0);
			buf=buf.substr(pos+1);
//			seqstat=buf; // sauvegarde pour l'affichage
			pos=0;
            while (buf.length()>0) {
			   while (buf.length()>pos && !isalnum(buf[pos])) pos++;
			   if (buf.length()==pos) goto nextline;
			   buf=buf.substr(pos);
			   pos=0;
			   while (buf.length()>pos && isalnum(buf[pos])) pos++;
			   subbuf=buf.substr(0,pos);
			   if(cmp_nocase(subbuf,"PAC")==0 || cmp_nocase(subbuf,"PAC2id")==0) {
				   PACinSeqBool=true; // sert [`a] ['e]viter les conflits avec d'autres options
			   } // and in all cases where !asymptoBool:
               else if(cmp_nocase(subbuf,"PACanc")==0 || cmp_nocase(subbuf,"PACanc2id")==0) {
				   PACancinSeqBool=true;
			   } else if(cmp_nocase(subbuf,"ISstrict")==0 || cmp_nocase(subbuf,"strictIS")==0) {
				   subbuf="ISstrict";
			   } else if(cmp_nocase(subbuf,"IS")!=0) {
				   cout<<"(!) StatisticSequence specification in "<<filename<<" is " << subbuf << " but is incorrect;"<<endl;
				   cout<<"(!) Only 'IS', 'ISstrict', 'PAC', 'PACanc' and 'PACanc2id' are implemented in the current version"<<endl;
				   cout<<"I exit."<< endl;
				   if (cinGetOnError) cin.get();
				   exit(-1);
			   }
               if (isdigit(subbuf[0])) {
                   int locit;
                   stringstream stst;
                   stst<<subbuf;
                   stst>>locit;
                   for (int ii=0;ii<locit-1;ii++) sequenceStat.push_back(prevbuf);
               } else sequenceStat.push_back(subbuf);
//cout<<sequenceStat[sequenceStat.size()-1];getchar();
			   if (buf.length()>pos) {
			    prevbuf=subbuf;
				buf=buf.substr(pos+1);
			   	pos=0;
			   } else goto nextline;
			}
			goto nextline;
		}
		 if(cmp_nocase(var,"AlternativeIS")==0 || cmp_nocase(var,"AlternatifIS")==0) {
			 string locstring;
			 stringstream strstr(buf.substr(pos+1));
			 strstr>>locstring;
			 if(cmp_nocase(locstring,"AncPopSize")==0) AlternativeIS_AncPopSizebool=true;
			 else if(cmp_nocase(locstring,"ExpMixAncActPopSize")==0) AlternativeIS_FRExpMixAncActPopSizebool=true;
			 else { cout<<"(!) AlternativeIS specification in "<<filename<<" is incorrect;"<<endl;
					cout<<"(!) Only AncPopSize and ExpMixAncActPopSize is implemented in the current version for the One Pop var size model"<<endl;
					cout<<"Check your setting file. I exit."<< endl;
					if (cinGetOnError) cin.get();
					exit(-1);
				 }
			 goto nextline;
		 }
		if(cmp_nocase(var,"HobolthProposal")==0) {
				stringstream strstr(buf.substr(pos + 1));
				evaluateBool(HobolthISM, buf.substr(pos + 1));
				goto nextline;
		}
		if(cmp_nocase(var,"NexusTaxonLabels")==0) {
				string locstring;
				stringstream strstr(buf.substr(pos + 1));
				strstr >> locstring;
				if (cmp_nocase(locstring, "Numerical") == 0 || cmp_nocase(locstring, "Numbered") == 0 || cmp_nocase(locstring, "Number") == 0)
					nexusNumberLabels = true;
				else if (cmp_nocase(locstring, "other") == 0)
					nexusNumberLabels = false;
				else {
					cerr << "(!) Unrecognised specification for NexusTaxonLabels in " << filename << endl;
					cerr << "\nAborting Migraine...Press any key to exit.\n";
					if (cinGetOnError)
						cin.get();
					exit(-1);
				}
				goto nextline;
		}
		if(cmp_nocase(var,"GenepopIndividualLabels")==0) {
				string locstring;
				stringstream strstr(buf.substr(pos + 1));
				strstr >> locstring;
				if (cmp_nocase(locstring, "NexusLabel") == 0)
					sameGpNexusLabels = true;
				else if (cmp_nocase(locstring, "Other") == 0)
					sameGpNexusLabels = false;
				else {
					cerr << "(!) Unrecognised specification for GenepopIndividualLabels in " << filename << endl;
					cerr << "\nAborting Migraine...Press any key to exit.\n";
					if (cinGetOnError)
						cin.get();
					exit(-1);
				}
				goto nextline;
		}
		if(cmp_nocase(var,"PromptForHyperSegSites")==0) {
				evaluateBool(promptHyperSegSites, buf.substr(pos + 1));
				goto nextline;
		}
		if(cmp_nocase(var,"ResolveForPerfectPhylogeny")==0) {
				resolveForPerfectPhylogeny = "manual";
				string locstring;
				stringstream strstr(buf.substr(pos + 1));
				strstr >> locstring;
				if (cmp_nocase(locstring, "manual") == 0)
					resolveForPerfectPhylogeny = "manual";
				else if (cmp_nocase(locstring, "sites") == 0
							|| cmp_nocase(locstring, "nucleotides") == 0)
					resolveForPerfectPhylogeny = "sites";
				else if (cmp_nocase(locstring, "haps") == 0
							|| cmp_nocase(locstring, "haplotypes") == 0)
					resolveForPerfectPhylogeny = "haps";
				else if (cmp_nocase(locstring, "auto") == 0
							|| cmp_nocase(locstring, "automatic") == 0)
					resolveForPerfectPhylogeny = "auto";
				else {
					cerr << "(!) Unrecognised specification for ResolveForPerfectPhylogeny in " << filename << endl;
					cerr << "\nAborting Migraine...Press any key to exit.\n";
					if (cinGetOnError)
						cin.get();
					exit(-1);
				}
				goto nextline;
		}
		if(cmp_nocase(var,"AncestralScaledTimes")==0 || cmp_nocase(var,"AncestralScaledTime")==0) {
            // times of events described by next keyword
			// generations times 2 \Ntot (2* 2* Nhaploid* ndemes for diploids)
            ancestralScaledTimes.resize(0); // discards default values
			float value;
			string reste=buf.substr(pos+1);
			stringstream strstr(reste);
			while (!strstr.eof()) {
				strstr>>value;
                 while (!strstr.eof() && !(isdigit(bidon=strstr.peek())) && bidon!='.' && bidon!='-') strstr.get();
				ancestralScaledTimes.push_back(value);
//ostream_vector(ancestralScaledTimes,cout);getchar();
			}
			strstr.clear(); // c'est le truc essentiel pour le r['e]util... snif
			goto nextline;
		}
		if(cmp_nocase(var,"ancestralSuffixes")==0 || cmp_nocase(var,"AncestralSuffix")==0) {
            // describes a sets of events to be taken into account
			ancestralSuffixes.resize(0);
			buf=buf.substr(pos+1);
//			seqstat=buf; // sauvegarde pour l'affichage
			pos=0;
            while (buf.length()>0) {
			   while (buf.length()>pos && !isalnum(buf[pos])&& !(buf[pos]=='_')) pos++;
			   if (buf.length()==pos) goto nextline;
			   buf=buf.substr(pos);
			   pos=0;
			   while (buf.length()>pos && (isalnum(buf[pos]) || (buf[pos]=='_'))) pos++;
			   ancestralSuffixes.push_back(buf.substr(0,pos));
//ostream_vector(ancestralSuffixes,cout);getchar();
			   if (buf.length()>pos) {
				buf=buf.substr(pos+1);
			   	pos=0;
			   } else goto nextline;
			}
			goto nextline;
		}
		if(cmp_nocase(var,"TmrcaComputation")==0 || cmp_nocase(var,"TmrcaComputations")==0) {
			//string locstring;
			stringstream strstr(buf.substr(pos+1));
			if (isdigit(bidon=strstr.peek()) || bidon=='.' || bidon=='-') {// if is number
			    TMRCABool=true;
            } else evaluateBool(TMRCABool,buf.substr(pos+1));
			goto nextline;
		}
		if(cmp_nocase(var,"PiSolver")==0 || cmp_nocase(var,"LinearSolver")==0) {// kept LinearSolver for backward compatibility
            if(cmp_nocase(var,"LinearSolver")==0) {
                cout<<"(!) Warning: LinearSolver is an obsolete keyword; use PiSolver instead.\n";
                cout<<"    Execution will nevertheless proceed as intended.\n";
            }
			stringstream strstr(buf.substr(pos+1));
			strstr>>PiSolverFromSettingsInfo;
			goto nextline;
		}
         if(cmp_nocase(var,"PiOnePopSolver")==0 || cmp_nocase(var,"PiSolverOnePop")==0) {
             piOnePopSolverVector.resize(0);
             buf = buf.substr(pos + 1);
             string arg;
             pos = 0;
             while (buf.length() > 0) {
                 while (buf.length() > pos && !isalnum(buf[pos])) {
                     if ((buf[pos] != ' ') && (buf[pos] != ',') && (buf[pos] != '\t'))
                         cerr << "(!) Unidentified specification \"" << buf[pos] << "\" for \"" << var << "\" in "
                         << filename << "! Ignored." << endl;
                     pos++;
                 }
                 if (buf.length() == pos)
                     goto nextline;
                 buf = buf.substr(pos);
                 pos = 0;
                 while (buf.length() > pos && isalnum(buf[pos]))
                     pos++;
                 arg = buf.substr(0,pos);
                 if ((cmp_nocase(arg,"DG") == 0)) {
                     piOnePopSolverVector.push_back("DG");
                     piOnePopSolverBool[DG] = true;
                 }
                 else if (cmp_nocase(arg,"SD") == 0) {
                     piOnePopSolverVector.push_back("SD");
                     piOnePopSolverBool[SD] = true;
                 }
                 else if (cmp_nocase(arg,"FRRL") == 0 || cmp_nocase(arg,"Eigen") == 0 || cmp_nocase(arg,"EigenValue") == 0 || cmp_nocase(arg,"EigenValues") == 0) {
                     piOnePopSolverVector.push_back("Eigen");
                     piOnePopSolverBool[Eigen] = true;
                 }
                 else {
                     cout<<"(!) Option for PiOnePopSolver in migraine.txt is misspecified." << endl;
                     cout << "\n (!)SD, DG or Eigen are the only implemented options. I exit."<<endl;
                     if (cinGetOnError) cin.get();
                     exit(-1);
                 }
                 if (buf.length() > pos) {
                     buf = buf.substr(pos+1);
                     pos = 0;
                 } else goto nextline;
             }
             goto nextline;
         }

        if(cmp_nocase(var,"RtermFullPath")==0) {
			Rtermfullpath=buf.substr(pos+1); //str pour aller de stringstream a string...
			ifstream zut(Rtermfullpath.c_str(),ios::in);
			if (!zut.is_open()) {
				cerr<<"\n\a\aRterm.exe not accessible. Calls to R will be ignored.";
				cerr<<"\nPath was read as "<<Rtermfullpath;
				Rtermfullpath=""; //because test on its length in hypercube.h
				zut.clear();
			}
			goto nextline;
		}
		if(cmp_nocase(var,"ConfidenceRegions")==0 || cmp_nocase(var,"ConfidenceRegion")==0) {
            cerr<<"ConfidenceRegions is obsolete";
            cerr<<"I exit."<< endl;
            if (cinGetOnError) cin.get();
            exit(-1);
			goto nextline;
		}
		if(cmp_nocase(var,"Plots")==0 || cmp_nocase(var,"Plot")==0) {
			buf=buf.substr(pos+1);
			pos=0;
            while (buf.length()>0) {
                while (buf.length()>pos && !isalnum(buf[pos])) pos++;
                if (buf.length()==pos) goto nextline;
                buf=buf.substr(pos);
                pos=0;
                while (buf.length()>pos && isalnum(buf[pos])) pos++;
                if (cmp_nocase(buf.substr(0,pos),"3D")==0 || cmp_nocase(buf.substr(0,pos),"3Dplot")==0 ||
                    cmp_nocase(buf.substr(0,pos),"3Dplots")==0) {
                    PlotOptions.push_back("3Dplots");
                } else if (cmp_nocase(buf.substr(0,pos),"BW")==0 || cmp_nocase(buf.substr(0,pos),"BWPlots")==0 ||
                    cmp_nocase(buf.substr(0,pos),"BWPlot")==0 || cmp_nocase(buf.substr(0,pos),"B&W")==0 ||
                    cmp_nocase(buf.substr(0,pos),"B&WPlots")==0 || cmp_nocase(buf.substr(0,pos),"B&WPlot")==0) {
                    PlotOptions.push_back("BWPlots");
                } else if (cmp_nocase(buf.substr(0,pos),"Incautious")==0) {
                    PlotOptions.push_back("Incautious");
                } else if (cmp_nocase(buf.substr(0,pos),"Cautious")==0) {
                    PlotOptions.push_back("Cautious");
                } else if (cmp_nocase(buf.substr(0,pos),"All1DProfiles")==0 or cmp_nocase(buf.substr(0,9),"All1DProf")==0) {
                    PlotOptions.push_back("All1DProfiles");
                } else if (cmp_nocase(buf.substr(0,pos),"All2DProfiles")==0 or cmp_nocase(buf.substr(0,9),"All2DProf")==0) {
                    PlotOptions.push_back("All2DProfiles");
                } else if (cmp_nocase(buf.substr(0,pos),"AllProfiles")==0 or cmp_nocase(buf.substr(0,7),"AllProf")==0) {
                    PlotOptions.push_back("All1DProfiles");PlotOptions.push_back("All2DProfiles");
                } else if (cmp_nocase(buf.substr(0,pos),"NoProfile")==0 or cmp_nocase(buf.substr(0,pos),"NoProfiles")==0 or cmp_nocase(buf.substr(0,7),"NoProf")==0) {
                    PlotOptions.push_back("NoProfiles");
                }
                else if (cmp_nocase(buf.substr(0,pos),"NoSurfaces")==0 or cmp_nocase(buf.substr(0,pos),"NoSurface")==0 or cmp_nocase(buf.substr(0,pos),"NoSurf")==0) {
                    PlotOptions.push_back("NoSurfaces");
                } else if (cmp_nocase(buf.substr(0,pos),"AllSurfaces")==0 or cmp_nocase(buf.substr(0,pos),"AllSurface")==0 or cmp_nocase(buf.substr(0,pos),"AllSurf")==0) {
                    PlotOptions.push_back("AllSurfaces");
                } else if (cmp_nocase(buf.substr(0,pos),"1DProfiles")==0 or cmp_nocase(buf.substr(0,pos),"1DProfile")==0 or cmp_nocase(buf.substr(0,pos),"1DProf")==0 or
                    cmp_nocase(buf.substr(0,pos),"2DProfiles")==0 or cmp_nocase(buf.substr(0,pos),"2DProfile")==0 or cmp_nocase(buf.substr(0,pos),"2DProf")==0) {
                    cerr<<"(!) Keywords 1DProfiles and 2Dprofiles could not be used as argument for the 'Plots' settings anymore.";
                    cerr<<"(!) They should be used as a settings to select specific 1D and 2D profiles that will be computed,";
                    cerr<<"(!) with the following format 1Dprofiles=ParamA,ParamB,ParamC,... and 2DProfiles=(ParamX,ParamY),(ParamZ,ParamU),..." << endl;
                    cerr<<"(!) Or use the keywords All1DProfiles, All2Dprofiles or AllProfiles as argument for the 'Plots'.";
                    cerr<<"I exit."<< endl;
                    if (cinGetOnError) cin.get();
                    exit(-1);
                } else {
                    cerr<<"(!) Unknown keyword " << buf.substr(0,pos) << " used as argument for the 'Plots' settings.";
                    cerr<<"I exit."<< endl;
                    if (cinGetOnError) cin.get();
                    exit(-1);

                }
                    // voir a la fin de apply_final_settings pour les defaults en OnePop, Npop, IM, OnePopVarSize et OnePopFounderFlush
                if (buf.length()>pos) {
                    buf=buf.substr(pos+1);
                    pos=0;
                } else goto nextline;
            }
            goto nextline;
        }
        if(cmp_nocase(var,"3Dplots")==0 || cmp_nocase(var,"3Dplot")==0) {
            cerr<<"(!) 3Dplots is obsolete, use Plots=...3D...";
            cerr<<"I exit."<< endl;
			if (cinGetOnError) cin.get();
			exit(-1);
		}
		if(cmp_nocase(var,"B&WPlots")==0 || cmp_nocase(var,"B&WPlot")==0) {
			cerr<<"(!) B&Wplots is obsolete, use Plots=...BW...";
            cerr<<"I exit."<< endl;
            if (cinGetOnError) cin.get();
			exit(-1);
		}
         if(cmp_nocase(var,"1DProfiles")==0 || cmp_nocase(var,"1DProfile")==0 || cmp_nocase(var,"1DProf")==0) {
             spec1DProfiles.resize(0);
             buf=buf.substr(pos+1);
             pos=0;
             while (buf.length()>0) {
                 while (buf.length()>pos && !isalnum(buf[pos])) pos++;
                 if (buf.length()==pos) goto nextline;
                 buf=buf.substr(pos);
                 pos=0;
                 while (buf.length()>pos && isalnum(buf[pos])) pos++;
                 if(cmp_nocase(buf.substr(0,pos),"Nmu")==0) {
                     cout << "(!) Old 1DProfiles parameters specification : 'Nmu' converted to 'twoNmu' in 1DProfiles" << endl;
                     spec1DProfiles.push_back("twoNmu");
                 } else if(cmp_nocase(buf.substr(0,pos),"Nanc")==0) {
                     cout << "(!) Old 1DProfiles parameters specification : 'Nanc' converted to 'twoNancmu' in 1DProfiles" << endl;
                     spec1DProfiles.push_back("twoNancmu");
                 } else if(cmp_nocase(buf.substr(0,pos),"Nfounder")==0) {
                     cout << "(!) Old 1DProfiles parameters specification : 'Nfounder' converted to 'twoNfoundermu' in 1DProfiles" << endl;
                     spec1DProfiles.push_back("twoNfoundermu");
                 } else spec1DProfiles.push_back(buf.substr(0,pos));
                 // voir a la fin de apply_final_settings pour les defaults en OnePop, Npop, IM, OnePopVarSize et OnePopFounderFlush
                 if (inputCheckBool) {cout<<"Parsed: |"<<buf.substr(0,pos)<< "|" << endl;}
                 if (buf.length()>pos) {
                     buf=buf.substr(pos+1);
                     pos=0;
                 } else {
                     if( StandardizeName(spec1DProfiles,standardParamNames)!=0) {
                         cerr<<"(!) Suspicious parameter name specification in 1DProfiles option: "<< buf <<endl;
                         cerr<<"    If in doubt, use one of the standard names: "<<endl;
                         ostream_vector(standardParamNames,cerr);
                         cerr<<"I exit."<< endl;
                         if (cinGetOnError) cin.get();
                         exit(-1);
                     }
                     goto nextline;
                 }
             }
             if( StandardizeName(spec1DProfiles,standardParamNames)!=0) {
                 cerr<<"(!) Suspicious parameter name specification in 1DProfiles option: "<< buf <<endl;
                 cerr<<"    If in doubt, use one of the standard names: "<<endl;
                 ostream_vector(standardParamNames,cerr);
                 cerr<<"I exit."<< endl;
                 if (cinGetOnError) cin.get();
                 exit(-1);
             }
             goto nextline;
         }

         if(cmp_nocase(var,"2DProfiles")==0 || cmp_nocase(var,"2DProfile")==0 || cmp_nocase(var,"2DProf")==0) {
             spec2DProfiles.resize(0);
             string subbuf,subbuf1,subbuf2;
             string::size_type pos2;

             buf=buf.substr(pos+1);
             while (buf.length()>0) { // lecture complete de la ligne 2DProfiles
                 pos=0;
                 while (buf.length()>pos && buf[pos]!=')') pos++; //cherche ) ou on est a la fin de la ligne
                 if (buf.length()==pos) goto nextline; //RL verifier l'utilite de cette ligne
                 subbuf=buf.substr(0,pos);
                 while (subbuf[0]==' ') subbuf.erase(0,1); // vire les blanc apres la parenthese
                 if (subbuf[0]!='(') {
                     cerr << "(!) Suspicious parameter pair specification in 2DProfiles option." << endl;
                     cerr << "(!) Migraine did not find the starting ( in the following text: " << buf.substr(1,pos+1) << endl;
                     cerr << "(!) You have to use the following format 2DProfiles=(ParamX,ParamY),(ParamZ,ParamU),..." << endl;
                     cerr<<"I exit."<< endl;
                     if (cinGetOnError) cin.get();
                     exit(-1);
                 }
                 vector<string> profParamNames;
                 subbuf.erase(0,1); //(] Param1, Param2 [),
                 if (inputCheckBool) {cout<<"Parsed: |"<<subbuf<< "|" << endl;}
                 
                 while (subbuf[0]==' ') subbuf.erase(0,1); // vire les blanc en debut de chaine
                 // il analyse la substring de forme Param1,Param2 ou Param1 Param2
                 pos2=0;
                 while (subbuf.length()>pos2 && !( subbuf[pos2]==',' || subbuf[pos2]==';' || subbuf[pos2]==' ') ) pos2++; //cherche une , ou ; ou un espace
                 subbuf1=subbuf.substr(0,pos2);//]Param1[,Param2
                 profParamNames.push_back(subbuf1);
                 if (inputCheckBool) {cout<<"Parsed: |"<<subbuf1<< "|" << endl;}

                 subbuf=subbuf.substr(pos2+1,pos); //(Param1,]Param2 [),
                 while (subbuf[0]==' ' || subbuf[0]==',' || subbuf[0]==';') subbuf.erase(0,1); // vire les blanc, les virgules ou les ; en début de chaine
                 // il analyse la substring de forme Param2 avec potentiellement des espaces en fin
                 pos2=subbuf.length();
                 while (subbuf[pos2-1]==' ' && pos2>0) pos2--; //vire les espaces finaux
                 subbuf2=subbuf.substr(0,pos2);//]Param1[,Param2
                 profParamNames.push_back(subbuf2);
                 if (inputCheckBool) {cout<<"Parsed: |"<<subbuf2<< "|" << endl;}
                 
                 if( StandardizeName(profParamNames,standardParamNames)!=0) {
                     cerr<<"(!) Suspicious parameter name specification in 2DProfiles option: "<< buf <<endl;
                     cerr<<"    If in doubt, use one of the standard names: "<<endl;
                     ostream_vector(standardParamNames,cerr);
                     cerr<<"\nI exit."<< endl;
                     if (cinGetOnError) cin.get();
                     exit(-1);
                 }
                 spec2DProfiles.push_back('('+profParamNames[0]+','+profParamNames[1]+')');
                 
                 if (buf.length()>pos) {
                     buf=buf.substr(pos+1); //tout le reste apr[`e]s ')' eventuellement
                     while (buf[0]==' ' || buf[0]==',' || buf[0]==';') buf.erase(0,1); // vire les blanc, les virgules ou les ; en début de chaine
                     pos=0;
                 } else goto nextline;
             }// now we're at the end of the 2DProfiles line
             //ostream_vector(spec2DProfiles, cout);
             goto nextline;
         }
		if(cmp_nocase(var,"KrigLoci")==0 || cmp_nocase(var,"KrigLocus")==0) { //selection of loci for Kriging
			KrigLoci.resize(0); // prophylactic
			unsigned int value;
			string reste=buf.substr(pos+1);
			stringstream strstr(reste);
			while (!strstr.eof()) {
//strstr.get(); to skip any one-character separator which is here
                while (!strstr.eof() && !(isdigit(bidon=strstr.peek())) && bidon!='.') strstr.get();
				strstr>>value;
                KrigLoci.push_back(value);
			}
			strstr.clear(); // c'est le truc essentiel pour le r['e]util... snif
			goto nextline;
		}
		if(cmp_nocase(var,"Rarguments")==0 || cmp_nocase(var,"Rargument")==0) { /// most of these will be copied in GP$options
			buf=buf.substr(pos+1);
			string arg;
			pos=0;
            while (buf.length()>0) {
			   while (buf.length()>pos && !isalnum(buf[pos])) pos++;
			   if (buf.length()==pos) goto nextline;
			   buf=buf.substr(pos);
			   pos=0;
			   while (buf.length()>pos && isalnum(buf[pos])) pos++;
			   arg=buf.substr(0,pos);
			   /*** dont forget else if on each line ***/
			   if (cmp_nocase(arg,"DontUseC")==0) {useCbool=false;use_Rpackage=false;} // neither Rmigraine nor CKrigcoefs
			   else if (cmp_nocase(arg,"useRmigraine")==0 || cmp_nocase(arg,"useRpackage")==0) {use_Rpackage=true;useCbool=true;} //
			   else if (cmp_nocase(arg,"DontUseRmigraine")==0 || cmp_nocase(arg,"DontUseRpackage")==0 ||
                        cmp_nocase(arg,"DonotUseRmigraine")==0 || cmp_nocase(arg,"DonotUseRpackage")==0) {use_Rpackage=false;useCbool=true;} // but stil use CKrigcoefs by default
			   else if (cmp_nocase(arg,"alwaysRational")==0) {Rarguments.push_back("alwaysRational");} //
			   else if (cmp_nocase(arg,"alwaysDouble")==0) {Rarguments.push_back("alwaysDouble");} //
			   else if (cmp_nocase(arg,"noElim")==0) {Rarguments.push_back("noElim");} //
			   else if (cmp_nocase(arg,"defaultPrecision")==0) {Rarguments.push_back("defaultPrecision");} // v
			   else if (cmp_nocase(arg,"automatedCleaning")==0) {Rarguments.push_back("automatedCleaning");} //
			   else if (cmp_nocase(arg,"emergencyCleaning")==0) {Rarguments.push_back("emergencyCleaning");} //
			   else if (cmp_nocase(arg,"savePredictor")==0) {Rarguments.push_back("savePredictor");} //FR->FR a documenter dans DOC
			   else if (cmp_nocase(arg,"extrapolateSurfaces")==0) {extrapTrueBool=true;} // push to Rarguments in apply_final settings
			   else if (cmp_nocase(arg,"DoNotOptimizeKriging")==0) {optimizeKrigingBool=false;} // push to Rarguments in apply_final settings (CovFnParam also affects the boolean)
			   else if (cmp_nocase(arg,"HGLM")==0) {Rarguments.push_back("HGLM");} //
			   else if (cmp_nocase(arg,"REML")==0) {Rarguments.push_back("HGLM");} //
			   else if (cmp_nocase(arg,"massBarycenter")==0) {Rarguments.push_back("massBarycenter");} //
			   else if (cmp_nocase(arg,"nloptLdMmaForCI")==0) {Rarguments.push_back("NLOPT_LD_MMA_for_CI");} //
               else {
                   cerr<<"Unknown Rargument '"<<arg<<"'. Check settings. I exit."<<endl;
                   if (cinGetOnError) cin.get();
                   exit(-1);
                }
               if (buf.length()>pos) {
    				buf=buf.substr(pos+1);
    			   	pos=0;
			   } else goto nextline;
			}
			goto nextline;
		}

         //PCL_SISR specifications
         if(cmp_nocase(var,"UsePCL_SISR")==0 || cmp_nocase(var,"UseSISR")==0) {
             //controls uses of processedLoci vs ad hoc files through two differetn versions of GetSample()
             UsePCL_SISRbool=false;
             evaluateBool(UsePCL_SISRbool,buf.substr(pos+1));
             goto nextline;
         }
         if(cmp_nocase(var,"PCL_SISR_UseIndivNbResampling")==0 || cmp_nocase(var,"SISR_CoaEventsOnly")==0) {
           //stringstream strstr(buf.substr(pos+1));
           //strstr>>PCL_SISR_UseIndivNbResampling;
           PCL_SISR_UseIndivNbResamplingbool=false;
           evaluateBool(PCL_SISR_UseIndivNbResamplingbool,buf.substr(pos+1));
           goto nextline;
         } // CM 25.03.14

         if(cmp_nocase(var,"PCL_SISR_IndivNbResampling")==0 || cmp_nocase(var,"SISR_EventsNbBetweenResampling")==0) {
           stringstream strstr(buf.substr(pos+1));
           strstr>>PCL_SISR_IndivNbResampling;
             PCL_SISR_StartESSAtEvent=PCL_SISR_IndivNbResampling;
           goto nextline;
         } // CM 25.03.14

         if(cmp_nocase(var,"PCL_SISR_ESSMinFall")==0 || cmp_nocase(var,"SISR_ESSMinDecrease")==0) {
            stringstream strstr(buf.substr(pos+1));
            strstr>>PCL_SISR_ESSMinFall;
            goto nextline;
         } // CM 18.02.14


         if(cmp_nocase(var,"PCL_SISR_MaxEventNbInPopAnc")==0) {
            stringstream strstr(buf.substr(pos+1));
            strstr>>PCL_SISR_MaxEventNbInPopAnc;
            goto nextline;
         } // CM 18.02.14

         if(cmp_nocase(var,"PCL_SISR_StartESSAtEvent")==0) {
             stringstream strstr(buf.substr(pos+1));
             strstr>>PCL_SISR_StartESSAtEvent;
             cout << "(!) Keyword PCL_SISR_StartESSAtEvent is obsolete, use SISR_EventsNbBetweenResampling instead." << endl;
             goto nextline;
         }
         if(cmp_nocase(var,"PCL_SISR_Alpha")==0 || cmp_nocase(var,"SISR_Alpha")==0) {
             stringstream strstr(buf.substr(pos+1));
             strstr>>PCL_SISR_Alpha;
             goto nextline;
         }
         if(cmp_nocase(var,"PCL_SISR_Beta")==0 || cmp_nocase(var,"SISR_Beta")==0) {
             stringstream strstr(buf.substr(pos+1));
             strstr>>PCL_SISR_Beta;
             goto nextline;
         }

		if(cmp_nocase(var,"cmdlinefilename")==0) {
// meaningful on command line only (and processed at the beginning of main()), but written in <cmdline> text file where it should be ignored
            if (lineCounter>1) {
                cerr<<"(!)(!) Cmdlinefilename setting not first argument. Anything could happen! I exit.";
                if(cinGetOnError) cin.get();
                exit(-1);
            }
            goto nextline;
    	}
		if(cmp_nocase(var,"SettingsFile")==0 || cmp_nocase(var,"SettingFile")==0 || cmp_nocase(var,"SettingsFileName")==0 || cmp_nocase(var,"SettingFileName")==0) { //ignored when read from file ! cf seeks_settings_file_name()
// meaningful on command line only but written in cmdline.txt which is read _after_ settingsfile
			goto nextline;
    	}
		if(cmp_nocase(var,"Pause")==0) {
/* Pause determines only two correct contexts for cin.get():
       cerr<< error message + if(cinGetOnError) cin.get() + exit
and
       if(pauseFinal) cin.get() juste avant de quitter le main (one instance only)
cinGetOnError is true at declaration in genepop.cpp but may then be set to false in latin.cpp
*/
			string locstring;
			stringstream strstr(buf.substr(pos+1));
			strstr>>locstring;
			if(cmp_nocase(buf.substr(pos+1),"Final")==0) pauseFinal=true;
			if(cmp_nocase(locstring,"OnError")==0) cinGetOnError=true; // also for a pause without error but then no cin.get()
			if((cmp_nocase(locstring,"Default")==0) || (cmp_nocase(locstring,"NoPause")==0) || (cmp_nocase(locstring,"False")==0)) {cinGetOnError=false; pauseFinal=false;}
			goto nextline;
		}
        if ( ! (var[0]=='%' || var[0]=='#' || var[0]=='/' )) // exclude comments
            cout<<"(!) Unknown keyword \""<<var<<"\" in file "<<filename<<"."<<endl;
// ici test on cinGetOnError cannot work since this setting may not yet have been set
nextline: ;	// the pleasure of sin :-)
        lineCounter++;
    } while(!settings.eof());
}
settings.close();
return 0;
}

int seeks_settings_file_name(const string cmdlinefilename,string& settingsfilename) {
// conceived to be executed only if the file has been created.
string buf,var;
string::size_type pos;
ifstream settings(cmdlinefilename.c_str(),ios::in);
    if(!settings.is_open()) {
    	cout << "Unable to open file "<<cmdlinefilename<< endl;
    	cerr << "Unable to open file "<<cmdlinefilename<< endl;
    	cerr << "Possible cause: several processes try to read the same file\n on a client/file server architecture.\n";
    	cerr << "Further execution would ignore the command line option.\n";
    	cerr << "Use the CmdLineFileName setting to avoid this.\n";
    	cerr << "I exit for safety";
    	if (cinGetOnError) cin.get();
        exit(-1);
    }
    else
    do {
    		getline(settings,buf);
    		if(buf.length()==0) break;
    		while((buf[0]==' ')||(buf[0]=='\t')) {buf.erase(0,1);}//vire les blancs initiaux
    		pos=std::min(buf.find('='),std::min(buf.find('\t'),buf.length()));
    		var=buf.substr(0,pos).c_str();
    		if(cmp_nocase(var,"SettingsFile")==0 || cmp_nocase(var,"SettingFile")==0 || cmp_nocase(var,"SettingsFileName")==0 || cmp_nocase(var,"SettingFileName")==0) {
    			stringstream strstr(buf.substr(pos+1));
    			strstr>>settingsfilename;
                cout<<"Will take settings from file "<<settingsfilename<<endl;
    			settings.close();
    			return 0;
    		}
    } while(!settings.eof());
settings.close();
return 0;
}

int crunchGenepopFile(const int mls,vector<string> optStrings) {
    crunchGenepopFile(mls); // always the base file (T0)
    if (optStrings.size()>0)
        for (vector<string>::iterator ii=optStrings.begin();ii!=optStrings.end();ii++) {
            crunchGenepopFile(mls,(*ii));
        }
return(0);
}

int SMMstepSize(int lt,bool& motifsizeChecked,bool& nomotifChecked,bool& ErrorSMMStepsizeFound){ // lt locus index from zero
    CLocusGP *it=fichier_genepop->loci[lt];
    int minkey=1000,maxkey=-1;
    int maxnotstep=0;
	int compatibleSteps=0;
    vector<int>notstep(6,0);
    for (map<int,CAllele *>::iterator jt=it->galleles.begin(); jt != it->galleles.end(); jt++) {
        minkey=min(minkey,(*jt).first);
        maxkey=max(maxkey,(*jt).first);
    }
    for (int s=1;s<7;s++) {//will check whether this fits into a size-s motif
        if (((maxkey-minkey) % s) ==0) { //check for largest and min allele differences
            for (map<int,CAllele *>::iterator jt=it->galleles.begin(); jt != it->galleles.end(); jt++) { //check for all alleles
                //cout << "s=" << s << " all=" << (*jt).first << " minkey=" << minkey << endl;
                if ((((*jt).first-minkey) % s)!=0) notstep[s-1]++;
                //cout << "nostep=" << notstep[s-1] << endl;
            }
            maxnotstep=max(maxnotstep,notstep[s-1]);
			if(notstep[s-1]==0) compatibleSteps++;
        } else notstep[s-1]++;
    }
    int locstep=SMMstepSizes[std::min(lt,int(SMMstepSizes.size())-1)]; //locstep is zero when there is no explicit SMMstepSizes setting
    if (locstep>=1) {
        if(notstep[locstep-1]>0) {
            cerr<<"\n(!) Motif size for locus "<<lt+1<<" not compatible with allele sizes at this locus." << endl;
            cerr << "Compatible sizes are: "<<endl;
            for (map<int,CAllele *>::iterator jt=it->galleles.begin(); jt != it->galleles.end(); jt++) {
                if ((((*jt).first-minkey) % locstep)==0) cerr<<(*jt).first<<" ";
            }
			cerr << endl;
			cerr << "Incompatible sizes are: "<<endl;
            for (map<int,CAllele *>::iterator jt=it->galleles.begin(); jt != it->galleles.end(); jt++) {
                if ((((*jt).first-minkey) % locstep)!=0) cerr<<(*jt).first<<" ";
            }
			cerr << endl;
            ErrorSMMStepsizeFound=true;
        }
    } else if (compatibleSteps>1) { // need to check which steps sizes are compatible with the allele sizes because multiple step sizes are possible,
        if(!motifsizeChecked) cout<<"(!) Allele sizes for locus "<<lt+1<<" compatible with motif size(s): ";
        int maxstep=0;
        for (int zut=1;zut<5;zut++) if (notstep[zut-1]==0) {if(!motifsizeChecked) {cout<<zut<<" ";} maxstep=zut;}
        if(!motifsizeChecked) {
            cout << endl;
            cout<<"but no explicit step size given in settings." << endl;
            cout<<"We will consider stepsizes of the largest possible value = " << maxstep << endl;
            cout << "However, you may want to stop the program and"<<endl;
            cout<<"use an explicit SMMstepSizes setting instead."<<endl;
            cout<<"No further message will be printed out for the other loci." << endl;
            //if (cinGetOnError) cin.get();
            motifsizeChecked=true;
        }
        locstep=maxstep;
    } else if (compatibleSteps==0 and !nomotifChecked) { //else steps size is 1
		cout<<"\n(!) Allele sizes for locus "<<lt+1<<" are only compatible with a motif size of 1 :" << endl;
		cout <<"Allele sizes seem to be written as a number of repeats in the data file;" << endl;
        cout<<"but no explicit step size given in settings." << endl;
		cout<<"No further message will be printed out for the other loci." << endl;
		//if (cinGetOnError) cin.get();
		nomotifChecked=true;
	}
    return(max(1,locstep));
} //end def SMMstepSize


int crunchGenepopFile(const int mls, string optst) {
// should be called only if genepopBool && dataFileNeededBool
// optionally operates conversion to ad hoc format through the crunchLocTable() fn
//cerr<<"debut crunchGenepopFile() "<<mls;getchar();

// DOES NOT BIN THE DATA (this will be done through ...crunchloctable

static bool nlociIsSet=false;
int itlociInfo=0; //va etre incr['e]ment['e] pour chaque locus informatif
vector<double> extremePop,otherExtremePop; // c'est une adresse...
vector<int> binsPerDim(2,0);
static bool motifsizeChecked=false,nomotifChecked=false,ErrorSMMStepsizeFound=false;

string gp_dg_name;
stringstream gpinfostr;

vector<vector<double> >coord_pop_plus_min_max; // les coord min si existent, idem pour max, puis les coord pops
int obsstateNbr;

       if (genepopRootBool) {
          stringstream stst;
          stst<<gp_fileRoot<<optst<<mls;
          glob_gp_filename=stst.str();
       } else {
          stringstream stst;
          stst<<gp_file<<optst;
          glob_gp_filename=stst.str();
       } // dans cette fonction mls sert uniquement a lire le bon fichier genepop et ['e]crire les bons fichiers de sortie
//cout<<glob_gp_filename;getchar();

       // CBR: imitating Genepop filename setup
       if (nexusRootBool) {
    	   stringstream stst;
    	   if (totalLociCount > 1)
    		   stst << nexusRootFilename << optst << "rep" << mls << "_"; //RL: loc is added later at the end of the string in CNexus::cleanNEXUSFile(const int &locusNum)
    	   else
    		   stst << nexusRootFilename << optst << "rep" << mls;
    	   globalNexusFilename = stst.str();
       } else {
    	   stringstream stst;
    	   if (totalLociCount > 1)
    		   stst << nexusFilename << optst << "_";//RL: loc is added later at the end of the string in CNexus::cleanNEXUSFile(const int &locusNum)
    	   else
    		   stst << nexusFilename << optst;
    	   globalNexusFilename = stst.str();
       } // dans cette fonction mls sert uniquement a lire le bon fichier genepop et ['e]crire les bons fichiers de sortie

       // delete, matching the 'new' below, concludes this function
       fichier_genepop=new CFichier_genepop(glob_gp_filename); // copy file name to member variable
       int rc=fichier_genepop->checkName(); // checks that the file (or file +".txt") can be opened.
       if (rc==-1) {//rc indicates file not found
          if (problem.size()>0) {
             cerr<<"Note some reason(s) the file may be needed:"<<endl;
	         cerr<<problem<<endl<<endl;
          }
          cerr<<"... now exiting";if (cinGetOnError) cin.get(); exit(-1);
       }

//     fichier_genepop->set_eof_check_EOLtype(); //sanitizes end of file
//     CBR: set_eof_check_EOLtype() no longer member function of CFichier_genepop, 17022012
       rc = set_eof_check_EOLtype(fichier_genepop->fileName, true); //sanitizes end of file
       if (rc == -1) { //rc indicates file not found
    	   cerr << "... now exiting";
    	   if (cinGetOnError)
    		   cin.get();
    	   exit(-1);
       }
//cout<<rc;getchar();
       // ELSE : genepop file has been found
       fichier_genepop->parseFile();
       fichier_genepop->extract_coord_pop(); // remplit le tableau membre coord_pop
	   unsigned int nb_sam= (int) fichier_genepop->pops.size(); //CM->RL 23.01.14

	   if (samplesPos.size()>0 && samplesPos.size()!=nb_sam) {
           cerr<<"Number of samples ("<<nb_sam<<")different from number of positions on lattice("<<samplesPos.size()<<")\n";
           cerr<<"Check data or SamplesPopOnLattice option.\nI exit.";
           if (cinGetOnError) cin.get();
           exit(-1);
       }
       /** if 2D la grille doit etre d['e]finie par habitatPars setting, les bords des bins sont exactement ceux donn['e]s par habitat.
        Sinon on contruit l'habitat comme decrit dans la doc... noter que les bords des bins vont d['e]passer geoAreaMin et Max...
        A REVOIR
       **/
       if (samplesPos.size()>0) {
             geoInfo.fill(geoBinNbr);
          // if samplesPos.size()==0 there's nothing else to do. This case is directly handled by IBDcrunchloctable.
       } else if (habitatPars.size()==5) { /*no geoBinNbr through samplesPos but from elsewhere*/
             geoInfo.fill(habitatPars,geoBinNbr); // 1D or 2D
             if (geoBinWidth==0) /*if not explicitly set by user*/ {
                 geoBinWidth=geoInfo.BinWidth; /*deduced from habitatPars and geoBinNbr*/
             }
       } else if (D1IBDbool) { //code utile uniquement en 1 D pour les facons "non preferred" de decrire l'habitat.
          // En 2D, habitatPars est requis, il n'y a pas a construire l'habitat
//cout<<geoBinNbr<<" "<<geoBinWidth;getchar();
          if (geoBinNbr!=0) { //we determine the bounds of the habitat, with different subcases
            double euclidianSq,maxEuclidianSq=0.,maxEuclidian;
            float xshift=numeric_limits<float>::quiet_NaN(),yshift=numeric_limits<float>::quiet_NaN();
            double minx=numeric_limits<double>::max(),
                   miny=numeric_limits<double>::max(),
                   maxx=numeric_limits<double>::min(),
                   maxy=numeric_limits<double>::min();
            int it1=0,it2;
            for(vector<vector<double> >::iterator ee=fichier_genepop->coord_pop.begin();ee!=fichier_genepop->coord_pop.end();ee++) {
                coord_pop_plus_min_max.push_back(*ee);
                if ((*ee)[0]<minx) minx=(*ee)[0];
                if ((*ee)[0]>maxx) maxx=(*ee)[0];
                if ((*ee)[1]<miny) miny=(*ee)[1];
                if ((*ee)[1]>maxy) maxy=(*ee)[1];
            }
            gpinfostr<<"Min x, Min y, Max x, Max y of genotypes: "<<minx<<" "<<miny<<" "<<maxx<<" "<<maxy<<endl;

            // now two or three subcases for bounds determination
            if (geoBinWidth>0) { /*"samples are binned into a number of populations
                    representing a given spatial range, the latter being (...) deduced from
                    number of bins and geographic width of each bin"*/
// si geoBinWidth donne dans settings (maxx-minx doit etre non nul)
// 1e cas: on centre les echantillons dans un habitat de longueur (geoBinNbr-1)*geoBinWidth (les 2 param[`e]tres [`a] donner)
                gpinfostr<<"Sample centered on habitat with length (geoBinNbr-1)*geoBinWidth;\n";
                double maxDistInSample=sqrt(pow(maxx-minx,2)+pow(maxy-miny,2));
                double xShift=(minx-maxx)*(maxDistInSample-(geoBinNbr-1)*geoBinWidth)/(2*maxDistInSample);
                gpinfostr<<"Max. Distance in sample: "<<maxDistInSample<<" "<<endl;
                vector<double> tempovec;
                tempovec.push_back(minx-xShift);
                tempovec.push_back(miny-xShift*(maxy-miny)/(maxx-minx));
                gpinfostr<<"x,y coordinates of one bound: "<<tempovec[0]<<" "<<tempovec[1]<<endl;
                coord_pop_plus_min_max.push_back(tempovec);
                tempovec.resize(0);
                tempovec.push_back(maxx+xShift);
                tempovec.push_back(maxy+xShift*(maxy-miny)/(maxx-minx));
                gpinfostr<<"x,y coordinates of other bound: "<<tempovec[0]<<" "<<tempovec[1]<<endl;
                coord_pop_plus_min_max.push_back(tempovec);
            } else /*geoBinWidth=0*/ {/*"samples are binned into a number of populations
                        representing a given spatial range, the latter being (...) deduced (...) from number
                        of bins and position of extreme points of the array"*/
// MAIS: les extremes de l'habitat ont pu etre donnes dans les settings
              if (geoAreaMinBool) coord_pop_plus_min_max.push_back(geoAreaMin);
              if (geoAreaMaxBool) coord_pop_plus_min_max.push_back(geoAreaMax);
            }
            if (coord_pop_plus_min_max.size()==1) {
              /*on est ici si geoBinNbr>0, un seul ['e]chantillon, ni samplesPos ni habitatPars*/
               cerr<<"So you have a single sample and no information about habitat boundaries.\n";
               cerr<<"Please provide such info (e.g. habitatPars).\nI exit.";
               if (cinGetOnError) cin.get();
               exit(-1);
            }
            // now we have the extremes of the habitat, and still abs(geoBinNbr)>0
// on va sortir la distance la plus grande entre les paires de (pops+AreaMin,Max) et la pop extreme de x minimal

// BUG suppose au moins deux pops.... pq ca plante pas en 2D mais seulement en 1D (avec ou sans habitatPars)...

            for(vector<vector<double> >::iterator ee=coord_pop_plus_min_max.begin();ee!=coord_pop_plus_min_max.end()-1;ee++) {
              it2=it1+1;
              for(vector<vector<double> >::iterator eee=ee+1;eee!=coord_pop_plus_min_max.end();eee++) {
// note this includes geoAreaMin, geoAreaMax
                 euclidianSq=pow((*ee)[0]-(*eee)[0],2)+pow((*ee)[1]-(*eee)[1],2);       //squared !
                 if (euclidianSq>maxEuclidianSq) {
                    maxEuclidianSq=euclidianSq;
                    if ((*ee)[0]<(*eee)[0]) {
                        extremePop=(*ee); // garde la pop extreme avec la coord x la plus ptte
                        otherExtremePop=(*eee);
                    } else {
                        extremePop=(*eee); // garde la pop extreme avec la coord x la plus ptte
                        otherExtremePop=(*ee);
                    }
                    xshift= (float) ( otherExtremePop[0]-extremePop[0] );
                    yshift= (float) ( otherExtremePop[1]-extremePop[1] );
//cerr<<sqrt(maxEuclidianSq)<<" "<<extremePop[0]<<" "<<extremePop[1];
                 }
                 it2++;
              }
              it1++;
            }
            maxEuclidian=sqrt(maxEuclidianSq);
            //nothing has been binned yet
            //we have the largest distance in the sample, possibly augmented with other habitat bounds if geoBinWidth>0
            //geoBinWidth may not yet be set
            if (abs(geoBinNbr)!=1) { //bin number is fixed!=(-1,0,1), we deduce bin width =>
              // must give back input geoBinWidth if we gave both geoBinWidth and geoBinNbr, but no other habitat extremes, in settings
              gpinfostr<<"Max distance between individuals and/or bounds: "<<maxEuclidian<<endl;
              geoBinWidth=maxEuclidian/(abs(geoBinNbr)-1);
              gpinfostr<<"geoBinWidth: "<<geoBinWidth<<endl;
            }
            //ex 4 bins, 4 pops aux pos 0,1,2,3 =>maxEucl=3 =>width=1
            else /*abs(geoBinNbr)=1*/geoBinWidth=numeric_limits<double>::max(); // will bin all samples into one
            //now geoBinWidth has been set to nonzero value.
            ofstream settout("settings.out.txt",ios::app);
            settout<<gpinfostr.str();
            settout.close();
            {// filling geoInfo in order to use IBDcrunchLocTable on 1D samples too
                if (std::isnan(xshift)) {
                   cerr<<"(!) From crunchGenepopFile: xshift (yshift) requested but not initialized. I exit";
                   if (cinGetOnError) cin.get();
                   exit(-1);
                }
                vector<float>boundingbox(0);
                boundingbox.push_back(extremePop[0]-geoBinWidth/2.);
                boundingbox.push_back(extremePop[1]-geoBinWidth/2.);
                boundingbox.push_back(maxEuclidian+geoBinWidth);
                boundingbox.push_back(geoBinWidth-numeric_limits<float>::epsilon());
                boundingbox.push_back(atan2(yshift,xshift)); //radians... note order of args atan(y/x) <-> atan2(y,x)
//ostream_vector(boundingbox,cout);getchar();
                geoInfo.fill(boundingbox,geoBinNbr); // 1D && genepop && geobinNbr!=0
            }
          } else /*geoBinNbr=0*/ { //geoBinWidth is ignored !
             geoBinWidth=0;// indicateur temporaire pas de bins pour crunchloctable...
             //coordinates of samples will be ignored (geoBinWidth is set to 1 afterwards)
             //could be modified, but consistent with documentation
             geoInfo.fill(nb_sam); // 1D && genepop && geoBinNbr not given in settings!
          }
       } //endif samplesPos / habitatpars / D1IBDbool
/******************************************************************
   pour avoir un code synthetique il faudrait convertir les infos 1 dim en une bounding box
   Plus reunir geoBinWidth et geoInfo.BinWidth.... attention geoBinWidth could still represent user input cf code if (habitatPars.size()==5)  above
******************************************************************/
       // actual binning
       int nlociInfoOrNot= (int) fichier_genepop->loci.size();//CM->RL 23.01.14
       if (multipleMarkersBool && (totalLociCount != nlociInfoOrNot)) {
   		cerr << "(!) The \"LociPerModel\" setting in " << settingsfilename << " does not add up to the loci count in the Genepop data file " << fichier_genepop->fileName << endl
   		     << "Aborting Migraine...Press any key to exit!" << endl;
   		if (cinGetOnError)
   			cin.get();
   		exit(-1);
       }
       finalSMMstepSizes.resize(0);
       for (int intloc=0;intloc<nlociInfoOrNot;intloc++) {
// test pour exclure les locus monomorphes ou non
          if (fichier_genepop->coding[intloc]>=4)
            obsstateNbr= (int) fichier_genepop->loci[intloc]->galleles.size(); //CM->RL 23.01.14
          else
            obsstateNbr= (int) fichier_genepop->loci[intloc]->alleles.size();
          if (!(lociPolymorphicBool && obsstateNbr==1)) {
//cerr<<nlociInfoOrNot<<" "<< intloc<<" "<<obsstateNbr<<endl;
             {stringstream stst(stringstream::in | stringstream::out);
                      stst<<samplerootname<<(mls-1)*nlociInfoOrNot+itlociInfo+1;
                      stst>>gp_dg_name;
//cout<<gp_dg_name<<" "<<mls;getchar();
             } // gp_dg_name is NOT necesssarily used in the following call.
    // attention au cas geoBinWidth=0 ou geoInfo n'a pas ete rempli...
             crunchLocTable(intloc,gp_dg_name,geoInfo);// should also fill geoInfo if not previously done
             if(mutModelBool[SMM] || mutModelBool[GSM]) finalSMMstepSizes.push_back(SMMstepSize(intloc,motifsizeChecked,nomotifChecked,ErrorSMMStepsizeFound));
             itlociInfo++;
          }
       }
    if(ErrorSMMStepsizeFound){
        cerr<<"\nSMMstepsize error(s) have been found, I exit."<<endl;
        if (cinGetOnError) cin.get();
        exit(-1);
    }

    if(motifsizeChecked) {
        cout<<"\n(!) Allele sizes for all loci were fixed to the following values :\nStepSizes=";
        for (int intloc=0;intloc<nlociInfoOrNot;intloc++) {
            cout << finalSMMstepSizes[intloc];
            if(intloc<nlociInfoOrNot-1) cout << ",";
            else cout << endl;
        }
        cout<<"This vector can be directly used in the settings file." << endl;

    }
       gpinfostr.str("");
       if(IBDbool) gpinfostr<<"\nBin numbers in each dimension: "<<geoInfo.xBins<<","<<geoInfo.yBins<<endl;
       cout<<gpinfostr.str();
       ofstream settout("settings.out.txt",ios::app); //FR->FR potential problem (elsewhere too ?) is that several concurrent processes can write in the same file
       settout<<gpinfostr.str();
       settout.close();
       if(D1IBDbool && geoBinNbr==0) { //see comments on "/*geoBinNbr=0*/" above
           geoBinWidth=1; // needed for R code
   	   }
       if (!nlociIsSet) { // we're here only if a genepop file has been found; hence otherwise nloci taken from settings
          nloci=itlociInfo;
   	      mloglikdim=int(perLocusBool)*nloci+1+int(asymptoBool);
          nlociIsSet=true;
          gpinfostr.str("");
          gpinfostr<<"\n"<<nloci<<" informative loci found"<<endl<<endl;
          cout<<gpinfostr.str();
          ofstream settout("settings.out.txt",ios::app);
          settout<<gpinfostr.str();
          settout.close();
       }
       if (geoBinNbr<0) { //signal subtil pour cr['e]er les fichiers ad hoc seulement.
          cout<<"Conversion to ad hoc format finished";
          if (pauseFinal) cin.get();
          exit(0);
       }
//cerr<<"avant delete dans crunchGenepopFile()";getchar();
       delete fichier_genepop;
//cerr<<"fin crunchGenepopFile()";getchar();
return 0;
}



int apply_final_settings() {
static bool executed=false;



    if (executed) {
        cerr<<"Do not call apply_final_settings() twice because of 'lb=new'... ";
        cerr<<"I exit."<< endl;
        if (cinGetOnError) cin.get();
        exit(-1);
    }

    if (markerCount > 1)
    	multipleMarkersBool = true;
    else if (markerCount == 0) {
    	mutationModelVector.push_back("PIM");
    	mutModelBool[PIM] = true;
    }
	if (!multipleMarkersBool && (totalLociCount > 0))
    	cout << "(!) Ignoring the \"LociPerModel\" setting in " << settingsfilename << " as the specified type of markers (i.e. with different mutation models) is less than 2." << endl;

	if (mutModelBool[ISM]) {
		if (!nexusBool) {
			cerr << "(!) The ISM model needs to analyse sequence data. No Nexus file has been specified." << endl
			     << "Aborting Migraine...Press any key to exit!" << endl;
			if (cinGetOnError)
				cin.get();
			exit(-1);
		}

		if (!nexusNumberLabels && !sameGpNexusLabels) {
			cerr << "(!) Specific Nexus taxon labels for sequence loci need to correspond to the individual labels in respective Genepop file." << endl
				 << "Please cross-check the Genepop indivual labels and the Nexus taxon labels." << endl
			     << "Aborting Migraine...Press any key to exit!" << endl;
			if (cinGetOnError)
				cin.get();
			exit(-1);
		}
	}
	if (mutModelBool[SNP]) {
		if (!nexusNumberLabels && !sameGpNexusLabels) {
			cerr << "(!) Specific Nexus taxon labels for sequence loci need to correspond to the individual labels in respective Genepop file." << endl
            << "Please cross-check the Genepop indivual labels and the Nexus taxon labels." << endl
            << "Aborting Migraine...Press any key to exit!" << endl;
			if (cinGetOnError)
				cin.get();
			exit(-1);
		}
	}

    if (multipleMarkersBool) {
    	// loci specification format must follow marker specification format
        if ((totalLociCount == 0) || (lociPerMarker.size() != mutationModelVector.size())) {
    		cerr << "(!) The \"LociPerModel\" setting doesn't correspond to the multiple marker \"MutationModel\" setting." << endl
    		     << "Aborting Migraine...Press any key to exit!" << endl;
    		if (cinGetOnError)
    			cin.get();
    		exit(-1);
        }

        if (mutModelBool[PIM]) {
    		cerr << "(!) PIM models cannot be specified along with other mutation models in Migraine!" << endl
    		     << "Aborting Migraine...Press any key to exit!" << endl;
    		if (cinGetOnError)
    			cin.get();
    		exit(-1);
        }

        if (mutModelBool[SNP]) {
    		cerr << "(!) SNP models (yet under development) will soon be implemented alongside other mutation models in Migraine!" << endl;
//    		cerr << "Aborting Migraine...Press any key to exit!" << endl;
//    		if (cinGetOnError)
//    			cin.get();
//    		exit(-1);
        }

        if (IBDbool) {
    		cerr << "(!) The multiple mutational model specification hasn't been extended to the IBD model." << endl
    		     << "Aborting Migraine...Press any key to exit!" << endl;
    		if (cinGetOnError)
    			cin.get();
    		exit(-1);
        }

        //	Expands unspecified/partly specified/marker-wise specified parameter values into a locus-wise specification
        if ( ( (int) mutationModelVector.size() ) != totalLociCount)
        	expandParamVector(mutationModelVector, "MutationModel");
        if ( (mutModelBool[SMM] || mutModelBool[GSM]) && sequenceGivenK.size() == 0) {
        	cout << "\n(!) Number of possible states (option GivenK) missing for stepwise mutation models in " << settingsfilename << ";" << endl;
        	cout << "(!) Migraine will use the default value of 400 possible states for SMM or 40 for GSM.\n" << endl;
            sequenceGivenK.resize(totalLociCount);
            for (int lociCount = 0; lociCount < totalLociCount; lociCount++) {
                if (mutationModelVector[lociCount] == "SMM") sequenceGivenK[lociCount]=400;
                else if(mutationModelVector[lociCount] == "GSM") sequenceGivenK[lociCount]=40;
            }
        } else expandParamVector(sequenceGivenK, "GivenK");

    	// IF (SMMstepSizes.size() == 0) leave it to SMMstepSize() to take care of everything
        if (SMMstepSizes.size() > 0)
        	expandParamVector(SMMstepSizes, "SMMstepSizes");

    	//	correcting for some "minor" incompatibility issues
    	for (int lociCount = 0; lociCount < totalLociCount; lociCount++) {
        	if (mutationModelVector[lociCount] == "ISM")
        		sequenceGivenK[lociCount] = 0;
        	else if (sequenceGivenK[lociCount] == -1) {
                if (mutationModelVector[lociCount] == "SMM") sequenceGivenK[lociCount]=400;
                else if(mutationModelVector[lociCount] == "GSM") sequenceGivenK[lociCount]=40;
            }

        }

        if( (DemographicModel.substr(0,6)=="OnePop" || DemographicModel=="IM") && piOnePopSolverVector.size()==0) {
            piOnePopSolverVector.resize(totalLociCount);
//            if (DemographicModel == "IM")
//                for(int lociCount=0;lociCount<totalLociCount;lociCount++)
//                	piOnePopSolverVector[lociCount] = "SD";
//            else {
				for (int lociCount = 0; lociCount < totalLociCount; lociCount++) {
					if (mutationModelVector[lociCount] == "SMM") {
						if (sequenceGivenK[lociCount] > 200)
							piOnePopSolverVector[lociCount] = "DG";
						else
							piOnePopSolverVector.push_back("Eigen");
					}
					else if (mutationModelVector[lociCount] == "GSM")
						piOnePopSolverVector[lociCount] = "Eigen";
//                }
            }
        }
        if (piOnePopSolverVector.size() > 0 && piOnePopSolverVector.size() < (size_t) totalLociCount)
        	expandParamVector(piOnePopSolverVector, "piOnePopSolver");


    } else { //adjusting mutation models for single marker analyses

        if (mutationModelVector[0]=="SMM") {
            if (sequenceGivenK.size() == 0) {
                cout << "\n(!) Number of possible states missing for the microsatellite locus/loci in " << settingsfilename << ";" << endl;
                cout << "(!) Migraine will use the default value of 400 possible states for SMM." << endl;
                cout << "(!) You can use AllelicBounds or GivenK settings to specify your own values.\n" << endl;
                sequenceGivenK.push_back(400);
            }
        } else if (mutationModelVector[0]=="GSM") {
                if (sequenceGivenK.size() == 0) {
                cout << "\n(!) Number of possible states missing for the microsatellite locus/loci in " << settingsfilename << ";" << endl;
                cout << "(!) Migraine will use the default value of 40 possible states for GSM." << endl;
                cout << "(!) You can use AllelicBounds or GivenK settings to specify your own values.\n" << endl;
                sequenceGivenK.push_back(40);
            }
        }
        if((DemographicModel.substr(0,6)=="OnePop" || DemographicModel=="IM") && piOnePopSolverVector.size()==0) {
            piOnePopSolverVector.resize(0);
//            if (DemographicModel == "IM")
//            	piOnePopSolverVector.push_back("SD");
//            else {
				if (mutationModelVector[0] == "SMM") {
					if (sequenceGivenK[0] > 200)
						piOnePopSolverVector.push_back("DG");
					else
						piOnePopSolverVector.push_back("Eigen");
				}
				else if (mutationModelVector[0] == "GSM")
					piOnePopSolverVector.push_back("Eigen");
//            }
        }
    }


    //RL->CBR check this, and add explanation, it is not intuitive!
    firstCallOfLocus.assign(totalLociCount,true);

    if (HobolthISM && ((DemographicModel.substr(0,6) != "OnePop") || ! (mutModelBool[ISM] || mutModelBool[SNP])) ) {
		cerr << "(!) For the moment the Hobolth proposal can only be specified for ISM loci and a single population model (irrespective of time inhomogeneity)." << endl
		     << "Aborting Migraine...Press any key to exit!" << endl;
		if (cinGetOnError)
			cin.get();
		exit(-1);
    }


//    if (genepopBool && nloci!=0) {
//		cout<<"Number of loci given by Genepop* input file.\nnloci will be ignored"<<endl;
//        nloci=0; //FER 6/8/2008 // en fait pas terrible car genepopBool peut etre true alors que fichier Genepop absent et pas necess
//   	}
// implements default values AND gives at least one element to vectors later assumed to have element [0]
	if (!allowsEmptyColumns && (mutModelBool[SMM] or mutModelBool[GSM]) && !(mutModelBool[ISM] or mutModelBool[SNP])) {
//		cout<<"\n\n(!) AllState must be true for SMM computations."<<endl;
//		cout<<"(!) It is now automatically set to true for further computations.\n\n"<<endl;
		allowsEmptyColumns=true;
//		if (cinGetOnError) cin.get();
	}
	if (minSmoothness>maxSmoothness) minSmoothness=maxSmoothness;
	/** check and standardisation of (demographic) model name**/
	{bool found=false;
	    if (DemographicModel.substr(0,6)=="OnePop") found=true;  //FR->RL. just a quick hack. The next loop will run too and standardize the name if it is valid
		for (vector<string>::iterator it2=standardDModelNames.begin();it2!=standardDModelNames.end();it2++) {
			if (cmp_nocase(DemographicModel,(*it2))==0) {DemographicModel=(*it2); found=true; break;}
		}
        if ( ! found ) {
            cout<<"(!) No valid demographic model specification in settings:"<<endl;
            cout<<"I exit."<< endl;
            if (cinGetOnError) cin.get();
            exit(-1);
        }
	} /** end check demo model**/
	if (DemographicModel=="OnePopFounderFlush" && !(VarSizeFunction=="Exponential")) {
		cout<<"(!) The One Pop Founder-Flush can only consider exponential continuous change in population after the founding event."<<endl;
		cout<<"(!) Check your input setting file for consistency."<<endl;
		cout<<"I exit."<< endl;
		if (cinGetOnError) cin.get();
		exit(-1);
	}
    if ((VarSizeFunction=="Discrete" && !(DemographicModel.substr(0,6) == "OnePop" || DemographicModel == "IM"))
    		|| ((VarSizeFunction=="Exponential" || VarSizeFunction=="Logistic") && DemographicModel.substr(0,6)!="OnePop")) {
		cout<<"(!) Variable population size/Founder-Flush specification with more than one population in settings:"<<endl;
		cout<<"(!) Such model is not implemented, only the OnePopmodel with variable size or  with founder flush has been implemented"<<endl;
		cout<<"I exit."<< endl;
		if (cinGetOnError) cin.get();
		exit(-1);
	}
	if (AlternativeIS_FRExpMixAncActPopSizebool && (DemographicModel!="OnePopVarSize" or VarSizeFunction=="Logistic" or piOnePopSolverBool[SD]==true or piOnePopSolverBool[Eigen]==true) ) {
		cout<<"(!) Alternative IS proposal based on a mix between Pi's computed" << endl;
		cout<<"(!) using ancestral and actual population sizes are only" << endl;
		cout<<"(!) implemented for the single pop model with exponential or discrete change in population size."<<endl;
		if(piOnePopSolverBool[SD]==true) cout<<"(!) It is not implemented for Stephens & Donnelly pi one pop solver (option SD)."<<endl;
		if(piOnePopSolverBool[Eigen]==true) cout<<"(!) It is not implemented for Leblois & Rousset pi one pop solver (option 'Eigen')."<<endl;
		if(VarSizeFunction=="Logistic") cout<<"(!) It is not implemented for Logistic change in size"<<endl;
		cout<<"(!) Check your input setting file for consistency."<<endl;
		cout<<"I exit."<< endl;
		if (cinGetOnError) cin.get();
		exit(-1);
	}
	if (AlternativeIS_AncPopSizebool && (DemographicModel!="OnePopVarSize" or VarSizeFunction=="Logistic" or piOnePopSolverBool[SD]==true or piOnePopSolverBool[Eigen]==true)) {
		cout<<"(!) Alternative IS proposal based on Ancestral population sizes are only implemented for the single pop model with discrete or exponential change in size."<<endl;
		if(piOnePopSolverBool[SD]==true) cout<<"(!) It is not implemented for Stephens & Donnelly pi one pop solver."<<endl;
		if(piOnePopSolverBool[Eigen]==true) cout<<"(!) It is not implemented for Leblois & Rousset pi one pop solver."<<endl;
		if(VarSizeFunction=="Logistic") cout<<"(!) It is not implemented for Logistic change in size"<<endl;
		cout<<"(!) Check your input setting file for consistency."<<endl;
		cout<<"I exit."<< endl;
		if (cinGetOnError) cin.get();
		exit(-1);
	}
	//CBR: this check needs to be modified as and when options are added to the ISM model
	if ( (mutModelBool[ISM]  || mutModelBool[SNP]) && (DemographicModel != "OnePop") && (DemographicModel != "OnePopVarSize") && (DemographicModel != "OnePopFounderFlush")) {
		cout << "(!) ISM & SNP mutation models are only implemented for the time-homogeneous panmictic population model"
				"\n or the time in-homogeneous panmictic population models (OnePopVarSize & OnepopFounderFlush) in the current version" << endl
		     << "Aborting Migraine...Press any key to exit!" << endl;
		if (cinGetOnError)
			cin.get();
		exit(-1);
	}
	//CBR: this check needs to be modified as and when options are added to the SNP model
	if (mutModelBool[SNP]) {
		cout << "(!) SNP models will soon be implemented in Migraine!" << endl;
//		cout << "Aborting Migraine...Press any key to exit!" << endl;
//		if (cinGetOnError)
//			cin.get();
//		exit(-1);
	}
	if (mutModelBool[SNP] && (DemographicModel != "OnePop" && DemographicModel != "OnePopVarSize" && DemographicModel != "OnePopFounderFlush")) {
		cout << "(!) SNP models are only implemented for the models with a single population"
            "\n e.g. OnePop, OnePopVarSize & OnepopFounderFlush in the current version" << endl
            << "Aborting Migraine...Press any key to exit!" << endl;
		if (cinGetOnError)
			cin.get();
		exit(-1);
	}
	if (IBDbool && !mutModelBool[PIM]) {
					cout<<"(!) Conflicting specifications in settings:"<<endl;
					cout<<"(!) only PIM mutation models are implemented for isolation by distance"<<endl<<" in the current version"<<endl;
					cout<<"I exit."<< endl;
					if (cinGetOnError) cin.get();
					exit(-1);
	}
	if(!mutModelBool[PIM]) {
				PiSolverFromSettingsInfo="SMM2popDGLR05"; //may be overwritten by OnePop
	} else {
				if(cmp_nocase(PiSolverFromSettingsInfo,"SymDirect")==0) {
					if(DemographicModel=="Npop" || DemographicModel=="IM"||DemographicModel.substr(0,6)=="OnePop") { //pas ['e]vident [`a] d['e]placer... car pas de SymDirectbool...
						cout<<"(!) Option SymDirect is not implemented for a symmetric Npop/IM model.\nSymDirect will be ignored and PiPim solver will be used instead"<<endl;
						PiSolverFromSettingsInfo="PiPim";
					}
                } else if ( ! IBDbool && PiSolverFromSettingsInfo=="") {
                    PiSolverFromSettingsInfo="PiPim";
                }
	}

    if (SMMstepSizes.size()==0) SMMstepSizes.push_back(0);

    if (DemographicModel.substr(0,6)=="OnePop") PiSolverFromSettingsInfo=DemographicModel; //"OnePop" or "OnePopVarSize" or "OnePopFounderFlush";
    
    if (cmp_nocase(PiSolverFromSettingsInfo,"PCG")==0) { // only specified PCG without further detail, For IBD/Island models
         if (D1IBDbool) PiSolverFromSettingsInfo="PCG_penta";
        else if (D2IBDbool) PiSolverFromSettingsInfo="PCG_island";
        else {
            cerr<<"(!) PCG not implemented for non-IBD model";
            cerr<<"I exit."<< endl;
            if (cinGetOnError) cin.get();
            exit(-1);
        }
    }
    
	if(!PopSizeScaleForTimebool && !(VarSizeFunction=="Discrete" or VarSizeFunction=="Exponential")) {
        cout<<"(!) Time scaled by mutation rate is only working for a discrete or exponential change"<<endl;
		cout<<"(!) other models are not implemented"<<endl;
		cout<<"I exit."<< endl;
		if (cinGetOnError) cin.get();
		exit(-1);
    }
	if (sequenceStat.size()==0) {// implemente le d['e]fault de compilation
		if (pacbool) {
		    sequenceStat.push_back("PAC");
		    PACinSeqBool=true;
        } else if (pacancbool) {
		    sequenceStat.push_back("PACanc");
		    PACancinSeqBool=true;
        } else sequenceStat.push_back("IS");
	}
	if (ancestralScaledTimes.size()!=ancestralSuffixes.size()) {
        cout<<"(!) Lengths of ancestralScaledTimes and ancestralSuffixes differ:"<<endl;
        ostream_vector(ancestralScaledTimes,cout);
        cout<<endl;
        ostream_vector(ancestralSuffixes,cout);
        cout<<endl;
        cout<<"case not yet implemented"<<endl;
        cerr<<"I exit."<< endl;
        if (cinGetOnError) cin.get();
		exit(-1);
    }
//time needed in particular if data are taken at different (known) times
// this is different from the IM model with only one sample, where the estimable parameters are only scaled ones
    if (ancestralSuffixes.size()>0 || asymptoBool) timeNeededBool=true;
    if ((timeNeededBool || TMRCABool)  && PACinSeqBool) {
        cerr<<"(!) PAC-likelihood setting incompatible with settings using temporal information."<<endl;
        cerr<<"I exit."<< endl;
        if (cinGetOnError) cin.get();
        exit(-1);
    }
	for(unsigned int i=0;i<sequenceStat.size();i++) {
		if (TMRCABool && cmp_nocase(sequenceStat[i],"ISstrict")!=0) {
			cerr<<"(!) strict IS computations is the only statistic compatible with TMRCA computations."<<endl;
			cerr<<"TMRCA Computations will not be considered for this run."<<endl;
			TMRCABool=false;
			//if (cinGetOnError) cin.get();
		}
	}

    if ((DemographicModel=="OnePopVarSize" || DemographicModel=="OnePopFounderFlush" || DemographicModel=="IM") && PACinSeqBool) {
       cerr<<"(!) PAC-likelihood setting incompatible with variable population size."<<endl;
       cout<<"I exit."<< endl;
       if (cinGetOnError) cin.get();
       exit(-1);
    }
    if (!(DemographicModel=="OnePopVarSize" || DemographicModel=="OnePopFounderFlush" || DemographicModel=="IM") && PACancinSeqBool) {
		cerr<<"(!) PACanc-likelihood setting incompatible with constant-time demographic models."<<endl;
		cout<<"Check your settings. I exit."<< endl;
		if (cinGetOnError) cin.get();
		exit(-1);
    }
    if (timeNeededBool) {
       if (!genepopBool) {
           cerr<<"(!) GenepopFile or GenepopRootFile specification required for temporal data."<<endl;
           cerr<<"I exit."<< endl;
           if (cinGetOnError) cin.get();
           exit(-1);
       }
// adds the scale parameter to the model parameters
//FR cr['e]e conflit dans test RL: args.pars->4 (OK), Disp_Pars->2 (OK), PopSizeTab->1 (?), le reste O
// on laisse en plan pour l'instant
//       _lb.push_back(0);
//       _ub.push_back(1000); //FR 11/01/2008 temporary default value
       //ParameterNumber is determ from their size below
    }
	if (sequenceWrite.size()==0) {
	    sequenceWrite.push_back("Over");// d['e]fault de compilation
        sequenceAddRepl.push_back(0); // because only filled when sequenceWrite is, and needs to be non empty
	}
	if(   cmp_nocase(sequenceWrite.back(),"WriteRKrig")==0
	    || cmp_nocase(sequenceWrite.back(),"KrigOnly")==0) noRCallBool=false; /* imperfect, is we have first declared writeRkrig then noRcall.
	   But this case does not seem relevant in practice. More relevant is the case where we first has noRcall (for block computation of points
	   then writeRKrig in a later call)	 */
	if (boundsSource.size()==0) { // implementing the default boundsSource
        /** in the first iteration, firstfill is called and it will readpoints based on the writeSequence=readpoints specification,
        not on the boundsSource **/
        boundsSource.push_back("pointsFromR"); // Readpoints in all iterations beyond the first
	}
	if (sequenceNrunsPerPoint.size()==0) sequenceNrunsPerPoint.push_back(__NrunsPerPoint);// d['e]fault de compilation
    // following code to reverse the default bools when not IBD Genepop
	if (!(IBDbool && genepopBool)) {
        if (!readAdHocInSettingsBool && !genepopBool) { //no explicit setting
           readAdHocFileBool=true; //default is false for IBD Genepop
        }
        if (!writeAdHocInSettingsBool) { //no explicit setting
           writeAdHocFileBool=false; //default is false for IBD Genepop
        }
	} /*else uses current read/write bools, either at default (true), or overridden in the settings file*/

    if (genepopBool && D1IBDbool && (geoAreaMinBool|| geoAreaMaxBool) && geoBinNbr!=0 && geoBinWidth>0) {
      cout<<"(!) 1D IBD: geoAreaMin/geoAreaMax overrides geoBinWidth setting when geoBinNbr!=0."<<endl;
    }
    if (genepopBool && D2IBDbool && habitatPars.size()==5 && geoBinNbr!=0 && geoBinWidth>0) {
      cout<<"(!) 2D IBD: habitatPars overrides geoBinWidth setting when geoBinNbr!=0."<<endl;
    }
    if (IBDbool && habitatPars.size()==5) {
      if(geoBinNbr==0) {
        cout<<"(!) IBD with habitatPars setting: bin _number_ must be specified in settings."<<endl;
		cout<<"I exit."<< endl;
		if (cinGetOnError) cin.get();
		exit(-1);
      }
    }
    if (D2IBDbool) {
      if(geoBinNbr==0) {
        cout<<"(!) 2D IBD: Axial bin number must be specified in settings."<<endl;
		cout<<"I exit."<< endl;
		if (cinGetOnError) cin.get();
		exit(-1);
      }
      if(habitatPars.size()!=5) {
        cerr<<"(!) 2D IBD: habitatPars must be specified in settings."<<endl;
		cerr<<"I exit."<< endl;
		if (cinGetOnError) cin.get();
		exit(-1);
      }
      if (D2IBDbool && geoInfo.yBins==1) {
             cerr<<"(!) 2D IBD: habitatPars/DistanceBins settings combination describes a linear habitat."<<std::endl;
		     cerr<<"Reconsider your settings. I exit."<< std::endl;
		     if (cinGetOnError) std::cin.get();
		     exit(-1);
      }
      if (D1IBDbool && geoInfo.yBins>1) {
             cerr<<"(!) 1D IBD: habitatPars setting describes a 2D habitat but DemographicModel is 1D IBD."<<std::endl;
		     cerr<<"Reconsider your settings. I exit."<< std::endl;
		     if (cinGetOnError) std::cin.get();
		     exit(-1);
      }
    }/* si D1IBDbool le nombre de bins n'est pas forcement deja connu.
    Le .fill() se fait dans crunchLocTable (donc une fois par locus).
    Ca serait bien d'organiser le code autrement.*/

//all PiSolver-related stuff moved to calclogl

	aleah.seed(_ptSamplingSeed);
	alea.seed(_EstimationSeed);
//update _pointmin,_pointmax,PointNumber
    if (singlePointBool) {//changement pour faire des plots avec plein de points sur une valeure de parametres
		//PointNumber=1;
		if (sequencePointNumber.size()==0) {
			sequencePointNumber.push_back(PointNumber); //compilation default
        } else PointNumber=sequencePointNumber[0];
        noRfileBool=true;
       _pointmin=1;
       _pointmax=max(1,int(PointNumber));
    } else { // on a diverses combin de PointNumber pointmin pointmax dans les settings
// 0n regarde si on a des valeurs *explicites* de min et max differentes de O, PointNumber.
// Sinon, on applique les defaults:
        if (sequencePointNumber.size()==0) {
           sequencePointNumber.push_back(PointNumber); //compilation default
        } else {
            if (sequenceWrite[0]=="ReadPoints") {
               PointNumber=sequencePointNumber[min(1,int(sequencePointNumber.size())-1)];
            } else PointNumber=sequencePointNumber[0];
        }
        if (_pointmin==-1) pointmin=1;
        if (_pointmax==numeric_limits<int>::max()) pointmax=PointNumber;
        // leaves _pointmin _pointmax untouched
    }
    if (userPointMinOrMax && (sequencePointNumber.size() > 1)) {
		cerr << "(!) A user-defined Pointmin/Pointmax setting is incompatible with more than a single Pointnumber." << endl
		     << "Aborting Migraine...Press any key to exit!" << endl;
		if (cinGetOnError)
			cin.get();
		exit(-1);
    }
    //	removing all association with R handling when Pointmin/Pointmax have been user-specified
    //	and checking if sequenceWrite contains parameters other than "Over"/"Readpoints"

    // FR->FR readPoints,Append could make sense, but perhaps mishandled by naive users ?
    if (userPointMinOrMax) {
    	noRCallBool = true;
    	noRfileBool = true;
    	for (size_t writeSeq = 0; writeSeq < sequenceWrite.size(); writeSeq++) {
    		if ((cmp_nocase(sequenceWrite[writeSeq], "Over") != 0) && (cmp_nocase(sequenceWrite[writeSeq], "Readpoints") != 0)) {
    			cerr << "(!) A user-defined Pointmin/Pointmax setting is only compatible with Over or Readpoints in WriteSequence. Please check your settings" << endl
    			     << "Aborting Migraine...Press any key to exit!" << endl;
    			if (cinGetOnError)
    				cin.get();
    			exit(-1);
    		}
    	}
    }
    if (sequenceAddRepl.size()>0){
        vector<int>::iterator it=max_element(sequenceAddRepl.begin(),sequenceAddRepl.end());
        if ((*it)>PointNumber) { // FR: >= replaced with > 18/12/2014
            cerr<<"addReplicates is used to ask for more replicates than 'PointNumber' ("<<(*it)  <<" vs. "<<PointNumber<<"). \n  Check settings. I exit.";
            /** one of the problem is that the user would need to change estimationSeed fro a useful replication of all points.
            But it's simpler to forbid a useless option that to allow it and explain all the complications **/
            if (cinGetOnError) cin.get();
            exit(-1);
        }
    } //end block test+iterator
// code suivant necess car la valuation de ChangefnPtr passe tjrs par sequenceStat //FR: commentaire incomprehensible. Obsolete ?
// at this point _ub and _lb should always contain the right values.
// Copy in ub and lb only needed because several procedures uses arrays...
	if (_ub.size()==_lb.size()) ParameterNumber= (int) _lb.size();//CM->RL 23.01.14
	else {
		cerr<<"\nNumbers of parameters in lower and upper bounds do not match."<<endl;
		cerr<<"Lower bounds ("<<_lb.size()<<" parameters): "<<endl;
		ostream_vector(_lb,cerr);
		cerr<<"Lower bounds ("<<_ub.size()<<" parameters): "<<endl;
		ostream_vector(_ub,cerr);
		cerr<<"Execution aborted. Check settings."<<endl;
		if (cinGetOnError) cin.get();
		exit(-1);
	};
// Now tries to find cases where it's not necessary to read the data file
    dataFileNeededBool=true;
    if (cmp_nocase(sequenceWrite.back(),"KrigOnly")==0) dataFileNeededBool=false;
    if (cmp_nocase(sequenceWrite.back(),"WriteRKrig")==0 || cmp_nocase(sequenceWrite.back(),"WriteROnly")==0) {
       //if (nloci==0) problem="\nNloci setting may be needed. "; //should no longer be the case as R code deduces nloci from pointls file
       if (IBDbool) {
           // either geoBinWidth has been given explicitly or can be deduced from the settings file, or the data file is needed
            if (habitatPars.size()==5) { /*no geoBinNbr through samplesPos but from elsewhere*/
                geoInfo.fill(habitatPars,geoBinNbr); // 1D or 2D
                if (geoBinWidth==0) /*if not explicitly set by user*/ {
                   geoBinWidth=geoInfo.BinWidth; /*deduced from habitatPars and geoBinNbr*/
                } else {
                    cerr<<"(!) Very suspicious combination of settings: HabitatPars, geoBinNbr and geoBinWidth given."<<std::endl;
                    cerr<<"Consider removing either geoBinNbr or geoBinWidth. I exit."<< std::endl;
                    if (cinGetOnError) std::cin.get();
                    exit(-1);
                }
            }
            if (geoBinWidth==0) problem+="\nGeoBinWidth setting may be needed. ";
       }
       if (problem.size()==0) dataFileNeededBool=false;
    }
    if (writeAdHocFileBool) dataFileNeededBool=true;

    if ((writeAdHocFileBool || readAdHocFileBool) && multipleMarkersBool) {
		  cerr << "Ad hoc file read/write is not supported when multiple markers have been specified (MutationModel=Multiple setting)."
		  	   << "\nAborting Migraine...Press any key to exit.\n" << endl;
		  if (cinGetOnError)
			  cin.get();
		  exit(-1);
    }
    if (multipleMarkersBool && !((DemographicModel == "OnePop") || (DemographicModel == "OnePopVarSize")  || (DemographicModel == "OnePopFounderFlush"))) {
		  cerr << "Currently the multiple markers setting (MutationModel=Multiple) only works for \na single population with constant size (DemographicModel=OnePop) or"
				  "\nthe model of a panmictic population with variable population size (DemographicModel=OnePopVarSize/FounderFlush)."
		  	   << "\nAborting Migraine...Press any key to exit.\n" << endl;
		  if (cinGetOnError)
			  cin.get();
		  exit(-1);
    }

	//cout << "parameter number is " << ParameterNumber << endl;
	args.pars.resize(ParameterNumber); //initialization [`a] la d['e]claration n'['e]tait pas terrible
	lb=new float[ParameterNumber]; // delete [`a] la fin du main() de latin.
	ub=new float[ParameterNumber];
    executed=true;
    if (multipleMarkersBool && mutModelBool[ISM])
    	CanonicalNames.push_back("twoNmuSEQ");
    if (mutModelBool[GSM])
    	CanonicalNames.push_back("pGSM");
	if(!mutModelBool[SNP]) CanonicalNames.push_back("twoNmu");
     if(IBDbool) {
           CanonicalNames.push_back("twoNm");
           CanonicalNames.push_back("g");
    } else if(DemographicModel=="IM") {
           CanonicalNames.push_back("Q1");
           CanonicalNames.push_back("M1");
           CanonicalNames.push_back("M2");
           CanonicalNames.push_back("T");
           CanonicalNames.push_back("twoNancmu");
	} else if(DemographicModel=="Npop") {
		if(BoundsDefs.size()>4) {
			cerr<<"Npop model is only implemented for 2 populations and thus 4 parameters."<<endl;
			cerr<<"Your parameterValue, lowerBounds, or Upperbounds settings suggest \nthat you want to consider larger number of populations : "<<BoundsDefs.size()<<endl;
			cerr<<"Reconsider your settings. I exit."<<endl;
			if (cinGetOnError) cin.get();
			exit(-1);

		} else {
			CanonicalNames.push_back("Q1");
			CanonicalNames.push_back("M1");
			CanonicalNames.push_back("M2");
		}
	} else if(DemographicModel=="OnePopVarSize" || DemographicModel=="OnePopFounderFlush") {
            if(VarSizeFunction=="Exponential") {
              CanonicalNames.push_back("T");
              CanonicalNames.push_back("D");
              if(DemographicModel=="OnePopFounderFlush") CanonicalNames.push_back("twoNfoundermu");
              if(!mutModelBool[SNP]) CanonicalNames.push_back("twoNancmu"); else CanonicalNames.push_back("Nratio");
		   } else if(VarSizeFunction=="Logistic" && !(DemographicModel=="OnePopFounderFlush")) {
			   CanonicalNames.push_back("T");
			   CanonicalNames.push_back("twoNancmu");
			   CanonicalNames.push_back("growthRate");
              } else if(VarSizeFunction=="Discrete" && !(DemographicModel=="OnePopFounderFlush")) {
				  CanonicalNames.push_back("T");
				  CanonicalNames.push_back("twoNancmu");
			  } else {
				  if(DemographicModel=="OnePopVarSize") cerr<<"OnePopVarSize is only implemented with discrete, exponential or Logistic change in population size."<<endl;
				  if(DemographicModel=="OnePopFounderFlush") cerr<<"OnePopFounderFlush is only implemented with founding event folowed by an exponential change in population size."<<endl;
				  cerr<<"Reconsider your settings. I exit."<<endl;
				  if (cinGetOnError) cin.get();
				  exit(-1);
			  }
	 } else if(DemographicModel=="OnePop") {
			//nothing to do
	 } else { //FR->FR This test should be obsolete as the model has been checked before; (but ideally this is where the 'generic' case should be handled).
		 cerr<<"Linear or planar IBD, IM, Npop, Onepop, OnePopVarSize and OnePopFounderFlush are \ncurrently the only demographic models implemented in Migraine."<<endl;
		 cerr<<"Reconsider your settings. I exit."<<endl;
		 if (cinGetOnError) cin.get();
		 exit(-1);
	 }

    //specify parameters names when oneDimCIvArs=all, RL 062017
    if( oneDimCIvars.size()==1 && cmp_nocase(oneDimCIvars[0],"all")==0 ) {
        oneDimCIvars.resize(0,"");
        if (multipleMarkersBool && mutModelBool[ISM])
            oneDimCIvars.push_back("twoNmuSEQ");
        if (mutModelBool[GSM])
            oneDimCIvars.push_back("pGSM");
        if(!mutModelBool[SNP]) oneDimCIvars.push_back("twoNmu");
        if(IBDbool) {
            oneDimCIvars.push_back("twoNm");
            oneDimCIvars.push_back("g");
        } else if(DemographicModel=="IM") {
            oneDimCIvars.push_back("Q1");
            oneDimCIvars.push_back("M1");
            oneDimCIvars.push_back("M2");
            oneDimCIvars.push_back("T");
            oneDimCIvars.push_back("Tgmu");
            oneDimCIvars.push_back("twoNancmu");
            oneDimCIvars.push_back("Nratio");
            oneDimCIvars.push_back("m1overmu");
            oneDimCIvars.push_back("m2overmu");
            oneDimCIvars.push_back("NMratio");
            oneDimCIvars.push_back("mratio");
        } else if(DemographicModel=="Npop") {
            oneDimCIvars.push_back("Q1");
            oneDimCIvars.push_back("M1");
            oneDimCIvars.push_back("M2");
            oneDimCIvars.push_back("m1overmu");
            oneDimCIvars.push_back("m2overmu");
            oneDimCIvars.push_back("NMratio");
            oneDimCIvars.push_back("mratio");
        } else if(DemographicModel=="OnePopVarSize" || DemographicModel=="OnePopFounderFlush") {
            if(VarSizeFunction=="Exponential") {
                oneDimCIvars.push_back("T");
                oneDimCIvars.push_back("D");
                oneDimCIvars.push_back("Tgmu");
                oneDimCIvars.push_back("Dgmu");
                if(DemographicModel=="OnePopFounderFlush") oneDimCIvars.push_back("twoNfoundermu");
                if(!mutModelBool[SNP]) oneDimCIvars.push_back("twoNancmu"); else oneDimCIvars.push_back("Nratio");
            } else if(VarSizeFunction=="Logistic" && !(DemographicModel=="OnePopFounderFlush")) {
                oneDimCIvars.push_back("T");
                oneDimCIvars.push_back("Tgmu");
                oneDimCIvars.push_back("twoNancmu");
                oneDimCIvars.push_back("growthRate");
            } else if(VarSizeFunction=="Discrete" && !(DemographicModel=="OnePopFounderFlush")) {
                oneDimCIvars.push_back("T");
                oneDimCIvars.push_back("Tgmu");
                oneDimCIvars.push_back("twoNancmu");
            }
            oneDimCIvars.push_back("Nratio");
            if(DemographicModel=="OnePopFounderFlush") {
                oneDimCIvars.push_back("NactNfounderratio");
                oneDimCIvars.push_back("NfounderNancratio");
            }

        }
    }
    for (vector<string>::iterator it=extraNames.begin();it!=extraNames.end();it++) {if (cmp_nocase((*it),"nb")==0) (*it)="latt2Ns2"; }
    /** standardize BoundsDefs=GP$samplingSpace
         except that BoundDefs can contain Nb where GP$samplingSpace would contain latt2Ns2**/
    {
      int ps=ParameterNumber - ( (int) BoundsDefs.size() );//CM->RL 23.01.14
      if (ps>0) {for (int ii=0;ii<ps;ii++) BoundsDefs.push_back("");}
    }
//FR->FR voir si on ne peut utiliser la fonction StandardizeNames...
    for (vector<string>::iterator it=BoundsDefs.begin();it!=BoundsDefs.end();it++) {if (it->size()==0) (*it)="Canonical";}
    for (vector<string>::iterator it=BoundsDefs.begin();it!=BoundsDefs.end();it++) {
        if (cmp_nocase((*it),"nb")==0) {
            /* do nothing, because the user who thinks about 'Nb' does not think about latt2Ns2. makes a difference from hypercube sampling
            (*it)="latt2Ns2";
            this does assume that the user (me) may use latt2Ns2 in settings, rather than Nb...
            now the copy of BoundsDefs as GP$samplingScale in R should not contain Nb...*/
        }
    }
    for (vector<string>::iterator it=BoundsDefs.begin();it!=BoundsDefs.end();it++) {if (cmp_nocase((*it),"DispersedSigma2")==0) (*it)="condS2"; }
	if(ParameterNumber!=CanonicalNames.size()) {
		cerr << "Number of parameters deduced from Lower/upperBounds = " << ParameterNumber << endl;
		cerr << "or from Parameter Value do not fit the model (" << DemographicModel << ") parameter number=" << CanonicalNames.size() << endl;
        cerr<<"Parameters are: "; ostream_vector(CanonicalNames,cerr); cerr << endl;
		cerr<<"Reconsider your settings. I exit."<<endl;
		if(cinGetOnError) cin.get();
		exit(-1);
	}
    for(unsigned int it=0;it<ParameterNumber;it++) {if(cmp_nocase(BoundsDefs[it],"Canonical")==0) BoundsDefs[it]=CanonicalNames[it];}
    /** standardize samplingScale **/
    {
      int ps=ParameterNumber - ( (int) samplingScale.size() );//CM->RL 23.01.14
      if (ps!=0) {
          if(samplingScale.size()>0) {
              cerr << "Length of SamplingScale = " << samplingScale.size() << " does not correspond to" << endl;
              cerr << "the number of parameters deduced from Lower/upperBounds = " << ParameterNumber << endl;
              cerr<<"Reconsider your settings. I exit."<<endl;
              if(cinGetOnError) cin.get();
              exit(-1);
              //for (int ii=0;ii<ps;ii++) samplingScale.push_back(""); // RL 102013

          } else {
              for (int ii=0;ii<ps;ii++) samplingScale.push_back("");
          }
      }
    }

    for (unsigned int it=0;it<samplingScale.size();it++) { // implements the default
        if (samplingScale[it]=="") {
			//RL -> FR test on Nratio should be obsolete as sampling scale for Nratio is now specified in extraScale
            if (cmp_nocase(BoundsDefs[it],"Nratio")==0) samplingScale[it]="logscale"; // default is logscale for Nratio
            else if (cmp_nocase(BoundsDefs[it],"Nancratio")==0) {BoundsDefs[it]="Nratio";samplingScale[it]="logscale";} // default is logscale for Nratio
            else if (cmp_nocase(BoundsDefs[it],"NactNfounderratio")==0) samplingScale[it]="logscale"; // default is logscale for Nratios
            else if (cmp_nocase(BoundsDefs[it],"NfounderNancratio")==0) samplingScale[it]="logscale"; // default is logscale for Nratios
            else samplingScale[it]="Canonical";
        } // Note that this is by default copied to KrigScale below
    }
    for (vector<string>::iterator it=samplingScale.begin();it!=samplingScale.end();it++) {if (cmp_nocase((*it),"logscale")==0) (*it)="logscale"; }
    /** standardize KrigSpace **/
    if (KrigSpace.size()==0) KrigSpace=BoundsDefs; // by default will use same space for sampling and for Kriging
    {
      int ps=ParameterNumber - ( (int) KrigSpace.size() );//CM->RL 23.01.14
      if (ps>0) {for (int ii=0;ii<ps;ii++) {KrigSpace.push_back("");}}
    }
    for (vector<string>::iterator it=KrigSpace.begin();it!=KrigSpace.end();it++) {if (it->size()==0) (*it)="Canonical";}
    for (vector<string>::iterator it=KrigSpace.begin();it!=KrigSpace.end();it++) {if (cmp_nocase((*it),"nb")==0) (*it)="latt2Ns2"; }
    for (vector<string>::iterator it=KrigSpace.begin();it!=KrigSpace.end();it++) {if (cmp_nocase((*it),"DispersedSigma2")==0) (*it)="condS2"; }
    for (vector<string>::iterator it=KrigSpace.begin();it!=KrigSpace.end();it++) {if (cmp_nocase((*it),"S2cond")==0) (*it)="condS2"; }
    //real names
    for(unsigned int it=0;it<ParameterNumber;it++) {if(cmp_nocase(KrigSpace[it],"Canonical")==0) KrigSpace[it]=CanonicalNames[it];}
    /** standardize KrigScale **/
    if (KrigScale.size()==0) KrigScale=samplingScale; // by default will use same scale for sampling and for Kriging
    {
      int ps=ParameterNumber - ( (int) KrigScale.size() );//CM->RL 23.01.14
      if (ps>0) {for (int ii=0;ii<ps;ii++) KrigScale.push_back("");}
    }
    for (unsigned int it=0;it<KrigScale.size();it++) { // implements the default
        if (KrigScale[it]=="") {
            if (cmp_nocase(KrigSpace[it],"Nratio")==0) KrigScale[it]="logscale"; // default is logscale for Nratio
            else if (cmp_nocase(KrigSpace[it],"Nancratio")==0) {KrigSpace[it]="Nratio";KrigScale[it]="logscale";} // default is logscale for Nratio
            else if (cmp_nocase(KrigSpace[it],"NactNfounderratio")==0) KrigScale[it]="logscale"; // default is logscale for Nratios
            else if (cmp_nocase(KrigSpace[it],"NfounderNancratio")==0) KrigScale[it]="logscale"; // default is logscale for Nratios
            else KrigScale[it]="Canonical";
        }
    }
    for (vector<string>::iterator it=KrigScale.begin();it!=KrigScale.end();it++) {if (cmp_nocase((*it),"logscale")==0) (*it)="logscale"; }
    for (vector<string>::iterator it=KrigScale.begin();it!=KrigScale.end();it++) {if (it->size()==0) (*it)="Canonical"; }
    //
	for(unsigned int ii=0;ii<ParameterNumber;ii++) { ub[ii]=_ub[ii];lb[ii]=_lb[ii]; }
    //FR->FR mettre une detection de large uppee bound for Nb et suggérer un codage plus approprié.
    if (ParameterNumber>2 && BoundsDefs.size()>2 && cmp_nocase(BoundsDefs[2],"condS2")==0) {
        float mincondS2;
        if(D2IBDbool) mincondS2= (float) ( 1./2.); else mincondS2= (float) 1.;
        if (lb[2]<mincondS2) { // unfeasible
          if (ub[2]<=1) { // feasible, but highly suspect in context => EXIT
            cerr<<"DispersedSigma2 in samplingSpace but Lower- and UpperBoound appear to be for 'g'. Check settings. I exit.";
            if (cinGetOnError) cin.get();
            exit(-1);
          } //ELSE kindly help user...
          cerr<<"Lower bound for DispersedSigma2 should be at least "<<mincondS2<<". I shift both bounds";
          ub[2]+=(mincondS2-lb[2]);
          lb[2]=mincondS2;
       }
    }
    /** Checks appropriateness of (one dimensional only, currently) LRT values and builds a list of values to be reused by hypercube_sampling**/
    LRTmin.resize(ParameterNumber,numeric_limits<float>::quiet_NaN());
    LRTmax.resize(ParameterNumber,numeric_limits<float>::quiet_NaN());
    if(testPointList.size()>0) {

        float tmp,minv,maxv;
        string var;

        for(unsigned int jt=0;jt<BoundsDefs.size();jt++) {
            bool info=false; // whether there is some info for each variable from the testPoints or not
            var=BoundsDefs[jt];
            minv=numeric_limits<float>::max();
            maxv=numeric_limits<float>::min();
            for (std::vector<Ctestpoint>::iterator it=testPointList.begin();it!=testPointList.end();it++) {
                if ((*it).names.size()==1) {  // currently only checking through one dimensional testpoints
                    tmp=(*it).scanForVar(var); // finds whether there is a value for the variable else returns NaN.
                    if( ! std::isnan(tmp) ) {
                        minv=min(minv,tmp);
                        maxv=max(maxv,tmp);
                        info= true;
                    }
                }
            }
             if( info ) {

                 if (cmp_nocase(var,"g")==0 && minv==0) {
        	           cerr<<"(!) Cannot test "<<var<<"="<<minv<<endl;
        	           cerr<<"    Try a slightly higher, practically equivalent value such as 0.00001. I exit."<<endl;
        	            if(cinGetOnError) cin.get();
        	           exit(-1);
                }
                if (cmp_nocase(BoundsDefs[jt],"g")==0 && maxv==1) {
        	           cerr<<"(!) Cannot test "<<var<<"="<<maxv<<endl;
        	           cerr<<"    Try a slightly lower, practically equivalent value such as 0.99999. I exit."<<endl;
        	            if(cinGetOnError) cin.get();
        	           exit(-1);
                }
                if (samplingScale.size()>jt && cmp_nocase(samplingScale[jt],"LogScale")==0) {
                    if (minv<=0) {
        	           cerr<<"(!) Cannot test "<<BoundsDefs[jt]<<"="<<minv<<"with a logarithmic sampling scale for this variable"<<endl;
        	           cerr<<"    Check testPoint settings. I exit."<<endl;
        	            if(cinGetOnError) cin.get();
        	           exit(-1);
                    }
                } // but DO NOT log transform minv,maxv
                if (minv<lb[jt] && cmp_nocase(sequenceWrite[0],"Over")==0) {
                    cerr<<"(!) testPoint parameter "<<var<<"="<<minv<<" lower than lowerBound "<<lb[jt]<<" ."<<endl;
                    cerr<<"    It is unwise to test values outside of sampled parameter range. I exit.";
       	            if(cinGetOnError) cin.get();
       	           exit(-1);
                }
                if (maxv>ub[jt] && cmp_nocase(sequenceWrite[0],"Over")==0) {
                    cerr<<"(!) testPoint parameter "<<var<<"="<<maxv<<" greater than upperBound "<<ub[jt]<<" ."<<endl;
                    cerr<<"    It is unwise to test values outside of sampled parameter range. I exit.";
       	            if(cinGetOnError) cin.get();
       	           exit(-1);
                }
                /** ELSE **/
                LRTmin[jt]=minv;LRTmax[jt]=maxv;
            }
        }
    } // has filled LRTmin,LRTmax with any 'interesting' value otherwise NaN's


   	if (nloci>0) // i.e. not to be set by crunchGenepopFile, see beginning of this fn
   	   mloglikdim=int(perLocusBool)*nloci+1+int(asymptoBool);
   	// else will be set by crunchGenepopFile
   /** try to gather all booleans in one structure...**/
   if (optimizeKrigingBool) {Rarguments.push_back("optimizeKriging");} // very general for the moemnt : all plots or none for IM model
   if (extrapTrueBool) Rarguments.push_back("extrapolateOutOfHull");  //  (\"extrapolateOutOfHull\" %innc% GP$options)
   Rarguments.push_back(illConditioningMethod);
//associe SIHINT et C_c_handler; cette dern fn n'['e]tant appel['e]e qu'apr[`e]s un C-c
	if (C_c_bool) {
	   C_c_bool=false; // reset to true by a call to C_c_handler which (by the next line) occurs *when* SIGINT is thrown. Then (C_c_bool) can be used elsewhere to dontrol display
	   signal(SIGINT,C_c_handler);
	}
	if (sequenceGridSteps.size()==0) sequenceGridSteps.push_back(21); //default value
    bool found=false;
    for(auto gs : sequenceGridSteps) {
        if( gs > 25 && !found) {
            found=true;
            cout << endl << "Be Carefull, At least one GridStepNb in " << settingsfilename << " is high (i.e. > 25)" << endl;
            cout << "Profile computation times can be very long with large GridStep Values." << endl;
            cout << "Default value is 21 and usually allows nice plots," << endl;
            cout << "Higher values (e.g. 30) are mostly recommanded for a last WriteRKrig iteration on a good final run." << endl;
        }
    }
	if (profileType.size()==0) profileType="1stOrder"; //default value


    //PlotOptions.push_back("AllSurf");//RL 14102015 not used with OKsmooth, was used with Rmigraine
    /*if (IBDbool) {
     PlotOptions.push_back("3Dplots");
     } //will plot all surface plots by default*/

	bool foundAllSurfaces=false,foundNoProfile=false,foundSpecProfiles=false,foundAll1Dprofiles=false,foundAll2Dprofiles=false;
	for (vector<string>::iterator it=PlotOptions.begin();it!=PlotOptions.end();it++) {
        if (cmp_nocase((*it),"AllSurfaces")==0) foundAllSurfaces=true;
        if (cmp_nocase((*it),"NoProfiles")==0) foundNoProfile=true;
        if (cmp_nocase((*it),"All1DProfiles")==0) {foundAll1Dprofiles=true;foundNoProfile=false;} //RL: incase NoProfiles was set in the settingfile but AllProfiles or All1D or All2Dprofiles set in the command line
        if (cmp_nocase((*it),"All2DProfiles")==0) {foundAll2Dprofiles=true;foundNoProfile=false;} //RL: incase NoProfiles was set in the settingfile but AllProfiles or All1D or All2Dprofiles set in the command line
    }
    
    if( !foundAllSurfaces ) PlotOptions.push_back("NoSurfaces");

    
    //RL 14102015 for IBD only specific profiles are plotted,
    //for other demographic models, the user can choose them using 1Dprofiles and 2Dprofiles settings or default are defined here (= all2Dprofiles+Nratio+Dgmu+Tgmu except for FounderFlush & IM=NoProfiles, and OnePop=All1Dprofiles)
    if( (!foundNoProfile) && DemographicModel=="OnePop") PlotOptions.push_back("All1DProfiles");
	if ( !(foundNoProfile || spec1DProfiles.size()>0 || spec2DProfiles.size()>0) and (DemographicModel=="Npop" or DemographicModel=="IM" or DemographicModel=="OnePopVarSize" or DemographicModel=="OnePopFounderFlush")) {

        if ( !(foundAll1Dprofiles || foundAll2Dprofiles) && DemographicModel!="OnePopFounderFlush"  && DemographicModel!="IM") {
                PlotOptions.push_back("All2DProfiles");
                PlotOptions.push_back("All2DProfiles");
        }
            else if ( !(foundAll1Dprofiles || foundAll2Dprofiles) ) {
                cout << "(!) No profiles are computed by default for the OnePopFounderFlush and IM models because of high computation times." << endl;
                cout << "(!) You may use the options 1DProfiles=XXX,YYY and 2DProfiles=(XXX,YYY),(YYY,ZZZ) to select the profiles to be plotted." << endl;
                //if (cinGetOnError) cin.get();

            }
        //RL always add Nratio+Dgmu+-Tgmu 1D profile when All1Dprofiles is used
        if(DemographicModel=="IM" or DemographicModel=="OnePopVarSize" or DemographicModel=="OnePopFounderFlush") {
            if (foundAll1Dprofiles) {
                PlotOptions.push_back("PopSizeRatioProf");
                if (PopSizeScaleForTimebool){
                    int i=0;
                    for (vector<string>::iterator it=CanonicalNames.begin();it!=CanonicalNames.end();it++,i++) {
                        if (cmp_nocase((*it),"D")==0) if(abs(ub[i]-lb[i])>1e-06) {
                            PlotOptions.push_back("DgmuProf");
                        }
                    }
                    i=0;
                    for (vector<string>::iterator it=CanonicalNames.begin();it!=CanonicalNames.end();it++,i++) {
                        if (cmp_nocase((*it),"T")==0) if(abs(ub[i]-lb[i])>1e-06) {
                            PlotOptions.push_back("TgmuProf");
                        }
                    }
                }
            }
            //Nratios in logscale by default
            bool found=false;
            for (vector<string>::iterator it=extraNames.begin();it!=extraNames.end();it++) {if (cmp_nocase((*it),"Nancratio")==0) (*it)="Nratio";}
            for (vector<string>::iterator it=extraNames.begin();it!=extraNames.end();it++) {if (cmp_nocase((*it),"Nratio")==0) found=true;}
            if(!found) {
                extraNames.push_back("Nratio");
                extraScale.push_back("logscale");
            }
            for (vector<string>::iterator it=PlotOptions.begin();it!=PlotOptions.end();it++) if (cmp_nocase((*it),"DgmuProf")==0) {
                found=false;
                for (vector<string>::iterator it=extraNames.begin();it!=extraNames.end();it++) {if (cmp_nocase((*it),"Dgmu")==0) found=true;}
                if(!found) {
                    extraNames.push_back("Dgmu");
                    int i=0;
                    for (vector<string>::iterator it=CanonicalNames.begin();it!=CanonicalNames.end();it++,i++) {
                        if (cmp_nocase((*it),"D")==0) {
                            if(cmp_nocase(CanonicalNames[i],"D")==0) extraScale.push_back(samplingScale[i]);
                            else {
                                cerr << "(!) Error in extranames iterator (D)." << endl;
                                cerr << "(!) Contact the authors." << endl;
                                cerr << "(!) I exit." << endl;
                                if (cinGetOnError) cin.get();
                                exit(-1);
                            }
                        }
                    }
                }
            }
            for (vector<string>::iterator it=PlotOptions.begin();it!=PlotOptions.end();it++) if (cmp_nocase((*it),"TgmuProf")==0) {
                found=false;
                for (vector<string>::iterator it=extraNames.begin();it!=extraNames.end();it++) {if (cmp_nocase((*it),"Tgmu")==0) found=true;}
                if(!found) {
                    extraNames.push_back("Tgmu");
                    int i=0;
                    for (vector<string>::iterator it=CanonicalNames.begin();it!=CanonicalNames.end();it++,i++) {
                        if (cmp_nocase((*it),"T")==0) {
                            if(cmp_nocase(CanonicalNames[i],"T")==0) extraScale.push_back(samplingScale[i]);
                            else {
                                cerr << "(!) Error in extranames iterator (T)." << endl;
                                cerr << "(!) Contact the authors." << endl;
                                cerr << "(!) I exit." << endl;
                                if (cinGetOnError) cin.get();
                                exit(-1);
                            }
                        }
                    }
                }
            }
            if(DemographicModel=="OnePopFounderFlush") {
                found=false;
                if(!found) for (vector<string>::iterator it=extraNames.begin();it!=extraNames.end();it++) {if (cmp_nocase((*it),"NactNfounderratio")==0) found=true;}
                if(!found) {
                    extraNames.push_back("NactNfounderratio");
                    extraScale.push_back("logscale");
                }
                found=false;
                if(!found) for (vector<string>::iterator it=extraNames.begin();it!=extraNames.end();it++) {if (cmp_nocase((*it),"NfounderNancratio")==0) found=true;}
                if(!found) {
                    extraNames.push_back("NfounderNancratio");
                    extraScale.push_back("logscale");
                }
            }
        }
        
        //RL always add m1overmu, m2overmu, NMratio, mratio 1D profile when All1Dprofiles is used
        if(DemographicModel=="IM" or DemographicModel=="Npop") {
            if (foundAll1Dprofiles) {
                PlotOptions.push_back("movermuProf");
                PlotOptions.push_back("MRatioProf");
            }
            //movermu and NMratio and mratio in logscale by default
            bool found=false;
            for (vector<string>::iterator it=extraNames.begin();it!=extraNames.end();it++) {if (cmp_nocase((*it),"twoNmratio")==0 || cmp_nocase((*it),"2Nmratio")==0) (*it)="NMratio";}
            for (vector<string>::iterator it=extraNames.begin();it!=extraNames.end();it++) {if (cmp_nocase((*it),"NMratio")==0) found=true;}
            if(!found) {
                extraNames.push_back("NMratio");
                extraScale.push_back("logscale");
            }
            found=false;
            for (vector<string>::iterator it=extraNames.begin();it!=extraNames.end();it++) {if (cmp_nocase((*it),"m1m2ratio")==0) (*it)="mratio";}
            for (vector<string>::iterator it=extraNames.begin();it!=extraNames.end();it++) {if (cmp_nocase((*it),"mratio")==0) found=true;}
            if(!found) {
                extraNames.push_back("mratio");
                extraScale.push_back("logscale");
            }
            for (vector<string>::iterator it=PlotOptions.begin();it!=PlotOptions.end();it++) if (cmp_nocase((*it),"movermuProf")==0) {
                found=false;
                for (vector<string>::iterator it=extraNames.begin();it!=extraNames.end();it++) {if (cmp_nocase((*it),"m1overmu")==0) found=true;}
                if(!found) {
                    extraNames.push_back("m1overmu");
                    int i=0;
                    for (vector<string>::iterator it=CanonicalNames.begin();it!=CanonicalNames.end();it++,i++) {
                        if (cmp_nocase((*it),"M1")==0) {
                            if(cmp_nocase(CanonicalNames[i],"M1")==0) extraScale.push_back(samplingScale[i]);
                            else {
                                cerr << "(!) Error in extranames iterator (M1)." << endl;
                                cerr << "(!) Contact the authors." << endl;
                                cerr << "(!) I exit." << endl;
                                if (cinGetOnError) cin.get();
                                exit(-1);
                            }
                        }
                    }
                }
                found=false;
                for (vector<string>::iterator it=extraNames.begin();it!=extraNames.end();it++) {if (cmp_nocase((*it),"m2overmu")==0) found=true;}
                if(!found) {
                    extraNames.push_back("m2overmu");
                    int i=0;
                    for (vector<string>::iterator it=CanonicalNames.begin();it!=CanonicalNames.end();it++,i++) {
                        if (cmp_nocase((*it),"M2")==0) {
                            if(cmp_nocase(CanonicalNames[i],"M2")==0) extraScale.push_back(samplingScale[i]);
                            else {
                                cerr << "(!) Error in extranames iterator (M2)." << endl;
                                cerr << "(!) Contact the authors." << endl;
                                cerr << "(!) I exit." << endl;
                                if (cinGetOnError) cin.get();
                                exit(-1);
                            }
                        }
                    }
                }
            }
        }
			
         
	}
    
    if (foundAll1Dprofiles && spec1DProfiles.size()>0) {
        cerr << "(!) Plots=All1DProfiles and specific parameters are specified for 1D profiles using the option 1DProfiles=XXX,YYY." << endl;
        cerr << "(!) Modify one of these two options." << endl;
        cerr << "(!) I exit." << endl;
        if (cinGetOnError) cin.get();
        exit(-1);
    }
    if (foundAll2Dprofiles && spec2DProfiles.size()>0) {
        cerr << "(!) Plots=All2DProfiles and specific parameter pairs are specified for 2D profiles using the option 2DProfiles=(XXX,YYY),(YYY,ZZZ)." << endl;
        cerr << "(!) Modify one of these two options.." << endl;
        cerr << "(!) I exit." << endl;
        if (cinGetOnError) cin.get();
        exit(-1);
    }
    if (!PopSizeScaleForTimebool)
        for (vector<string>::iterator it=spec1DProfiles.begin();it!=spec1DProfiles.end();it++)
            if ( (cmp_nocase((*it),"Dgmu")==0) || (cmp_nocase((*it),"Tgmu")==0) ) {
                cerr << "(!) Impossible to consider Timescale=mutationrate and specific 1Dprofiles containing Dgmu or Tgmu simultaneously." << endl;
                cerr << "(!) Modify one of these two options.." << endl;
                cerr << "(!) I exit." << endl;
                if (cinGetOnError) cin.get();
                exit(-1);

    }
    if (foundNoProfile && spec1DProfiles.size()>0) {
        cout << "Plots=noProfiles and specific parameters are specified for 1D profiles using the option 1DProfiles=XXX,YYY." << endl;
        cout << "I will use specific 1D profiles." << endl;
    }
    if (foundNoProfile && spec2DProfiles.size()>0) {
        cout << "Plots=noProfiles and specific parameter pairs are specified for 2D profiles using the option 2DProfiles=(XXX,YYY),(YYY,ZZZ)." << endl;
        cout << "I will use the specific 2D profiles." << endl;
    }
	if(outlierGeneFileBool && promptHyperSegSites) {
		cerr << "(!) Migraine cannot output an outlier gene analysis file when \"PromptForHyperSegSites=true\" has been set." << endl;
        cerr << "(!) I exit." << endl;
		if (cinGetOnError) cin.get();
		exit(-1);
	}

	/** NOW forward info to ****Rparams**** (here should comme all info that cannot be modified over iterations) **/
	// CBR modified 15.02.2013
    Rparams.fill(0); //(0) for iteration 0... not that Rparams.fill() is called for each further iteration
	//private members of Rparams but apply_final_settings has been declared as friend
	// next lines unelegant but handy and safe
	Rparams.graphicsFormat=graphicsFormat;
	Rparams.hullExpandFactor=hullExpandFactor;
	Rparams.profileType=profileType;
	Rparams.DemographicModel=DemographicModel;
#ifdef DEBUG
    cout << "\n !!!!!!!!!!  Run in DEBUG mode !!!!!!!!!!!" << endl;
    if (cinGetOnError || pauseFinal) cin.get();
#endif
    

//if(cinGetOnError) { // no error occurred, but the test being true means that the session is interactive
#ifdef WIN32

      Sleep(4000);

#else
      sleep(4);//so that the user can check all messages

#endif

      // but not cin.get, no exit since no error

//}
return 0;
} //end apply_final_settings

int COptions::Ocollect() {
//ost est ostringstream membre de COptions
// Oappend pour ligne complete chaine constante. Sinon ost+Oappend+ost
//==================================
Oappend("\n=============================================");
//ost<<"This is \033[34;01m Migraine \033[39;49;00m version "<<version<<"."; // essai.. marchepas sous DOS au moins...
ost<<"This is Migraine version "<<version<<".";
Oappend(ost.str());
ost.str("");
Oappend("Likelihood analysis of genetic data");
Oappend("based on the de Iorio & Griffiths algorithms.");
Oappend("=============================================");
#ifdef WIN32
Oappend("Local operating system is Windows");
#else
Oappend("Local operating system is Unix");
#endif
#ifdef __APPLE__ // && __MACH__
Oappend("on a MacOs computer.");
#endif
ost<<"with "<<coreNbr<<" core allocated for R analyses.";
Oappend(ost.str());
ost.str("");
Oappend("=============================================");
Oappend("Statistical model:");
if(DemographicModel=="Npop" || DemographicModel=="IM") Oappend("n populations model with arbitrary migration;");
if(DemographicModel=="IM") Oappend("with ancestral splits (Isolation with Migration);");
if(DemographicModel=="Npop") Oappend("constant model in time (No splits)");

if(D1IBDbool) {
  if(linabsbool)
	Oappend("Linear lattice with absorbing boundaries.");
  else
	Oappend("Circular lattice.");
}
if(D2IBDbool) {
  if(linabsbool)
	Oappend("Planar lattice with absorbing boundaries.");
  else
	Oappend("Toroidal lattice.");
}
if(IBDbool) {
	Oappend("Geometric dispersal model;");
}
if (DemographicModel=="OnePopVarSize") {
	if(VarSizeFunction=="Discrete") Oappend("Single population model with discrete population size change;");
	else if(VarSizeFunction=="Exponential") Oappend("Single population model with exponential population size change;");
	else if(VarSizeFunction=="Logistic") Oappend("Single population model with logistic population size change;");
} else if (DemographicModel=="OnePopFounderFlush") {
	Oappend("Single population model with a discrete population size \n change (e.g. Founder event) followed by an exponential population size change;");
} else if (DemographicModel=="OnePop") Oappend("Single population model with constant population size;");

if (multipleMarkersBool)
	Oappend("Multiple mutational models have been specified:");
if(mutModelBool[PIM]) {
	Oappend("Symmetric K-alleles mutation model");
#ifdef PIMDIG
	Oappend("   (! theta parameter includes k->k mutations).");
#endif
#ifdef PIMtrue
	Oappend("   (theta parameter excludes k->k mutations).");
#endif
}
if(mutModelBool[SMM])	Oappend("Stepwise mutation model");
if(mutModelBool[GSM])	Oappend("Generalised stepwise mutation model");
if(mutModelBool[ISM])	Oappend("Infinite sites mutation model");
if(mutModelBool[SNP])	Oappend("Single Nucleotide Polymorphisms considered");
Oappend("=============================================");
Oappend("Algorithm:");
if (sequenceStat.size()==1) {
     if (cmp_nocase(sequenceStat[0],"PAC")==0) {
	    Oappend("PAC-likelihood (Product of Approximate Conditional Likelihoods).");
    } else if (cmp_nocase(sequenceStat[0],"PAC2id")==0) {
	    Oappend("'PAC2id': PAC-likelihood (Product of Approximate Conditional Likelihoods), ");
        Oappend("and likelihood for pairs of genes.");
    } else if (cmp_nocase(sequenceStat[0],"PACanc2id")==0) {
	    Oappend("'PACanc2id': Importance Sampling for the actual population(s)");
		Oappend("and PAC-likelihood (Product of Approximate Conditional Likelihoods) for the ancestral population(s).");
        Oappend("and analytical likelihood for pairs of genes.");
    } else if (cmp_nocase(sequenceStat[0],"PACanc")==0) {
        Oappend("'PACanc': Importance Sampling for the actual population(s)");
        Oappend("and PAC-likelihood (Product of Approximate Conditional Likelihoods) for the ancestral population(s).");
	}  else if (HobolthISM) {
		Oappend("Importance Sampling.");
		Oappend("Using the Hobolth proposal (see Hobolth et al., 2008) for the ISM/SNP mutation models;");
	}
	else
		Oappend("Importance Sampling.");
}
if(AlternativeIS_AncPopSizebool) {
	Oappend("Alternative Importance Sampling distribution based on ancestral population size will be used in the actual population.");
}
if(AlternativeIS_FRExpMixAncActPopSizebool) {
	Oappend("Alternative Importance Sampling distribution \nbased on mixed Pi's considering actual and \nancestral population size will be used.");
}
if (sequenceStat.size()>1) {
	ost<<"Sequence: ";
   	ostream_vector(sequenceStat,ost);
	Oappend(ost.str());
	ost.str("");
}
if (asymptoBool) {
	Oappend("Approximation of remote ancestral process by scaled WF coalescent:");
	Oappend("    Fst computed in large N, low dispersal limit");
	}
#ifdef ASYMPTO
	Oappend("Approximation of remote ancestral process by scaled WF coalescent:");
	Oappend("    Fst computed in large N, low dispersal limit");
#endif
#ifdef ASYMPTO_PAR_TAUX_COAL
	Oappend("  & criterion: convergence of coalescence times");
#endif
#ifdef ASYMPTO_PAR_FST
	ost<<"  & criterion: convergence of Fst at distance !!!!!2!!!!!";
	Oappend(ost.str());
	ost.str("");
#endif
#ifdef ASYMPTO_PAR_CPU
	Oappend("  & criterion: CPU time per sampled tree");
#endif
#ifdef ASYMPTO_PAR_EVENTS
	ost<<"  & criterion: "<<max_nbr_events<<" events occurred";
	Oappend(ost.str());
	ost.str("");
#endif
#ifdef ASYMPTO_PAR_DISP_EVENTS
	ost<<"  & criterion: 2n/sigma [1D !] dispersal events occurred";
	Oappend(ost.str());
	ost.str("");
#endif
if (AsymptoPerDispRateBool) {
	ost<<"  & criterion: generation = "<<disp_rate_control_param<<"*(#demes)/sigma2.";
	Oappend(ost.str());
	ost.str("");
	}
#ifdef ASYMPTO_PAR_DISP_RATE
	ost<<"  & criterion: generation = "<<disp_rate_control_param<<"*(#demes)/sigma2.";
	Oappend(ost.str());
	ost.str("");
#endif
if (AsymptoPerMutRateBool) {
	ost<<"  & criterion: generation = "<<mut_rate_control_param<<"/(2u).";
	Oappend(ost.str());
	ost.str("");
	}
#ifdef ASYMPTO_PAR_MUT_RATE
	ost<<"  & criterion: generation = "<<mut_rate_control_param<<"/(2u).";
	Oappend(ost.str());
	ost.str("");
#endif
Oappend("=============================================");
Oappend("Sample:");
	if (genepopRootBool) ost<< "Genepop root file name: " <<gp_fileRoot<<endl;
	else if (genepopBool) ost<< "Genepop file name: " <<gp_file<<endl;
	else ost<< "Root file name (ad hoc format): " <<samplerootname<<endl;
	if (genepopBool) { ost<<" (Write/Read ad hoc files? Write: "<<writeAdHocFileBool<<"; Read: "<<readAdHocFileBool<<")"<<endl;}
	if (genepopRootBool  && dataFileNeededBool) ost <<"Number of loci as given by first Genepop file (more info below)";
	else if (genepopBool  && dataFileNeededBool) ost <<"Number of loci as given by Genepop file (more info below)";
	else if (dataFileNeededBool) { //sinon affichage debile
        ost<< nloci <<" loci per sample";
        Oappend(ost.str());
        ost.str("");
        ost<<"Ad hoc files to be used # "<<(JobMin-1)*nloci+1<<" to "<<(JobMax+1)*nloci;
    }
	if (nexusRootBool)
		ost << "\nNexus root file name (for sequence data): " << nexusRootFilename << endl;
	else if (nexusBool)
		ost << "\nNexus file name (for sequence data): " << nexusFilename << endl;
	Oappend(ost.str());
	ost.str("");
Oappend("=============================================");
if(DemographicModel=="Npop") Oappend("Parameter (Mutation rate, relative population sizes and dispersal rates) space explored:");
if(DemographicModel=="IM") Oappend("Parameter (Mutation rate, relative population sizes, dispersal rates and divergence times) space explored:");
if (DemographicModel=="OnePopVarSize") {
	if (multipleMarkersBool) {
		if (mutModelBool[GSM] && mutModelBool[ISM]) {
			if(VarSizeFunction=="Exponential")
				Oappend("Six Parameters (Current Theta(SEQ),pGSM,Current Theta(MIC),PopSize change time, duration and ancestral theta(MIC)) explored:");
			else if(VarSizeFunction=="Logistic")
				Oappend("Six Parameters (Current Theta(SEQ),pGSM,Current Theta(MIC),PopSize change time, ancestral theta(MIC) and Logistic growth rate) explored:");
			else
				Oappend("Five Parameters (Current Theta(SEQ),pGSM,Current Theta(MIC),PopSize change time, ancestral theta(MIC)) explored:");
		}
		else if (mutModelBool[GSM]) {
			if(VarSizeFunction=="Exponential")
				Oappend("Five Parameters (pGSM,Current Theta,PopSize change time, duration and ancestral theta) explored:");
			else if(VarSizeFunction=="Logistic")
				Oappend("Five Parameters (pGSM,Current Theta,PopSize change time, ancestral theta and Logistic growth rate) explored:");
			else
				Oappend("Four Parameters (pGSM,Current Theta,PopSize change time, ancestral theta) explored:");
		}
		else if (mutModelBool[ISM]) {
			if(VarSizeFunction=="Exponential")
				Oappend("Five Parameters (Current Theta(SEQ),Current Theta(MIC),PopSize change time, duration and ancestral theta(MIC)) explored:");
			else if(VarSizeFunction=="Logistic")
				Oappend("Five Parameters (Current Theta(SEQ),Current Theta(MIC),PopSize change time, ancestral theta(MIC) and Logistic growth rate) explored:");
			else
				Oappend("Four Parameters (Current Theta(SEQ),Current Theta(MIC),PopSize change time, ancestral theta(MIC)) explored:");
		}
	}
	else {
		if (mutModelBool[GSM]) {
			if(VarSizeFunction=="Exponential")
				Oappend("Five Parameters (pGSM,Current Theta,PopSize change time, duration and ancestral theta) explored:");
			else if(VarSizeFunction=="Logistic")
				Oappend("Five Parameters (pGSM,Current Theta,PopSize change time, ancestral theta and Logistic growth rate) explored:");
			else
				Oappend("Four Parameters (pGSM,Current Theta,PopSize change time, ancestral theta) explored:");
		}
		else if(mutModelBool[SNP]) {
			if(VarSizeFunction=="Exponential")
				Oappend("Three Parameters (PopSize change time, duration and 1/Nratio=Nanc/Nact) explored:");
			else if(VarSizeFunction=="Logistic")
				Oappend("Four Parameters (PopSize change time, 1/Nratio=Nanc/Nact and Logistic growth rate) explored:");
			else
				Oappend("Two Parameters (PopSize change time, 1/Nratio=Nanc/Nact) explored:");
		} else {
			if(VarSizeFunction=="Exponential")
				Oappend("Four Parameters (Current Theta,PopSize change time, duration and ancestral theta) explored:");
			else if(VarSizeFunction=="Logistic")
				Oappend("Four Parameters (Current Theta,PopSize change time, ancestral theta and Logistic growth rate) explored:");
			else
				Oappend("Three Parameters (Current Theta,PopSize change time, ancestral theta) explored:");
		}

	}
}
if (DemographicModel=="OnePopFounderFlush") {
    if (multipleMarkersBool) {
        if (mutModelBool[GSM] && mutModelBool[ISM]) Oappend("Seven Parameters (Current Theta(SEQ),pGSM,Current Theta(MIC),PopSize change time, duration, founder theta(MIC) and ancestral theta(MIC)) explored:");
         else if (mutModelBool[GSM]) Oappend("Six Parameters (pGSM,Current Theta,PopSize change time, duration,founder theta and ancestral theta) explored:");
         else if (mutModelBool[ISM]) Oappend("Six Parameters (Current Theta(SEQ),Current Theta(MIC),PopSize change time, duration, founder theta(MIC) and ancestral theta(MIC)) explored:");
    }
    else {
        if (mutModelBool[GSM]) Oappend("Six Parameters (pGSM,Current Theta,PopSize change time, duration, founder theta and ancestral theta) explored:");
        else Oappend("Five Parameters (Current Theta,PopSize change time, duration, founder theta and ancestral theta) explored:");
    }
}

if (DemographicModel=="OnePop") {
	if (multipleMarkersBool) {
		if (mutModelBool[GSM] && mutModelBool[ISM])
			Oappend("Three Parameters (Theta(SEQ),pGSM,Theta(MIC)) explored:");
		else if (mutModelBool[GSM])
			Oappend("Two Parameters (pGSM,Theta) explored:");
		else if (mutModelBool[ISM])
			Oappend("Two Parameters (Theta(SEQ),Theta(MIC)) explored:");
	}
	else {
		if (mutModelBool[GSM])
			Oappend("Two Parameters (pGSM,Theta) explored:");
		else
			Oappend("One Parameter (Theta) explored:");
	}
}
if(IBDbool) Oappend("Parameter (Mutation rate, relative population sizes and dispersal rates) space explored:");
#ifdef LOCALSCALING
	if(DemographicModel=="Npop") Oappend("Mutation rates scaled by total population size \nDispersal rates scaled by deme size");
	if(DemographicModel=="IM") Oappend("Mutation rates and divergence times scaled by total population size \nDispersal rates scaled by deme size");
	if(IBDbool) Oappend("Mutation and dispersal rates scaled by deme size");
#endif
#ifdef GLOBALSCALING
	cerr << "Be carrefull, GLOBAL SCALING is not used anymore, I exist." << endl;
	if (cinGetOnError) cin.get();
	exit(-1);

	if(DemographicModel=="Npop") Oappend("Mutation rates and dispersal rates scaled by total population size");
	if(DemographicModel=="IM") Oappend("Mutation rates, dispersal rates and divergence time scaled by total population size");
	if(IBDbool) Oappend("Mutation and dispersal rates scaled by total population size");
#endif
	ost<<"*** (Initial) parameter range ***";
//#ifdef LOCALSCALING
if(DemographicModel=="Npop") ost<<"\n (2*Ntoth*u, N1h/Ntoth, 2*N1h*m12, 2*N2h*m21)";
if(DemographicModel=="IM") ost<<"\n (2*Ntoth*u, N1h/Ntoth, 2*N1h*m12, 2*N2h*m21,T,2*Nh_ancestral*u)";
if(IBDbool) {
	ost<<" (2 Nh u";
    if (BoundsDefs.size()>1 && cmp_nocase(BoundsDefs[1],"Nb")==0)
       ost<<", *Nb*";
    else ost<<", 2 Nh m";
    if (BoundsDefs.size()>2 && cmp_nocase(BoundsDefs[2],"condS2")==0)
       ost<<", *DispersedSigma2*";
    else ost<<", g";
   }
if (DemographicModel=="OnePop") {
	if (multipleMarkersBool) {
		if (mutModelBool[GSM] && mutModelBool[ISM])
			ost<<"\n2*Nh*u(SEQ),pGSM,2*Nh*u";
		else if (mutModelBool[GSM])
			ost<<"\npGSM,2*Nh*u";
		else if (mutModelBool[ISM])
			ost<<"\n2*Nh*u(SEQ),2*Nh*u";
	}
	else {
		if (mutModelBool[GSM])
			ost<<"\npGSM,2*Nh*u";
		else
			ost<<"\n2*Nh*u";
	}
}
if (DemographicModel=="OnePopVarSize") {
	if (multipleMarkersBool) {
		if (mutModelBool[GSM] && mutModelBool[ISM]) {
			if(VarSizeFunction=="Exponential")
				ost<<"\n2*Nh_current*u(SEQ),pGSM,2*Nh_current*u,T,D,2*Nh_ancestral*u";
			else if(VarSizeFunction=="Logistic")
				ost<<"\n2*Nh_current*u(SEQ),pGSM,2*Nh_current*u,T,2*Nh_ancestral*u,GrowthRate";
			else
				ost<<"\n2*Nh_current*u(SEQ),pGSM,2*Nh_present*u,T,2*Nh_ancestral*u";
		}
		else if (mutModelBool[GSM]) {
			if(VarSizeFunction=="Exponential")
				ost<<"\npGSM,2*Nh_current*u,T,D,2*Nh_ancestral*u";
			else if(VarSizeFunction=="Logistic")
				ost<<"\n,pGSM,2*Nh_current*u,T,2*Nh_ancestral*u,GrowthRate";
			else
				ost<<"\n,pGSM,2*Nh_present*u,T,2*Nh_ancestral*u";
		}
		else if (mutModelBool[ISM]) {
			if(VarSizeFunction=="Exponential")
				ost<<"\n2*Nh_current*u(SEQ),2*Nh_current*u,T,D,2*Nh_ancestral*u";
			else if(VarSizeFunction=="Logistic")
				ost<<"\n2*Nh_current*u(SEQ),2*Nh_current*u,T,2*Nh_ancestral*u,GrowthRate";
			else
				ost<<"\n2*Nh_current*u(SEQ),2*Nh_present*u,T,2*Nh_ancestral*u";
		}
	}
	else {
		if (mutModelBool[GSM]) {
			if(VarSizeFunction=="Exponential")
				ost<<"\n pGSM,2*Nh_current*u,T,D,2*Nh_ancestral*u";
			else if(VarSizeFunction=="Logistic")
				ost<<"\n pGSM,2*Nh_current*u,T,2*Nh_ancestral*u,GrowthRate";
			else
				ost<<"\n pGSM,2*Nh_present*u,T,2*Nh_ancestral*u";
		}
		else if (mutModelBool[SNP]) {
			if(VarSizeFunction=="Exponential")
				ost<<"\n T,D,Nh_ancestral/Nh_current";
			else if(VarSizeFunction=="Logistic")
				ost<<"\n T,Nh_ancestral/Nh_current,GrowthRate";
			else
				ost<<"\n T,Nh_ancestral/Nh_current";
		} else {
			if(VarSizeFunction=="Exponential")
				ost<<"\n2*Nh_current*u,T,D,2*Nh_ancestral*u";
			else if(VarSizeFunction=="Logistic")
				ost<<"\n2*Nh_current*u,T,2*Nh_ancestral*u,GrowthRate";
			else
				ost<<"\n2*Nh_present*u,T,2*Nh_ancestral*u";
		}
	}
} // end if OnePopVarSize
if (DemographicModel=="OnePopFounderFlush") {
    if (multipleMarkersBool) {
        if (mutModelBool[GSM] && mutModelBool[ISM]) ost<<"\n2*Nh_current*u(SEQ),pGSM,2*Nh_current*u,T,D,2*Nh_founder*u,2*Nh_ancestral*u";
         else if (mutModelBool[GSM]) ost<<"\npGSM,2*Nh_current*u,T,D,2*Nh_founder*u,2*Nh_ancestral*u";
          else if (mutModelBool[ISM]) ost<<"\n2*Nh_current*u(SEQ),2*Nh_current*u,T,D,2*Nh_founder*u,2*Nh_ancestral*u";
    } else {
        if (mutModelBool[GSM]) ost<<"\n,pGSM,2*Nh_current*u,T,D,2*Nh_founder*u,2*Nh_ancestral*u";
         else ost<<"\n2*Nh_current*u,T,D,2*Nh_founder*u,2*Nh_ancestral*u";
    }
} // end if onepopfounderflush

//#endif
//#ifdef GLOBALSCALING
//	ost<<"(2 Nh nd u, 2 Nh nd m...)";
//#endif
	Oappend(ost.str());
	ost.str("");
	ost<<"lower bound: ";for(unsigned int ii=0;ii<ParameterNumber;ii++) ost<<lb[ii]<<" ";
	Oappend(ost.str());
	ost.str("");
	ost<<"upper bound: ";for(unsigned int ii=0;ii<ParameterNumber;ii++) ost<<ub[ii]<<" ";
	Oappend(ost.str());
	ost.str("");
Oappend("=============================================");
Oappend("Numerical options:");
if(cmp_nocase(PiSolverFromSettingsInfo,"PCG_island")==0)
        Oappend("Pi computation by iterative algorithm (PCG) initialized by island model solution;");
else if (cmp_nocase(PiSolverFromSettingsInfo,"PCG_penta")==0)
        Oappend("Pi computation by iterative algorithm (PCG) initialized by pentadiagonal matrix solution;");
else if (cmp_nocase(PiSolverFromSettingsInfo,"SymDirect")==0)
        Oappend("Pi computation by direct solver;");
else if(cmp_nocase(PiSolverFromSettingsInfo,"SMM2popDGLR05")==0)
        Oappend("Pi computation by Fourier transform for 2 pops and SMM (de Iorio et al. 2005);");
else if (cmp_nocase(PiSolverFromSettingsInfo,"PiPim")==0)
        Oappend("Pi computation by PiPim (see De Iorio & Griffiths 2004b);");
else if (cmp_nocase(PiSolverFromSettingsInfo,"PiTwoPop")==0)
    Oappend("Pi computation by PiTwoPop (see De Iorio & Griffiths 2004b);");
else if (IBDbool) {
            Oappend("Pi computation by iterative algorithm (PCG) if >=60 demes;");
        }
else {
	if (mutModelBool[ISM] || mutModelBool[SNP]) {
		if (!HobolthISM)
			Oappend("Pi computation (see De Iorio & Griffiths 2004b) for the ISM/SNP mutation models;");
	}

	if (mutModelBool[SMM] || mutModelBool[GSM] || mutModelBool[PIM]) {
		if(piOnePopSolverBool[SD]==true)
			Oappend("For single pop, Pi computation by S&D algorithm \nusing specific SMM or GSM mutation matrices (PiOnePop);");
		else if(piOnePopSolverBool[Eigen]==true)
			Oappend("For single pop, Pi computation by Leblois & Rousset algorithm using eigenvalues \ncomputed on specific SMM or GSM mutation matrices (PiOnePop);");
		else
			Oappend("Pi computation by D&G algorithm using specific solutions for PIM & SMM mutation models (PiOnePop);");
	}
}

	if (singlePointBool) {
		ost<<"Single point computed.";
		Oappend(ost.str());
		ost.str("");
	}
	if (noRfileBool) {
		ost<<"no R batch file.";
		Oappend(ost.str());
		ost.str("");
	}
	if (sequenceWrite[0]=="WriteROnly") {
		ost<<"Stops after writing the R batch file; no genetic data analysis.";
		Oappend(ost.str());
		ost.str("");
	}
	if (noRCallBool && !noRfileBool) {
		ost<<"No Rcall (but R batch file written).";
		Oappend(ost.str());
		ost.str("");
	}
	if ( ! noRCallBool) {
		ost<<"Migraine computation and kriging (once).";
		Oappend(ost.str());
		ost.str("");
	}
	if (sequenceWrite.size()>1) {
		ost.str("");
	}
Oappend("=============================================");
	ost<<"Kriging: ";
    	Oappend(ost.str());
	    ost.str("");
    if (use_Rpackage) {
        ost<<"The blackbox R package will be used.";
    	Oappend(ost.str());
	    ost.str("");
    }
	ost<<krigingCov<<" covariance family;";
    	Oappend(ost.str());
	    ost.str("");
    if (optimizeKrigingBool) {
    	ost<<"         scale parameter(s) estimated by generalized cross-validation (GCV);";
    	Oappend(ost.str());
	    ost.str("");
    } else {
        ost<<"         Values of scale parameter(s) provided by the user (NOT RECOMMENDED);";
    	Oappend(ost.str());
	    ost.str("");
    }
   if (maxSmoothness-minSmoothness>sqrt(numeric_limits<long double>::epsilon())) { //krigxytype in CKrigcoefs is long double
        ost<<"         Matern smoothness parameter estimated by GCV";
        Oappend(ost.str());
        ost.str("");
    } else {
        ost<<"         Matern smoothness parameter fixed to "<<minSmoothness;
        Oappend(ost.str());
        ost.str("");
    }
Oappend("=============================================");
Oappend("Likelihood ratio test:");
	bool found=false;
	for (vector<string>::iterator it=PlotOptions.begin();it!=PlotOptions.end();it++) {if (cmp_nocase((*it),"NoProfiles")==0) found=true;}
    for (vector<string>::iterator it=PlotOptions.begin();it!=PlotOptions.end();it++) {
        if (cmp_nocase((*it),"All1DProfiles")==0 or cmp_nocase((*it),"All2DProfiles")==0) found=false;
    }
	if(!found) {
		if (cmp_nocase(profileType,"modified")==0) {
			Oappend("Modified profile LRT using empirical approximation of sample-space derivatives");
		} else {
			Oappend("Standard likelihood ratio profiles & LRT");
		}
	}
Oappend("=============================================");

return(0);
}

int COptions::Oinspect(ostream& st) {
    for(vector<string>::iterator ii=liste.begin();ii!=liste.end();ii++) {
	    st<<(*ii)<<endl;
	}
   st<<pointmax-pointmin+1<<" parameter points (first iteration)";
   if (pointmax==pointmin) st<<" (point "<<pointmin<<")";
   st<<endl;
   for(unsigned int ii=0;ii<sequenceNrunsPerPoint.size()-1;ii++) st<<sequenceNrunsPerPoint[ii]<<",";
   st<<sequenceNrunsPerPoint.back()<<" sampled tree(s) per locus "<<endl;
//cout<<"Maximum integer: "<<numeric_limits<int>::max(); //2 10^9...
   st<<JobMax-JobMin+1<<" job(s) to run (multilocus sample(s) to analyse) on " << MaxSimultJobs << " processor(s)"<<endl;
   st<<endl;
return 0;
}

//implementation =======================================

Arguments::Arguments() {
   pars.resize(ParameterNumber);
   SampleFileName="NO SAMPLE FILE NAME";
   locus=0; // indicator that no valid locus has been specified
};

Cparvals::Cparvals() {pars.resize(ParameterNumber);};

Cparvals::Cparvals(Cparvals *parva) {
pars.resize(parva->pars.size()); //hmmm
copy(parva->pars.begin(), parva->pars.end(), pars.begin());
};

Cparvals::Cparvals(vector<float> v) {
pars.resize(v.size()); //hmmm
copy(v.begin(),v.end(), pars.begin());
};

Cparvals origin() {
Cparvals lin;
lin.pars.resize(ParameterNumber);
fill(lin.pars.begin(),lin.pars.end(),0);
return lin;}

vector<float> externalfn(const vector<double> pars,const int *mlsp) {
	vector<float> tlik(int(asymptoBool)+1,0);
	vector<float> summloglik(int(asymptoBool)+1,0);
	vector<float> return_vector(0);
	CMigMod* MigMod1=new CMigMod(); //doit etre recr['e]['e] [`a] chaque nouvelle s['e]rie de loci
//cout<<ParameterNumber<<" "<<pars.size()<<" "<<args.pars.size();getchar();
    if (calculQused) calculQ.newPars(true); // indicator for recomputing eigensystem in calclogL
	copy(pars.begin(),pars.end(),args.pars.begin());
	// ne pas oublier d'avoir sp['e]cifi['e] args.nruns qqpart...
//	summloglik=0.0;
    SkipLoci.resize(nloci+1,false);
    for(int nl=1;nl<nloci+1;nl++)
        for(size_t i=0;i<TempSkipLoci.size();i++) {
            if(TempSkipLoci[i]==nl) SkipLoci[nl]=true;

        }
    for(int nl=1;nl<nloci+1;nl++){
        if(!SkipLoci[nl]) {
        {
            stringstream stst(stringstream::in | stringstream::out);
                stst<<samplerootname<<nloci*(*mlsp-1)+nl;
                stst>>args.SampleFileName; //only ad hoc name
                args.locus=nl; //from 1
        }
#ifdef DEUBG
        if(PrintLikelihood) cout << "\n\nLocus=" << nl << endl;
#endif
        if (multipleMarkersBool) { // RL -> CBR, FR : je déplacerai tout ca dans calclog
        	mutationModel = mutationModelVector[nl - 1];
        	if ((DemographicModel.substr(0,6)=="OnePop" || DemographicModel=="IM") && ( (mutationModel == "SMM") || (mutationModel == "GSM") ) ) {
                if(piOnePopSolverVector[nl-1]=="SD") {piOnePopSDbool=true;piOnePopEigenbool=false;}
                    else if(piOnePopSolverVector[nl-1]=="Eigen") {piOnePopSDbool=false;piOnePopEigenbool=true;}
                        else {piOnePopSDbool=false;piOnePopEigenbool=false;}//DGRL2005 + PP
        	}
        } else {
        	mutationModel = mutationModelVector[0];
        	if ((DemographicModel.substr(0,6)=="OnePop" || DemographicModel=="IM") && ( (mutationModel == "SMM") || (mutationModel == "GSM") ) ) {
                if(piOnePopSolverVector[0]=="SD") {piOnePopSDbool=true;piOnePopEigenbool=false;}
                 else if(piOnePopSolverVector[0]=="Eigen") {piOnePopSDbool=false;piOnePopEigenbool=true;}
                  else {piOnePopSDbool=false;piOnePopEigenbool=false;}//DGRL2005 + PP
        	}
        }

        // RL added 02072013 for markers with different number of allelic states (different GivenK), otherwise IBDtable does not have the good size...
        //cout << "sequenceGivenK.size()=" << sequenceGivenK.size() << "; calculQ.newPars()=" << calculQ.newPars() << endl;
        if (sequenceGivenK.size()>0 && calculQused) {
            calculQ.newPars(true); // indicator for recomputing eigensystem in calclogL
        }
    /********************* here the computation of the likelihood ***************/
		tlik=calclogl(args,MigMod1);
   /*****************************************************************************/
#ifdef DEBUG
        if(PrintLikelihood) {
            cout << "\n*************************************************************\n" << endl;
            cout << "Run finished for Locus=" << nl << endl;
            cout << "LnLikelihood is " << endl; ostream_vector(tlik,cout);
            cout << "*************************************************************\n" << endl;
        }
#endif
		if(std::isnan(tlik.front())||std::isnan(tlik.back())){
//            if (!(IBDbool & pars[2]<0)) { // FER 31/03/2008 gnegBool
                cerr<<"\n(!) From externalfn: the estimated likelihood tlik at locus "<<nl<<" is not a number"<<endl;
                cerr<<"    Parameter point was: "<<endl;
                ostream_vector(pars,cout);
				cerr<<"\nIf the problem persists, report it to the authors. Press any key resume..." << endl;
                if (cinGetOnError) cin.get();
				//exit(-1);
//            }
        } else {
			if (perLocusBool) return_vector.push_back(tlik[0]); // stores only the first statistic for each locus
			{vector<float>::iterator aa=tlik.begin();
				for(vector<float>::iterator ii=summloglik.begin();ii!=summloglik.end();ii++,aa++)
					(*ii)+=(*aa);
			} //bloc aa local

		}
        }//end if(!skipLoci[nl])
	} // boucle sur loci
    for(vector<float>::iterator ii=summloglik.begin();ii!=summloglik.end();ii++)
		return_vector.push_back(*ii);
    delete MigMod1;
    return(return_vector);
} // end externalfn()
